;	[]===========================================================[]
;
;	NOTICE: THIS PROGRAM BELONGS TO AWARD SOFTWARE INTERNATIONAL(R)
;	        INC. IT IS CONSIDERED A TRADE SECRET AND IS NOT TO BE 	
;	        DIVULGED OR USED BY PARTIES WHO HAVE NOT RECEIVED	
;	        WRITTEN AUTHORIZATION FROM THE OWNER.
;
; 	[]===========================================================[]
;

;=============================================================================
; FILE: DMI_FUNC.INC
;
; DESC: DMI function support.  Please refer to the DMI BIOS Specification
;       Version 2.0 for complete information on each of these functions.
;
;
; BY:         Tim Markey
; DATE:       12/20/95
; NOTE:       This code is written for 5.10 MASM so that it can be used 
;             by both Award PowerBIOS and EliteBIOS with no modifications.
;
;=============================================================================
;Rev  Date     Author Description
;--------------------------------------------------------------------------
;R32  12/10/97 STV    Added SMBIOS V2.1 Table Entry Point(ifdef SMBIOS_v21_Support)
;R31  10/22/97 KVN    Added DMI pool checksum byte to avoid system hang at verify
;		      DMI data if DMI pool data not correct
;R30  09/10/97 RCH    Return supporting GPNV(Func. 55H) cause Symbios 64Kb
;		      SCSI/BIOS hang up during initialization	
;R29  04/08/97 KVN    Fix some utility (e.g. SMDMI.EXE (USI offer)) selector
;		      set error so we use CS to get DMI structure
;R28  03/22/97 KVN    Store DMI data to F000 shadow to fix Intel LDCM bug
;R27  02/14/97 KVN    Add function 53h-57h for LDCM utility
;R19A 12/11/96 KVN    public dmi_shadow_info label for DMIPOST.ASM
;R26  08/30/96 KVN    Don't check MAX_STRUCTURE_SIZE for each DMI structure
;R25  08/29/96 KVN    Don't care flash rom brand for SET_DMI_STRUCT (DMI function 52h)
;R24  08/21/96 KVN    Added 2MB flash rom DMI function support
;R23  08/15/96 KVN    Added update DMI error code if flash write is fail
;R22  08/14/96 RCH    Replace all branch instruction with short branch to
;		      to save codes		
;R21  08/12/96 KVN    Fixed coding error
;R20  08/12/96 KVN    If none any DMI structure then reture error status in
;		      func 51h	
;R19  07/12/96 TJM    Add dmi_shadow_info to this code to reserve for shadowing
;R18  07/11/96 TJM    Add SDS_SUCCESS common exit jmp
;R17  07/11/96 TJM    Update Shadow RAM with max size and num structs in func 52h
;R16  07/02/96 KVN    Get component total length before delete it
;R15  07/10/96 TJM    Add option to use BIOSSelector to get DMI information
;R14  07/02/96 TJM    Add control bit to force flash updates even if no change
;R13  07/02/96 TJM    Compare new value with old, if same don't flash return error
;R12B 02/07/96  KVN    Pack DMI function become a module and put it in
;                       0F000h-0F800h
;R12  07/01/96 TJM    Change block size to match flash
;R11  07/01/96 TJM    Add new control definitions for fast POST update
;R10  07/01/96 TJM    No optimization for Intel parameter block support
;R09  06/27/96 TJM    Change GET_DMI_INFORMATION to return words for num structs and max size
;R08  06/26/96 TJM    Use DMI selector in GET_DMI_STRUCTURE
;R07  06/25/96 TJM    Changed GET_DMI_INFORMATION to not read flash
;R06  06/04/96 TJM    Change method of determining number of bytes to change
;R05  06/04/96 TJM    Changed Pool terminator to 6 bytes to cover handle
;R04  06/01/96 TJM    Fixed problem of updating structure number for first handle
;R03  05/15/96 TJM    Change change byte/word/dword to write back only dword
;R02  05/07/96 TJM    Beta release version 1
;R01  01/03/96 TJM    Alpha release for ASUS Demo
;=============================================================================

;R12B start
.386P
    INCLUDE BIOS.CFG
    INCLUDE COMMON.EQU
    INCLUDE PNP.EQU
    include dmi.equ
ifdef   DMI_ENABLED
    extrn  F000_Shadow_W:Near    ;R03
    extrn  F000_Shadow_R:Near    ;R03
	extrn  Flash_Write:near	;R31
DGROUP    GROUP  FCODE
;R28FCODE    SEGMENT USE16 DWORD PUBLIC 'CODE'
FCODE    SEGMENT USE16 PARA PUBLIC 'CODE'	;R28
    ASSUME  CS:DGROUP,DS:NOTHING
;R12B end

ifndef MASM611
  extrn  Get_CMOS:near
  extrn  Flash_Read:near
  extrn  Flash_Write:near
endif

;R28 start
		align	16
		public	DMI_BIOS_Struct_Table
DMI_BIOS_Struct_Table:
;R32 start
ifdef SMBIOS_V21_Support
		db	'_SM_'			;header
		db	0			;checksum of SMBIOS entry point structure
		db	1eh			;length of the Entry Point Structure
		db	02h
		db	01h
		dw	0
		db	0
		db	5 dup (?)
endif	;SMBIOS_V21_Support
;R32 end
		db	'_DMI_'			;header
		db	0			;checksum of DMI BIOS entry point structure
		dw	DMI_MAX_POOL_SIZE	;total length of DMI BIOS structure table
		dw	offset Shadow_DMI_Address	;32-bit physical address (offset)
		dw	0fh			;segment
		dw	0			;Total number of structures in the DMI BIOS structure table
;R32		db	20h			;Revision of the DMI BIOS extensions
;R32 start
ifdef SMBIOS_V21_Support
		db	21h			;Revision of the SMBIOS extensions
else	;SMBIOS_V21_Support
		db	20h			;Revision of the DMI BIOS extensions
endif	;SMBIOS_V21_Support
;R32 end
;R28 end

Public  BEGIN_DMI_RUNTIME           ; Used to track overall size of
BEGIN_DMI_RUNTIME LABEL BYTE           ;  DMI Post 

; R19 - Begin 
;=============================================================================
; Reserve space to copy DMI information into shadow area
;=============================================================================
		public	dmi_shadow_info	;R19A
dmi_shadow_info label byte
dmi_shadow_signature       dd 00000000h ; DMI Shadow of signature
dmi_shadow_struct_num      dw 0000h     ; DMI Shadow of number of structs
dmi_shadow_struct_max_size dw 0000h     ; DMI Shadow of max struct size
; R19 - End

;=============================================================================
; FUNC: GET_DMI_INFORMATION (Function 50h - Get DMI Information)
;
; DESC: Returns number of DMI structures, largest DMI structure size,
;       the DMI BIOS revision, the DMI storage base, and storage size
;       used to hold DMI structures (this last one was proposed by me
;       to the DMI BIOS committee and hopefully will be approved).
;
; SYNOPSIS:
;
; short FAR (*entryPoint) (Function, DMIBIOSRev, NumStructures, \
;   StructureSize, BiosSelector);
;
; short Function;                   - PnP BIOS Function 50h
; unsigned char FAR *DMIBIOSRev     - Revision of the DMI BIOS Extension
; unsigned short FAR *NumStructures - Maximum num of structs BIOS will ret
; unsigned short FAR *StructureSize - Size of largest DMI Structure
; unsigned long FAR *DMIStorageBase - 32-bit phys addr of mem-mapped DMI data
; unsigned long FAR *DMIStorageSize - Size of block to store DMI/other info
; unsigned short BiosSelector       - PnP BIOS read/writable selector
;
; IN:   NONE
; OUT:  AX  - Error Code (0 if successful)
;=============================================================================
    public  get_dmi_information  ;R12B
get_dmi_information proc  near
  gdi_func_num      equ [ebp+FN_OFS]     ; PnP BIOS Function 50h
  gdi_dmi_bios_rev  equ [ebp+FN_OFS+02h] ; Revision of DMI BIOS Extension
  gdi_num_dmi       equ [ebp+FN_OFS+06h] ; Max num of structs BIOS will ret
  gdi_dmi_size      equ [ebp+FN_OFS+0Ah] ; Size of largest DMI Structure
  gdi_storage_base  equ [ebp+FN_OFS+0Eh] ; 32-bit phys addr of mem-mapped DMI data
  gdi_storage_size  equ [ebp+FN_OFS+12h] ; Size of block to store DMI/other info
  gdi_bios_selector equ [ebp+FN_OFS+16h] ; PnP BIOS read/writable selector

;R22  mov   al,gdi_func_num             ; send function number to port 80h
;R22  out   80h,al
;R22
;R22  mov   al,gdi_func_num             ; send function number to port 80h
;R22  out   80h,al

  ; R17 - start
  mov   es,gdi_bios_selector
  mov   di,offset dmi_shadow_info
  cmp   es:[di],DMI_SIGNATURE
  je    short GDI_SIGNATURE_VALID
;R22  mov   eax,00000000h
  xor	eax,eax				;zero out EAX
  jmp   short GDI_GET_INFO

GDI_SIGNATURE_VALID:
  add   di,4                        ; skip signature
  mov   eax,es:[di]                 ; get max size and num structures

GDI_GET_INFO:  
  lds   bx,gdi_num_dmi
  mov   word ptr [bx],ax            ;
  lds   bx,gdi_dmi_size             ; Size of largest DMI Structure
  shr   eax,16
  mov   word ptr [bx],ax            ; Maximum number of structures
  ; R17 - End

  lds   bx,gdi_dmi_bios_rev
  mov   byte ptr [bx],DMI_BIOS_REV       ; DMI BIOS Rev
  lds   bx,gdi_storage_base
  mov   dword ptr [bx],DMI_STORAGE_BASE  ; DMI storage base
  lds   bx,gdi_storage_size
  mov   word ptr [bx],DMI_STORAGE_SIZE   ; physical block size

  mov   ax,DMI_SUCCESS

  ret

get_dmi_information endp


;=============================================================================
; FUNC: GET_DMI_STRUCT (Function 51h - Get DMI Structure)
;
; DESC: This function will copy the information for the specified
;       DMI Strucuture into the buffer specified by the caller.
;
; SYNOPSIS:
;
; short FAR (*entryPoint) (Function, Structure, devStructBuffer, \
;   BiosSelector)
;
; short Function                    - PnP BIOS Function 51h
; unsigned short FAR *Structure     - Struct number/handle to retrieve
; unsigned char FAR *dmiStrucBuffer - Buffer to copy device struc data to
; unsigned short BiosSelector       - PnP BIOS readable/writable selector
;
; IN:   NONE
; OUT:  AX  - Error Code (0 if successful)
;=============================================================================
    public  get_dmi_struct  ;R12B
get_dmi_struct proc  near
  gds_func_num       equ [ebp+FN_OFS]     ; PnP BIOS Function 51h
  gds_structure      equ [ebp+FN_OFS+02h] ; Struct number/handle to retrieve
  gds_dmi_struct_buf equ [ebp+FN_OFS+06h] ; Buffer to copy device struc data to
  gds_dmi_selector   equ [ebp+FN_OFS+0Ah] ; Selector for DMI struct pool
  gds_bios_selector  equ [ebp+FN_OFS+0Ch] ; PnP BIOS read/writable selector

;R22  mov   al,gds_func_num             ; send function number to port 80h
;R22  out   80h,al

  cld

;R28 start
	push	esi
ifdef ESCD_M2
;R29	mov	ax,gds_bios_selector
	mov	ax,cs		;R29
	mov	ds,ax
	lea	si,dmi_shadow_info
	cmp	dword ptr ds:[si],DMI_SIGNATURE           ; check for valid pool
	jne	GDS_INVALID_HANDLE    ; return error
	add	si,4
else ;ESCD_M2
;R28 end
;R29  mov   ax,gds_dmi_selector         ; R09 - use DMI selector
  mov   ax,cs			;R29
  mov   ds,ax                       ; get seg/selector of sys dmi tbl
  mov   si,DMI_OFFSET               ; get offset of DMI Structure Tbl

  les   di,gds_dmi_struct_buf       ; let check_dmi_signature use buffer
  call  check_dmi_signature         ; check DMI signature
;R28  jc    short GDS_INVALID_HANDLE    ; return error
  jc    GDS_INVALID_HANDLE    ;R28 return error
endif ;ESCD_M2				;R28

  les   di,gds_structure            ; ES:DI = pointer to structure handle
  mov   bx,word ptr es:[di]         ; load BX with dmi struct handle number

ifdef ESCD_M2				;R28
	mov	cx,ds:[si]		;R28
else ;ESCD_M2				;R28
  les   di,gds_dmi_struct_buf       ; get seg/selector of buffer
  mov   ax,4                              ; read in size & max
  call  read_dmi

ifdef MASM611
  assume di: ptr dmi_info_struct
endif
  mov   cx,es:[di].num_structs
endif ;ESCD_M2					;R28

  or   cx,cx			;R20
;R28  jz   short GDS_INVALID_HANDLE	;R20 if none any DMI structure then skip
  jz   GDS_INVALID_HANDLE	;R28 if none any DMI structure then skip
  cmp  cx,MAX_STRUCTS
;R28  jge  short GDS_INVALID_HANDLE
  jge  GDS_INVALID_HANDLE	;R28

;R28 start
ifdef ESCD_M2
	mov	si,Shadow_DMI_Len_Address
	shl	esi,16
	mov	si,Shadow_DMI_Address
else ;ESCD_M2
;R28 end
  add   si, size dmi_info_struct
endif ;ESCD_M2					;R28
  cmp   bx,DMI_TERMINATOR           ; if struct handle is out of range
;R28  jz    short GDS_INVALID_HANDLE
  jz    GDS_INVALID_HANDLE	;R28

;R28  les   di,gds_dmi_struct_buf

GDS_CHECK_NEXT_STRUCT:
;R26  call  check_max_struct_size       ; verify that num structs less than max
;R26  jge   short GDS_INVALID_HANDLE    ; AX = current maximum
;R28 start
	les   di,gds_dmi_struct_buf
ifdef ESCD_M2
	rol	esi,16
	mov	ax,ds:[si]		;get DMI structure length of single component
	add	si,2
	rol	esi,16
	mov	dx,ds:[si].dmi_handle
	push	cx
	mov	cx,ax
	rep	movsb
	pop	cx
else ;ESCD_M2
;R28 end
  mov   ax,2                        	;R26
  call  read_dmi			;R26
  mov   ax,word ptr es:[di]		;R26
  add   si,2
  call  read_dmi
  mov   dx,es:[di].dmi_handle       ; get handle of current DMI
endif ;ESCD_M2					;R28
  cmp   dx,DMI_TERMINATOR           ; check for terminator
  je    short GDS_INVALID_HANDLE

  or    bx,bx                       ; if handle == 0 then return first handle
  jz    short GDS_GET_NEXT_STRUCTURE_HANDLE ; R04

  cmp   dx,bx                       ; correct handle found
  je    short GDS_GET_NEXT_STRUCTURE_HANDLE
ifndef ESCD_M2				;R28
  add   si,ax                       ; advance to next DMI structure
endif ;ESCD_M2				;R28
  loop  short GDS_CHECK_NEXT_STRUCT

  mov   ax,DMI_INVALID_HANDLE		;R28
  jmp	short GDS_EXIT			;R28

GDS_GET_NEXT_STRUCTURE_HANDLE:

ifdef ESCD_M2				;R28
	mov   dx,DMI_TERMINATOR		;R28
	cmp	cx,1			;R28
	je	short @F		;R28
	mov	dx,ds:[si+2]		;R28
@@:					;R28
else ;ESCD_M2				;R28
  mov   cx,ax                       ; get size of current DMI struct
  push  si
  add   si,cx                       ; advance to next DMI structure
  add   si,2                        ; skip size
  add   si,offset dmi_struct_hdr.dmi_handle
  mov   ax,2
  call  read_dmi
  mov   dx,word ptr es:[di]         ; get handle of next DMI struct
  pop   si

  mov   ax,cx
  call  read_dmi                    ; re-read in structure
endif ;ESCD_M2				;R28
;R28LDCM start
	push	dx
	smsw	bx
	test	bl,1				;if in real mode?
	jz	short No_any_string		;Yes,skip LDCM progress
	les	di,gds_dmi_struct_buf
	push	es
	pop	ds
	movzx	bx,byte ptr es:[di+1]		;get component length
	add	di,bx
	sub	ax,bx
	jz	short No_any_string
	xor	dl,dl
Scan_Next_Loop:
	cmp	byte ptr es:[di],0
	jne	short Scan_Next_Byte1
	or	dl,dl
	jnz	short Scan_Next_Byte
	push	di
	mov	si,di
	inc	di
	mov	cx,ax
	rep	movsb
	pop	di
	mov	byte ptr es:[di],' '
	inc	di
Scan_Next_Byte:
	xor	dl,dl
	jmp	short @F
Scan_Next_Byte1:
	or	dl,1
@@:
	inc	di
	dec	ax
	cmp	ax,1				;if ending?
	jne	short Scan_Next_Loop		;no,continue.
No_any_string:
	pop	dx
;R28LDCM end

  mov   ax,DMI_SUCCESS              ; handle found and is being returned
  jmp   short GDS_LOAD_HANDLE

GDS_INVALID_HANDLE:
  mov   ax,DMI_INVALID_HANDLE
  mov   dx,DMI_TERMINATOR

GDS_LOAD_HANDLE:
  les   di,gds_structure            ; load ES:DI with dmi structure handle
  mov   word ptr es:[di],dx         ; update dmi handle

GDS_EXIT:
	pop	esi			;R28

  ret

get_dmi_struct endp

;=============================================================================
; FUNC: SET_DMI_STRUCT (Function 52h - Set DMI Structure)
;
; DESC: This function will copy the information for the specified
;       DMI Strucuture into the buffer specified by the caller.
;
;       Internally, this function will read the 4K parameter block
;       into the dmiWorkBuffer provided by the caller.  The changes
;       to the DMI pool will be made to dmiWorkBuffer and then this
;       buffer will be written back out to the 4K paramter block.
;
; SYNOPSIS:
;
; short FAR (*entryPoint) (Function, dmiDataBuffer, dmiWorkBuffer, \
;         Control, DMISelector, BiosSelector);
;
; short Function                    - Function 52h
; unsigned char FAR* dmiDataBuffer  - Pntr to new/changed data
; unsigned char FAR* dmiWorkBuffer  - Pntr to dmi work buffer area
; unsigned char Control             - Conditions for setting struct
;   
; R11  07/01/96   TJM     Add new control definitions for fast POST update
; Control Definitions:
;   
; Bit 0 - 0 = Validate Parameters, do not flash in ESCD
; Bit 1 - 1 = Do not read in ESCD, use ESCD presently in Work Buffer
; Bit 2 - 1 = Force Flash update of DMI Pool even if no change (For POST)
;
; unsigned short DMI Selector       - Selector for DMI struct pool
; unsigned short BiosSelector       - BIOS read/writable selector
;
; IN:   NONE
; OUT:  AX  - Error Code (0 if successful)
;=============================================================================
    public  set_dmi_struct  ;R12B
set_dmi_struct proc near
  sds_func_num       equ [ebp+FN_OFS]     ; Function 52h
  sds_dmi_data_buf   equ [ebp+FN_OFS+02h] ; Pntr to new/changed data
  sds_dmi_work_buf   equ [ebp+FN_OFS+06h] ; Pntr to dmi work buffer area
  sds_control_flag   equ [ebp+FN_OFS+0Ah] ; Conditions for setting struct
  sds_dmi_selector   equ [ebp+FN_OFS+0Ch] ; Selector for DMI struct pool
  sds_bios_selector  equ [ebp+FN_OFS+0Eh] ; BIOS read/writeable selector

;R22  mov   al,sds_func_num             ; send function number to port 80h
;R22  out   80h,al

  pushf                             ; save flags
  cld                               ; clear direction
  cli                               ; disable interupts during operation

  ; R11 - Change validation code
  ; test word ptr sds_control_flag,SET_DMI_POOL
  ; jnz  sds_set_structure
  ; R11
  ; jmp  SDS_EXIT                    ; parameters are correct

sds_set_structure:

;R28  mov   ax,sds_dmi_selector         ; load selector of DMI pool in AX
;R28  mov   ds,ax                       ; get seg/selector of DMI pool
;R28  mov   si,PARAM_BLOCK_BASE         ; start at beginning of parameter block

  les   di,sds_dmi_work_buf         ; load ES:DI with work buffer
  mov   ax,DMI_STORAGE_SIZE         ; R12 - change block size to match flash

  test  word ptr sds_control_flag,DONT_READ_IN_POOL ; R11
  jnz   short sds_no_read1

;R28 start
ifdef ESCD_M2
	smsw	dx
	test	dl,1				;if in real mode?
	jz	short Read_Flash_DMI		;Yes,read from flash
;R29	mov	ax,sds_bios_selector
	mov	ax,cs		;R29
	mov	ds,ax
	lea	si,dmi_shadow_info
	cmp	dword ptr ds:[si],DMI_SIGNATURE           ; check for valid pool
	jne	short DMI_shadow_invalid
	push	di
	mov	dx,ds:[si+4]			;get DMI shadow structure number
	mov	cx,8
	rep	movsb
	mov	si,Shadow_DMI_Len_Address
	shl	esi,16
	mov	si,Shadow_DMI_Address
Get_DMI_shadow_data_Loop:
	rol	esi,16
	mov	cx,2
	rep	movsb
	mov	cx,ds:[si-2]			;get component size
	rol	esi,16
	rep	movsb
	dec	dx
	jnz	short Get_DMI_shadow_data_Loop
	pop	di
	jmp	short sds_no_read1
Read_Flash_DMI:
endif ;ESCD_M2
;R29	mov	si,sds_dmi_selector         ; load selector of DMI pool in AX
	mov	si,cs		;R29
	mov	ds,si                       ; get seg/selector of DMI pool
	mov	si,PARAM_BLOCK_BASE         ; start at beginning of parameter block
;R28 end

  call  read_dmi

sds_no_read1:

  add   di,DMI_TBL_OFFSET           ;

  mov   eax,es:[di]
  cmp   eax,DMI_SIGNATURE
  je    short SDS_SIGNATURE_VALID
DMI_shadow_invalid:			;R28

  mov   dword ptr es:[di],DMI_SIGNATURE ; add signature
  add   di,DMI_SIGNATURE_SIZE       
ifdef MASM611
  assume di:ptr dmi_info_struct
endif
  xor   cx,cx
  mov   es:[di].num_structs,cx      ; set num structs to zero
  mov   es:[di].max_struct_size,cx  ; set max structs to zero
  jmp   short SDS_NUM_STRUCTS_OK

SDS_SIGNATURE_VALID:
ifdef MASM611
  assume di:ptr dmi_info_struct
endif
  add   di,DMI_SIGNATURE_SIZE       
  mov   cx,es:[di].num_structs      ; get number of structures in DMI TBL
  cmp   cx,MAX_STRUCTS              ; check to see if pool initialized yet
  jl    short SDS_NUM_STRUCTS_OK
  xor   cx,cx                       ; pool is not initialized, set to zero
  mov   es:[di].num_structs,cx      ; set num structs to zero
  mov   es:[di].max_struct_size,cx  ; set max structs to zero

SDS_NUM_STRUCTS_OK:
  add   di,size dmi_info_struct     ; skip table information
  lds   si,sds_dmi_data_buf         ; read into DS:SI dmiDataBuffer

ifdef MASM611
  assume si:ptr dmi_set_struct
endif
  cmp   [si].command,DMI_RESERVED_CMD ; if command is change and no structs
  jge   SDS_BAD_PARAMETER 	      ; bad command passed in

  or    cx,cx                       ; check to see if structures present
  jnz   short SDS_1

ifdef MASM611
  assume si:ptr dmi_set_struct
endif
  cmp   [si].command,DMI_CHANGE_DWORD ; if command is change and no structs
  jle   SDS_INVALID_HANDLE	      ;  exist then exit w/ error

SDS_1:
  mov   bx,[si].struc_handle        ; get current structure handle
  cmp   bx,DMI_TERMINATOR           ; compare with terminator
  je    SDS_INVALID_HANDLE          ; if equal to terminator then exit w/ error

  call  get_structure               ; get the DMI structure into ES:DI
  jnc   short SDS_STRUCT_FOUND      ; DMI structure found

  cmp   [si].command,DMI_ADD_STRUCT ; check for Add DMI structure command
  je    short SDS_ADD_STRUCT        ; check if there is room to add
  jmp   SDS_INVALID_HANDLE          ; structure not found

SDS_STRUCT_FOUND:
  cmp   [si].command,DMI_CHANGE_DWORD ; check for Change DMI structure command
  jle   SDS_CHANGE_STRUCT           ; change structure byte, word or dword
  cmp   [si].command,DMI_ADD_STRUCT ; check for Add DMI structure command
  je    SDS_INVALID_HANDLE          ; structure already exists
  cmp   [si].command,DMI_DEL_STRUCT ; check for Delete DMI structure command
  je    short SDS_DELETE_STRUCT     ; delete structure
  cmp   [si].command,DMI_CHANGE_STRING ; check for Change DMI string
  je    SDS_CHANGE_STRING           ; change string

;=============================================================================
; ADD NEW DMI STRUCTURE
;=============================================================================
SDS_ADD_STRUCT:

SDS_CHECK_FOR_ROOM:
  mov   cx,di                       ; check for room
  push  di
  les   di,sds_dmi_work_buf         ; load ES:DI with work buffer
  add   di,DMI_TBL_OFFSET+DMI_SIGNATURE_SIZE       
  sub   cx,di
  pop   di
  add   cx,[si].data_length         ; add full length of struct
  cmp   cx,DMI_MAX_POOL_SIZE
  jb    short SDS_ADD_NEW_DMI_STRUCT
  mov   ax,DMI_ADD_STRUCTURE_FAILED ; pool full
  jmp   SDS_EXIT

SDS_ADD_NEW_DMI_STRUCT:
  mov   cx,[si].data_length         ; CX = full length of struct
  push  cx                          ; save struct size
  mov   word ptr es:[di],cx
  add   di,2
  add   si,offset dmi_set_struct.struc_type
  cld
  rep   movsb                       ; add new dmi structure to end
  mov   ax,DMI_TERMINATOR           ; AX = TERMINATOR
  stosw                             ; write terminator to size field
  stosw                             ; R05 - Write terminator to type, len
  stosw                             ; R05 - Write terminator to handle
  pop   cx                          ; restore struct size
  les   di,sds_dmi_work_buf         ; load ES:DI with work buffer
  add   di,DMI_TBL_OFFSET+DMI_SIGNATURE_SIZE       
ifdef MASM611
  assume  di:ptr dmi_info_struct
endif

  cmp   es:[di].max_struct_size,0FFFFh  ; compare new struct w/ max
  je    short SDS_SET_NEW_MAX_STRUCT

  cmp   cx,es:[di].max_struct_size  ; compare new struct w/ max
  jbe   short SDS_INCREMENT_NUM_STRUCTS

SDS_SET_NEW_MAX_STRUCT:
  mov   es:[di].max_struct_size,cx  ; new max struct

SDS_INCREMENT_NUM_STRUCTS:
;R24  mov   cx,es:[di].num_structs      ; get number of structures in DMI TBL
;R24  inc   cx                          ; increment number of structures
;R24  mov   es:[di].num_structs,cx      ; set number of structures in DMI TBL
	inc   word ptr es:[di].num_structs      ;R24
  jmp   SDS_WRITE_TO_FLASH

;=============================================================================
; DELETE EXISTING DMI STRUCTURE
; IN:  AX    - Length of current struct
;      ES:DI - points to structure to delete
;      CX    - number of structures left in DMI pool
;=============================================================================
SDS_DELETE_STRUCT:                  ; delete structure

  mov   dx,di                       ; save current string start location in DX
  sub   dx,2                        ; backup to length
  mov    ax,es:[di-2]                ; get component size in AX - R16
  add   ax,2                        ; account for length
  les   di,sds_dmi_work_buf         ; load ES:DI with work buffer
  add   di,DMI_TBL_OFFSET+DMI_SIGNATURE_SIZE       
  mov   cx,ax
  call  move_everything_up          ; delete structure
  jmp   SDS_WRITE_TO_FLASH          ; write back to flash

;=============================================================================
; CHANGE DMI STRUCTRE BYTE, WORD, OR DWORD
; IN:  AX    - Length of current struct
;      ES:DI - points to structure to change
;      CX    - number of structures left in DMI pool
;=============================================================================
SDS_CHANGE_STRUCT:

ifdef MASM611
  assume  si:ptr dmi_set_struct
endif
  mov   bl,[si].field_ofs           ; save field offset BL

ifdef MASM611
  assume  di:ptr dmi_struct_hdr
endif

  mov   al,[si].command             ; get command in AL
  mov   cl,al                       ; use this for the shift value in CL
  mov   dl,1
  shl   dl,cl                       ; DL contains number of bytes to change (R06)
  mov   cl,es:[di].dmi_len          ; AL = struct length

SDS_CHECK_RANGE:

  add   bl,dl
  cmp   bl,cl                       ; check to see if offset out of range
;R21  jge   SDS_BAD_PARAMETER           ; attempting to change structure
  ja    SDS_BAD_PARAMETER            ;R21 attempting to change structure
                                    ; which is out of range

  movzx bx,[si].field_ofs           ; load field offset into BX
  add   di,bx                       ; add field offset to DI
  mov   ecx,[si].change_mask        ; clear dword
  mov   ebx,[si].change_value       ; set dword

  cmp   al,DMI_CHANGE_BYTE          ; check for change byte command
  jne   short SDS_CHECK_FOR_CHANGE_WORD

  ; R13 - Compare new value with old, if same don't flash return error
  mov   al,byte ptr es:[di]         ; get the old value               R13
  and   byte ptr es:[di],cl         ; clear byte
  or    byte ptr es:[di],bl         ; set byte
  mov   ah,byte ptr es:[di]         ; get the new value               R13
  cmp   ah,al                       ; compare the old and new values  R13
  je    short SDS_NO_CHANGE         ; Signal no change in value       R13
  jmp   short SDS_WRITE_CHANGED_VALUE     ; just write out new BYTE

SDS_CHECK_FOR_CHANGE_WORD:
  cmp   al,DMI_CHANGE_WORD          ; check for change word command
  jne   short SDS_CHANGE_DWORD
  mov   ax,word ptr es:[di]         ; get the old value           R13
  and   word ptr es:[di],cx         ; clear word
  or    word ptr es:[di],bx         ; set word
  mov   bx,word ptr es:[di]         ; Get new value               R13
  cmp   ax,bx                       ; Compare old and new value   R13
  je    short SDS_NO_CHANGE         ; Signal no change in value   R13
  jmp   short SDS_WRITE_CHANGED_VALUE     ; just write out new WORD

SDS_CHANGE_DWORD:
  mov   eax,dword ptr es:[di]       ; get the old value           R13
  and   es:[di],ecx                 ; clear dword
  or    es:[di],ebx                 ; set dword
  mov   ebx,dword ptr es:[di]       ; Get new value               R13
  cmp   eax,ebx                     ; Compare old and new value   R13
  je    short SDS_NO_CHANGE               ; Signal no change in value   R13
  jmp   short SDS_WRITE_CHANGED_VALUE     ; just write out new DWORD

SDS_NO_CHANGE:                      ; Signal no change in value   R13

  test  word ptr sds_control_flag,FORCE_UPDATE ; Test for force update R14
  jnz   short SDS_WRITE_CHANGED_VALUE     ; just write out new DWORD         R14
  mov   ax,DMI_NO_CHANGE            ; Signal old value same as new     R13
  jmp   SDS_EXIT                    ; exit

; R03 - start
SDS_WRITE_CHANGED_VALUE:

  

;R25 ifdef INTEL_28F001BXT

  jmp   SDS_WRITE_TO_FLASH          ; Don't optimize for Intel parts R10

;R25;R24 elseif SST_PH29EE010
;R25else	;SST_PH29EE010			;R24
;R25
;R25  ;========================================================================
;R25  ; Try to write minimum amount of bytes to flash as possible
;R25  ; since writing to flash is time consuming.  Be careful of updates
;R25  ; which straddle across the granularity
;R25  ;========================================================================
;R25  lds   si,sds_dmi_work_buf         ; load DS:SI with work buffer
;R25  sub   di,si                       ; get the offset
;R25  mov   dx,di
;R25
;R25  mov   di,sds_dmi_selector
;R25  mov   es,di                       ; ES:DI = get seg/selector of DMI pool
;R25  mov   di,PARAM_BLOCK_BASE         ; start at beginning of parameter block
;R25  add   di,dx                       ; and add offset to DI
;R25
;R25  ; check to see if changed value is across two granules in the case of a
;R25  ; change word or change dword command
;R25
;R25  mov   cx,FLASH_GRANULARITY        ; CX = minimum update size
;R25
;R25  dec   cx                          ; 
;R25  and   dx,cx                       ; 
;R25  sub   cx,dx                       
;R25  cmp   cx,01h          
;R25  jae   short one_granual_only            ; only 1 GRANUAL required
;R25
;R25  cmp   al,DMI_CHANGE_BYTE          ; check for change word command
;R25  je    short one_granual_only            ; only 1 GRANUAL required
;R25
;R25  mov   cx,FLASH_GRANULARITY * 2    ; CX = minimum update size
;R25
;R25  jmp   short sds_write_change
;R25
;R25one_granual_only:
;R25  mov   cx,FLASH_GRANULARITY        ; CX = minimum update size
;R25
;R25sds_write_change:
;R25
;R25  push  cx                          ; save amount to write
;R25  dec   cx                          ;
;R25  xor   cx,0FFFFh                   ;
;R25  and   di,cx                       ; align to granularity 
;R25  mov   cx,di                       ;
;R25  sub   cx,PARAM_BLOCK_BASE         ; start at beginning of parameter block
;R25  add   si,cx                       ; amount to add to work buffer
;R25  pop   cx                          ; restore amount to write
;R25
;R25  test  word ptr sds_control_flag,SET_DMI_POOL   ; R11
;R25  jz    short sds_no_flash1                      ; R11
;R25
;R25  call  Flash_Write                 ; write to flash
;R25
;R25sds_no_flash1:                                   ; R11
;R25
;R25  jmp   SDS_SUCCESS                 ; Add exit for success R18
;R25
;R25;R24 else
;R25
;R25;R24  jmp   SDS_WRITE_TO_FLASH    ; R10 - Don't optimize for Intel parts
;R25
;R25endif

; R03 - end


;=============================================================================
; CHANGE DMI STRUCTURE STRING
; IN:  AX    - Length of current struct
;      CX    - number of structures left in DMI pool
;      DS:SI - points to new/change data information
;      ES:DI - points to structure to change
;=============================================================================
SDS_CHANGE_STRING:
ifdef MASM611
  assume  si:ptr dmi_set_struct
endif
  
  movzx bx,[si].field_ofs           ; move field offset into BX
  push  di                          ; save DI

ifdef MASM611
  assume  di:ptr dmi_struct_hdr
endif

  movzx ax,byte ptr es:[di].dmi_len ; get offset to start of string info

ifdef MASM611
  assume  di:ptr nothing
endif

  add   di,bx                       ; add field offset to DI to point to string
  movzx cx,byte ptr es:[di]         ; get string number
  pop   di                          ; restore DI to start of structure

  mov   dx,di                       ; save start of structure
  shl   edx,16                      ; save DI in upper word of EDX
  add   di,ax                       ; skip to start of dmi data
  xor   ax,ax                       ; set AX to terminating zero
  cmp   cx,1                        ; check if first string
  jle   short SDS_STRING_FOUND      ; if first string don't search for 0
  dec   cx

SDS_NEXT_STRING:
  push  cx
  mov   cx,MAX_STRING_LEN
  cld
  repne scasb                       ; scan ES:DI to find terminating zero
  pop   cx
  loop  SDS_NEXT_STRING             ; process next string

SDS_STRING_FOUND:
  push  di
  mov   cx,di
  push  cx
  mov   cx,MAX_STRING_LEN
  repne scasb                       ; scan ES:DI to find terminating zero
  pop   cx
  mov   ax,di
  sub   ax,cx                       ; AX contains size of old string
  pop   di

  mov   cx,[si].data_length         ; get new string length
  mov   dx,di                       ; save current string start location in DX
  cmp   cx,ax                       ; determine if new string larger or smaller
  je    short COPY_NEW_STRING       ; if strings equal size then straight copy

  les   di,sds_dmi_work_buf         ; load ES:DI with work buffer
  add   di,DMI_TBL_OFFSET+DMI_SIGNATURE_SIZE       
  cmp   cx,ax
  jg    short NEW_STRING_LARGER     ; new string is larger
  jl    short NEW_STRING_SMALLER    ; new string is smaller

NEW_STRING_LARGER:

  push  cx                          ; save length
  sub   cx,ax                       ; CX = length to increase

  call  move_everything_down        ; make room for new string

  push  dx
  shr   edx,16                      ; restore from upper word of DX
  mov   di,dx                       ; ES:DI points to string to increase
  sub   di,2                        ; backup to length
  mov   ax,es:[di]                  ; read length into AX
  add   ax,cx                       ; length to increase
  mov   es:[di],ax                  ; write back new length
  pop   dx

  pop   cx                          ; restore length
  jmp   short COPY_NEW_STRING       ; copy new string in

NEW_STRING_SMALLER:

  push  cx                          ; save length
  sub   ax,cx
  mov   cx,ax                       ; CX = length to decrease

  call  move_everything_up          ; make room for new string

  push  dx
  shr   edx,16                      ; restore from upper word of DX
  mov   di,dx                       ; ES:DI points to string to decrease
  sub   di,2                        ; backup to length
  mov   ax,es:[di]                  ; read length into AX
  sub   ax,cx                       ; length to increase
  mov   es:[di],ax                  ; write back new length
  pop   dx

  pop   cx                          ; restore length
  jmp   short COPY_NEW_STRING

COPY_NEW_STRING:

  mov   di,dx                       ; restore current string start location in DX
  lds   si,sds_dmi_data_buf         ; read into DS:SI dmiDataBuffer
  add   si,size dmi_set_struct      ; jump to string to be added
  cld
  rep   movsb
  jmp   short SDS_WRITE_TO_FLASH

;=============================================================================
; WRITE TO FLASH
;=============================================================================
SDS_WRITE_TO_FLASH:

  lds   si,sds_dmi_work_buf         ; load DS:SI with work buffer
  call  set_max_and_num_structs     ; adjust Max Struct Size and Num Structs

  mov   ax,sds_dmi_selector
  mov   es,ax                       ; ES:DI = get seg/selector of DMI pool
;  mov   cx,ESCD_SIZE                ; CX = size in bytes RXX
  mov   di,PARAM_BLOCK_BASE         ; start at beginning of parameter block

;R28LDCM  test  word ptr sds_control_flag,SET_DMI_POOL  ; R11
;R28LDCM  jz    short sds_no_flash2                           ; R11

  call  Flash_Write                 ; write to flash
;R23  jmp   short SDS_SUCCESS
;R23 start
  jnc   short SDS_SUCCESS
  mov   ax,DMI_NO_CHANGE
  jmp   short SDS_EXIT
;R23 end

sds_no_flash2:                      ; R11
  jmp   short SDS_SUCCESS

SDS_SUCCESS:                        ; R18 - common success exit point
  mov   ax,DMI_SUCCESS              ; handle found and is being returned
  jmp   short SDS_EXIT                    ; exit

SDS_INVALID_HANDLE:
  mov   ax,DMI_INVALID_HANDLE       ; invalid handle (handle already used)
  jmp   short SDS_EXIT                    ; exit

SDS_BAD_PARAMETER:                  ; bad command passed in
  mov   ax,DMI_BAD_PARAMETER        ; 
  jmp   short SDS_EXIT                    ; exit

SDS_EXIT:


; R17 start
;=============================================================================
; Update Shadow RAM with Max and Num Structs
;=============================================================================
  cmp   ax,DMI_SUCCESS              ; handle found and is being returned
  je    short UPDATE_SHADOW_MEM
  cmp   ax,DMI_NO_CHANGE            ; Signal old value same as new     R13
  je    short UPDATE_SHADOW_MEM
  jmp   short NO_UPDATE_SHADOW

UPDATE_SHADOW_MEM:

  lds   si,sds_dmi_work_buf         ; load DS:SI with work buffer
  mov   es,sds_bios_selector        ; BiosSelector has base of 0F0000h
  mov   di,offset dmi_shadow_info   ; point to reserved shadow area R19

  pusha
  call  F000_Shadow_W               ; Enable F000 shadow writeable
  mov   cx,8                        ; size of signature+max size+num structs
  rep   movsb                       ; move signature, max size and num structs to shadow
  call	Store_DMI_to_Shadow		;R28
  call  F000_Shadow_R               ; Enable F000 shadow readonly
  popa

; R17 end

NO_UPDATE_SHADOW:

  popf                              ; restore flags (including interupts)

  ret

set_dmi_struct endp

;R28 start
		public	Store_DMI_to_Shadow
Store_DMI_to_Shadow:
;R31 start
		push	ds			;store register
		push	es
		push	si

ifdef	ESCD_M2
		mov	ax,DMI_STORAGE_BASE
else	;ESCD_M2
		mov	ax,DMI_STORAGE_BASE_16
endif	;ESCD_M2
		mov	es,ax			;get DMI pool base

		sub	si,8			;set SI to start offset of DMI
		mov	cx,DMI_STORAGE_SIZE-1	;get DMI pool size - 1(checksum byte)
		xor	ah,ah			;clear value
@@:
		lodsb				;get data
		add	ah,al			;count
		loop	@B			;next loop
		mov	ds:[si],ah		;store to checksum byte
		pop	si			;restore original SI offset
		push	si
		sub	si,8			;set SI to start offset of DMI
		mov	di,PARAM_BLOCK_BASE+DMI_TBL_OFFSET	;set to DMI pool offset in flash ROM
		mov	cx,DMI_STORAGE_SIZE	;get dmi pool size
		call	Flash_Write		;reflash it

		pop	si			;restore register
		pop	es
		pop	ds
;R31 end
	push	word ptr ds:[si-4]	;store number of structure
	mov	di,Shadow_DMI_Address
	xor	dx,dx
	mov	bx,Shadow_DMI_Len_Address
@@:
	lodsw
	or	ax,ax
	jz	short @F
	cmp	ax,0ffffh
	je	short @F
	mov	es:[bx],ax
	mov	cx,ax
	rep	movsb
	add	dx,ax
	add	bx,2
	cmp	dx,DMI_MAX_POOL_SIZE
	jb	short @B
@@:
	pop	ax			;restore number of structure
	lea	di,DMI_BIOS_Struct_Table
;R32 start
ifdef SMBIOS_V21_Support
	mov	es:[di+01ch],ax		;write total number to DMI table
	mov	es:[di+16h],dx		;write total length to DMI table
	xor	al,al
	mov	es:[di+15h],al		;clear checksum
	push	di
	mov	cx,15
@@:
	add	al,es:[di+10h]
	inc	di
	loop	@B
	not	al
	inc	al
	pop	di
	mov	es:[di+15h],al		;set checksum byte


	lea	si,dmi_shadow_info
	mov	ax,es:[si]
	mov	es:[di+08h],ax
	xor	al,al
	mov	es:[di+4],al		;clear checksum
	push	di
	mov	cx,16
@@:
	add	al,es:[di]
	inc	di
	loop	@B
	not	al
	inc	al
	pop	di
	mov	es:[di+4],al		;set checksum byte
else	;SMBIOS_V21_Support
;R32 end
	mov	es:[di+0ch],ax		;write total number to DMI table
	mov	es:[di+6],dx		;write total length to DMI table
	xor	al,al
	mov	es:[di+5],al		;clear checksum
	push	di
	mov	cx,15
@@:
	add	al,es:[di]
	inc	di
	loop	@B
	not	al
	inc	al
	pop	di
	mov	es:[di+5],al		;set checksum byte
endif	;SMBIOS_V21_Support			;R32
	ret
;R28 end

;R27 start
;=============================================================================
; FUNC: DMI_GET_STRUCT_CHNG_INFO (Function 53h - Get DMI Struct Change Info)
;
; DESC:  This function will allow system software to get information
; about what type of DMI event occured.  The DMI event information will be
; returned in the 16 byte memory buffer pointed to by dmiEventStructure in
; the following format:
;
; SYNOPSIS:
;
; short FAR (*entryPoint)(Function, dmiChangeStructure, BiosSelector);
;
; short Function;                       - PnP BIOS Function 53h
; unsigned char FAR *dmiChangeStructure - Pointer to DMI Change Structure
; unsigned short BiosSelector;          - PnP BIOS read/writable selector
;
; IN:   NONE
; OUT:  AX  - Error Code (0 if successful)
;=============================================================================
		public	dmi_get_struct_chng_info 	;R02
dmi_get_struct_chng_info proc near
  gsci_func_num          equ [ebp+FN_OFS]     ; PnP BIOS Function 53h
  gsci_dmi_change_struct equ [ebp+FN_OFS+02h] ; Pointer to DMI Change Structure
  gsci_bios_selector     equ [ebp+FN_OFS+06h] ; BIOS read/writeable selector

;  lds   si,gsci_dmi_change_struct
;  mov   al,DMI_CMOS_CHANGE_STATUS   ; the spec defines a word for this
;  call  Get_cmos                    ; although only a byte is needed
;  mov   [si].change_status,ax
;
;  cmp al,DMI_SINGLE_STRUC_CHANGE
;  jne short DGSCI_1
;  mov   al,DMI_CMOS_STRUC_HNDL_BYTE1
;  call  Get_cmos                    ; read high byte
;  mov   ah,al
;  mov   al,DMI_CMOS_STRUC_HNDL_BYTE0
;  call  Get_cmos                  read low byte
;  mov   [si].struct_handle,ax
;
;DGSCI_1:

  mov   ax,DMI_FUNCTION_NOT_SUPPORTED
  ret

dmi_get_struct_chng_info endp

;=============================================================================
; FUNC: DMI_CONTROL (Function 54h - DMI Control)
;
; DESC: Provides the interface to perform implementation-specific functions
;       for the systen, as defined by the SubFunction parameter and its
;       optional data values
;
; SYNOPSIS:
;
; short FAR (*entryPoint) (Function, SubFunction, Data, Control, \
;   BiosSelector);
;
; short Function;                   - PnP BIOS Function 53h
; short SubFunction;                - Defines the specific operation
; void FAR *Data;                   - Input/Output data buffer, Subfunc specific
; unsigned char Control             - Conditions for performing operation
; unsigned short BiosSelector;      - PnP BIOS read/writable selector
;
; IN:   NONE
; OUT:  AX  - Error Code (0 if successful)
;=============================================================================
		public	dmi_control 	;R02
dmi_control proc near
  dc_func_num            equ [ebp+FN_OFS]     ; PnP BIOS Function 54h
  dc_sub_func_num        equ [ebp+FN_OFS+02h] ; Defines the specific operation
  dc_data                equ [ebp+FN_OFS+04h] ; Input/Output data buffer, Subfunc specific
  dc_control             equ [ebp+FN_OFS+08h] ; Conditions for performing operation
  dc_bios_selector       equ [ebp+FN_OFS+0Ah] ; BIOS read/writeable selector

;  mov   ax,dc_func_num
;  .if   ax == DMI_CLEAR_EVENT_LOG
;  .elseif ax == DMI_CONTROL_LOGGING
;  .endif
;
;  cmp ax,DMI_CLEAR_EVENT_LOG
;  jne  short DC_2
;  ; do some processing for equal case
;DC_2:
;  cmp ax,DMI_CONTROL_LOGGING
;  jne short DC_3
;  ; do some processing here for equal case
;DC_3:
;  mov   ax,DMI_FUNCTION_NOT_SUPPORTED
  mov   ax,DMI_SUCCESS
  ret

dmi_control endp

;=============================================================================
; FUNC: GET_GPNV_INFO (Function 55h - Get General-Purpose NonVolatile Info)
;
; DESC:  This function provides information about a General Purpose
; nonvolatile area (GPNV).  The handle is a pointer to a number that ident-
; ifies which GPNV's information is requested, a value of zero accesses the
; first (or only) area.
;
; SYNOPSIS:
;
; short FAR (*entryPoint) (Function, Handle, MinGPNVWSize, GPNVSize, \
;   NVStorageBase, BiosSelector);
;
; short Function;                         - PnP BIOS Function 55h
; unsigned short FAR *Handle              - Identifies which GPNV to access
; unsiged short FAR *MinGPNVWriteSize;    - Min buf size for writing NVS
; unsigned short FAR *GPNVSize;           - Size allocated for the GPNV
; unsigned long FAR *NVStorageBase;       - 32-bit physical base addr for NVS
; unsigned short BiosSelector;            - PnP BIOS read/writable selector
;
;
; IN:   NONE
; OUT:  AX  - Error Code (0 if successful)
;=============================================================================
		public	get_gpnv_info
get_gpnv_info proc near
  ggi_func_num      equ [ebp+FN_OFS]       ; short Function
  ggi_handle	    equ [ebp+FN_OFS+02h]   ; Identifies which GPNV to access
  ggi_min_gpnv_size equ [ebp+FN_OFS+06h]   ; unsiged short FAR *MinGPNVWriteSize
  ggi_gpnv_size     equ [ebp+FN_OFS+0Ah]   ; unsigned short FAR *GPNVSize
  ggi_nvs_storeBase equ [ebp+FN_OFS+0Eh]   ; unsigned long FAR *NVStorageBase
  ggi_bios_selector equ [ebp+FN_OFS+012h]   ; BIOS read/writeable selector

;;  les   di,ggi_handle        ; load ES:DI with handle
;;  mov   dx,es:[di]
;;	lea	si,Shadow_DMI_Address
;;  mov	cx,word ptr cs:[DMI_BIOS_Struct_Table+12]
;;	lea	bx,Shadow_DMI_Len_Address
;;Check_next_handle:
;;  mov	ax,cs:[si+2]
;;  add	si,cs:[bx]
;;  add	bx,2
;;  cmp	ax,0ffffh
;;  je	short Got_struct
;;  cmp	dx,ax
;;  jne	short Check_next_handle
;;  mov	ax,cs:[si+2]
;;Got_struct:
;;  mov	es:[di],ax
;;
;;  les   di,ggi_min_gpnv_size        ; load ES:DI with MinGPNVWriteSize
;;  mov	cx,word ptr cs:[DMI_BIOS_Struct_Table+6]
;;  mov   word ptr es:[di],cx
;;
;;  les   di,ggi_gpnv_size            ; load ES:DI with GPNVSize
;;  mov   word ptr es:[di],PARAM_BLOCK_SIZE
;;
;;  les   di,ggi_nvs_storeBase        ; load ES:DI with NVStorageBase
;;  mov   dword ptr es:[di],DMI_STORAGE_BASE
;R30  mov   ax,DMI_SUCCESS
  mov   ax,DMI_FUNCTION_NOT_SUPPORTED	;R30
  ret

get_gpnv_info endp

;=============================================================================
; FUNC: 56h - Read Extended System Configuration Data (GPNV)
;
; SYNOPSIS:
;
; short FAR (*entryPoint)(Function, GPNVBuffer, GPNVSelector, BiosSelector)
;
; short Function;                /* PnP BIOS Function 042h */
; unsigned short FAR *Handle              - Identifies which GPNV to access
; char FAR *GPNVBuffer;        /* Addr of caller's buffer for storing GPNV */
; unsigned short GPNVSelector;   /* GPNV read/writable selector */
; unsigned short BiosSelector;   /* PnP BIOS read/writable selector */
;
; DESC:
;
; Optional.  This function is used to read the GPNV data from nonvolatile
; storage on the system into the buffer specified by GPNVBuffer.
; The caller should use the output from Function 41 (the GPNVSize field)
; when calculating the size of the GPNVBuffer.  The system BIOS will return
; the entire GPNV, including information about system board devices.
; In protected mode, the GPNVSelector has base = NVStorageBase and limit of
; at least NVStorageSize. In real mode, the GPNVSelector is a segment that
; points to NVStorageBase.
;
; IN:   NONE
; OUT:  AX  - Error Code (0 if successful)
;=============================================================================
		public	read_gpnv 	;R02
read_gpnv proc near
  rg_func_num      equ [ebp+FN_OFS]       ; short Function
  rg_gpnv_buffer   equ [ebp+FN_OFS+02h]   ; char FAR *GPNVBuffer
  rg_gpnv_selector equ [ebp+FN_OFS+06h]   ; unsigned short GPNVSelector
  rg_bios_selector equ [ebp+FN_OFS+08h]   ; BIOS read/writeable selector

;  les   di,rg_gpnv_buffer           ; load ES:DI with GPNVBuffer
;
;  mov   ax,rg_gpnv_selector
;  mov   ds,ax                       ; put GPNVSelector in DS
;  mov   si,GPNV_OFFSET              ; put offset of GPNV in SI
;
;  mov   cx,GPNV_BLOCK_SIZE
;  call  Flash_Read
;  mov   ax,DMI_SUCCESS

  mov   ax,DMI_FUNCTION_NOT_SUPPORTED
  ret

read_gpnv endp

;=============================================================================
; FUNC: 57h - Write general-purpose nv data (GPNV)
;
; SYNOPSIS:
;
; short FAR (*entryPoint)(Function, Handle, GPNVBuffer, GPNVSelector, BiosSelector);
;
; short Function;                /* PnP BIOS Function 043h */
; unsigned short FAR *Handle              - Identifies which GPNV to access
; char FAR *GPNVBuffer;        /* Buffer containing complete GPNV to write... */
; unsigned short GPNVSelector;   /* GPNV read/writable selector */
; unsigned short BiosSelector;   /* PnP BIOS read/writable selector */
;
; DESC:  This function will write the entire (GPNV) from the GPNVBuffer
;               into the nonvolatile storage area.
;
; IN:   NONE
; OUT:  AX  - Error Code (0 if successful)
;=============================================================================
		public	write_gpnv 	;R02
write_gpnv proc near
  wg_func_num      equ [ebp+FN_OFS]       ; short Function
  wg_gpnv_buffer   equ [ebp+FN_OFS+02h]   ; char FAR *GPNVBuffer
  wg_gpnv_selector equ [ebp+FN_OFS+06h]   ; unsigned short GPNVSelector
  wg_bios_selector equ [ebp+FN_OFS+08h]   ; BIOS read/writeable selector

;  mov   ax,wg_gpnv_selector
;  mov   es,ax                       ; get seg/selector of GPNV
;  mov   di,GPNV_OFFSET              ; get offset of GPNV
;  lds   si,wg_gpnv_buffer           ; load DS:SI with GPNVBuffer
;  mov   cx,GPNV_BLOCK_SIZE          ; get size of block
;  call  Flash_Write
;  mov   ax,DMI_SUCCESS

  mov   ax,DMI_FUNCTION_NOT_SUPPORTED
  ret

write_gpnv endp

;R27 end

;=============================================================================
;                             UTILITY FUNCTIONS
;=============================================================================

;=============================================================================
; FUNC: SET_MAX_AND_NUM_STRUCTS
;
; DESC: Find the maximum structure size in the DMI pool and set max struct
;       size to this value
; 
; IN:   DS:SI - Start of 4K Buffer
; OUT:  CX    - Size of block to update (to the nearest granularity)
; ASSUMES THAT DMI POOL HAS A VALID TERMINATOR OF 0FFFFh AT END
; DESTROYED: DX
;=============================================================================
set_max_and_num_structs proc near

  push  si                          ; save start of buffer
  add   si,DMI_TBL_OFFSET+DMI_SIGNATURE_SIZE       

ifdef MASM611
  assume si:ptr dmi_info_struct
endif

  push  si
  xor   cx,cx                       ; clear num structs to zero
  xor   dx,dx                       ; clear max struct to zero
  add   si,size dmi_info_struct     ; skip table information

SMSS_CHECK_NEXT_STRUCT:
  mov   ax,word ptr ds:[si]         ; read in length

;R22  cmp   ax,0
  or	ax,ax				;R22
  je    short SMSS_END_OF_POOL
;R26  cmp   ax,MAX_STRUCTURE_SIZE       ; if struct length is > max
;R26  jge   short SMSS_END_OF_POOL

  add   si,2
ifdef MASM611
  assume si: ptr dmi_struct_hdr
endif
  mov   bx,ds:[si].dmi_handle       ; check for terminator
  cmp   bx,DMI_TERMINATOR
  je    short SMSS_END_OF_POOL

  cmp   ax,dx
  jle   short SMSS_STRUCT_NOT_BIGGER
  mov   dx,ax                       ; struct is bigger
SMSS_STRUCT_NOT_BIGGER:

  add   si,ax
  inc   cx                          ; increment number of structs
  jmp   short SMSS_CHECK_NEXT_STRUCT

SMSS_END_OF_POOL:

  mov   ax,si
  pop   si


ifdef MASM611
  assume si:ptr dmi_info_struct
endif

  mov   ds:[si].num_structs,cx      ; get num structs
  mov   ds:[si].max_struct_size,dx  ; get max structs

  pop   si                          ; restore start of buffer

;R25 ifdef INTEL_28F001BXT
  mov   cx,DMI_STORAGE_SIZE         ; size needed to hold block
;R25;R24 elseif SST_PH29EE010
;R25else	;SST_PH29EE010			;R24
;R25
;R25  mov   cx,ax                       ; CX = end of pool
;R25  add   cx,offset dmi_handle        ; add offset of dmi_handle
;R25
;R25  sub   cx,si                       ; CX = size of pool
;R25  mov   ax,FLASH_GRANULARITY        ; get the mod of the granual size
;R25  dec   ax                          
;R25  xor   ax,0FFFFh
;R25  and   cx,ax                       ; CX = size of pool % Granularity
;R25  add   cx,FLASH_GRANULARITY        ; adjust up by one granual
;R25
;R25  cmp   cx,DMI_MAX_POOL_SIZE
;R25  jle   short smn_exit
;R25  mov   cx,DMI_MAX_POOL_SIZE
;R25endif

smn_exit:

  ret

set_max_and_num_structs endp

;=============================================================================
; FUNC: MOVE_EVERYTHING_UP
;
; DESC: Move everything up (Shrink DMI Pool)
;       
; IN:   CX    - Amount in bytes to decrease 
;       DX    - Current offset of start of String to overwrite
;       ES:DI - Start of DMI pool
; OUT:  NONE
;=============================================================================
move_everything_up proc

  pusha                             ; save all general registers

  push  cx                          ; save number of bytes to decrease
  mov   ax,dx                       ; 
  sub   ax,di                       ; AX - offset from beginning of pool
  mov   cx,ax                       ; 
  mov   ax,DMI_MAX_POOL_SIZE        ; 
  sub   ax,cx                       ; AX - number of bytes to move
  mov   cx,ax                       ; CX - number of bytes to move
  pop   ax                          ; restore number of bytes to decrease

  push  es
  pop   ds                          ; 
  mov   di,dx                       ; restore current location
  mov   si,di
  add   si,ax                       ; 

  cld
  rep   movsb                       ; mov strings down further

  popa                              ; restore all general registers

  ret

move_everything_up endp

;=============================================================================
; FUNC: MOVE_EVERYTHING_DOWN (Grow DMI Pool)
;
; DESC: Move everything down - need to check to see if it will fit
;       1.  Move Structures down
;       2.  Move Strings Down
;       
; 
; IN:   CX    - Amount in bytes to increase
;       DX    - Current offset of start of String to overwrite
;       ES:DI - Start of DMI pool
; OUT:  CY    - Move failed
;=============================================================================
move_everything_down proc near

  pusha                             ; save all general registers

  push  cx                          ; save length to move
  mov   bx,DMI_TERMINATOR           ; goto end of pool
ifdef MASM611
  assume di: ptr dmi_info_struct
endif
  mov   cx,es:[di].num_structs      ; get number of structures in DMI TBL
  push  di                          ; save offset of beginning of pool
  add   di,size dmi_info_struct     ; skip table information
  call  get_structure               ; point ES:DI to end of DMI pool
  pop   si                          ; restore offset from beggining of pool
  pop   cx                          ; restore length to move
  call  check_for_room              ; verify that there is room to add
  jc    short MED_FAILED            ; quit if no room to add

  push  cx                          ; save number of bytes to increase
  mov   ax,di                       ; AX = end of used pool
  sub   ax,dx                       ; AX = end - current
  mov   cx,ax                       ; CX = number of bytes to copy
  inc   cx                          ; increase number of bytes to copy by one
  pop   dx                          ; restore number of bytes to increase

  mov   si,di                       ; SI = end of pool
  add   di,dx                       ; DI = end of pool + num bytes to add
  push  es                          ; push destination segment
  pop   ds                          ; destination segment same as source

  std                               ; set direction as backward
  rep   movsb                       ; mov strings down further

MED_FAILED:
  stc                               ; move failed
  jmp   short MED_EXIT

MED_SUCCESS:
  clc                               ; move successfull

MED_EXIT:

  popa                              ; restore all general registers

  ret

move_everything_down endp


;=============================================================================
; FUNC: CHECK_FOR_ROOM
;
; DESC: Check for room to move down
; 
; IN:   SI    - Offset from beggining of DMI Work Buffer
;       DI    - Current end of used pool area
;       CX    - length to add
; OUT:  CY    - No room to move
; DESTROYED: AX
;=============================================================================
check_for_room proc near

  push  di
  sub   di,si                       ; DI = offset from begining
  xor   ax,ax                       ; clear size
  add   ax,di                       ; add offset of last structure
  add   ax,cx                       ; add length to move down
  cmp   ax,DMI_MAX_POOL_SIZE        ; check if there is room to move
  jae   short CFR_NO_ROOM
  clc
  jmp   short CFR_EXIT

CFR_NO_ROOM:
  stc   

CFR_EXIT:
  pop   di

  ret

check_for_room endp

ifndef ESCD_M2				;R28
;=============================================================================
; FUNC: CHECK_DMI_SIGNATURE
;
; DESC: Check for valid DMI signature
; 
; IN:   DS:SI - Start of Pool - Points to Signature location
;       ES:DI - Buffer of atleast 4 Bytes for use by check DMI signature
; OUT:  CY    - Signature invalid
;       DS:SI - Moved to beginning of Structure Information
; DESTROYED:  EAX,ECX
;=============================================================================
check_dmi_signature proc

  mov   ax,DMI_SIGNATURE_SIZE       
  call  read_dmi
  mov   eax,es:[di]
  mov   ecx,DMI_SIGNATURE           ; check for valid pool
  add   si,DMI_SIGNATURE_SIZE       
  cmp   eax,ecx                     ; compare high byte
  je    short CDS_SIGNATURE_OK

CDS_INVALID_POOL:
  stc                               ; signal signature not found
  jmp   short CDS_EXIT

CDS_SIGNATURE_OK:
  clc                               ; signal signature found

CDS_EXIT:

  ret

check_dmi_signature endp
endif ;ESCD_M2				;R28


;=============================================================================
; FUNC: READ_DMI
;
; DESC: Read information into buffer at ES:DI
;
; IN:   AX    - Size
;       DS:SI - Source
;       ES:DI - Destination
; OUT:  NONE
;=============================================================================
read_dmi proc near

  push  ds
  push  si
  push  di
  push  cx
  mov   cx,ax
  call  Flash_Read
  pop   cx
  pop   di
  pop   si
  pop   ds

  ret

read_dmi endp


;=============================================================================
; FUNC: GET_STRUCTURE
;
; DESC: Get a DMI Structure out of the pool
;
; IN:   BX    - Structure Handle to Find
;       ES:DI - Start of first structure
;       CX    - Number of structures in pool
; OUT:  ES:DI - Start of structure to find
;       NC    - Structure has been found
;       CY    - Structure not found
;=============================================================================
get_structure proc near

  push  ax
  push  dx
  push  cx                          ; save number of structures

  or    cx,cx                       ; if no structures present,
  jz    short GS_STRUCT_NOT_FOUND  ;  then skip search

GS_CHECK_NEXT_STRUCT:
  mov   ax,word ptr es:[di]         ; read in length
  add   di,2                        ; advance DI to structure header

  cmp   ax,0                        ; if struct length is zero
  je    short GS_STRUCT_NOT_FOUND   ; struct not found
;R26  cmp   ax,MAX_STRUCTURE_SIZE       ; if struct length is > max
;R26  jge   short GS_STRUCT_NOT_FOUND   ; struct not found

ifdef MASM611
  assume di: ptr dmi_struct_hdr
endif
  mov   dx,es:[di].dmi_handle
  cmp   dx,DMI_TERMINATOR
  je    short GS_STRUCT_NOT_FOUND

  cmp   bx,dx
  jz    short GS_STRUCT_FOUND 
  add   di,ax
  loop  short GS_CHECK_NEXT_STRUCT

GS_STRUCT_NOT_FOUND:
  stc
  jmp   short GS_EXIT

GS_STRUCT_FOUND:
  clc

GS_EXIT:

  pop   cx                          ; restore number of structures
  pop   dx
  pop   ax

  ret

get_structure endp

;R26;=============================================================================
;R26; PROC: CHECK_MAX_STRUCT_SIZE
;R26;
;R26; DESC: Verify that the number of structs is less than maximum
;R26;
;R26; IN:   ES:DI - buffer that can be used to read in maximum
;R26; OUT:  AX    - Current structure size
;R26;=============================================================================
;R26check_max_struct_size proc near
;R26
;R26  mov   ax,2                        ; 
;R26  call  read_dmi
;R26  mov   ax,word ptr es:[di]
;R26  cmp   ax,MAX_STRUCTURE_SIZE       ; compare size
;R26  ret
;R26
;R26check_max_struct_size endp


Public  END_DMI_RUNTIME              ; Used to track overall size of
END_DMI_RUNTIME LABEL BYTE           ;  DMI Post 

;R12B start
FCODE    ENDS
endif;   DMI_ENABLED
    END
;R12B end

