;	[]===========================================================[]
;
;	NOTICE: THIS PROGRAM BELONGS TO AWARD SOFTWARE INTERNATIONAL(R)
;	        INC. IT IS CONSIDERED A TRADE SECRET AND IS NOT TO BE 	
;	        DIVULGED OR USED BY PARTIES WHO HAVE NOT RECEIVED	
;	        WRITTEN AUTHORIZATION FROM THE OWNER.
;
; 	[]===========================================================[]
;
;=============================================================================
; FILE:     DMI_CT.INC
;
; DESC:     Chipset Specific BIOS Support
; CHIPSET:  SiS 5597
;
; BY:       Lerry, Award Software Intl., Inc.
; DATE:     03/26/97
;=============================================================================
;----------------------------------------------------------------------------
;Rev	Date	 Name	Description
;--------------------------2A5II---------------------------------------------
;R03  12/12/97 STV  Fixed that some configuration show fail in DMI.
;		    Added 
;R02	11/13/97 KVN	Fixed DRAM size and type return wrong data to DMIPOST
;----------------------------------------------------------------------------
;

;R02 start
F000_Get_Ct:
	F000_call	Get_Ct
	ret
;R02 end

;=============================================================================
; FUNC: CT_GET_DMI_MEMORY_TYPE
;
; DESC: Get the memory type for this chipset
;
; IN:   CL  - Bank
; OUT:  AX  - Memory type define as:
;		MEM_TYPES_OTHER
;		MEM_TYPES_UNKNOWN
;		MEM_TYPES_STANDARD
;		MEM_TYPES_FAST_PAGE
;		MEM_TYPES_EDO
;		MEM_TYPES_PARITY
;		MEM_TYPES_ECC
;		MEM_TYPES_SIMM
;		MEM_TYPES_DIMM
;		MEM_TYPES_BURST_EDO
;		MEM_TYPES_SDRAM
;
;=============================================================================
ct_get_dmi_memory_type proc uses bx edx cx  ;change dx to edx because SiS-5597
 					    ;F000_call Get_Ct routin could destroy edx


	push	cx
	push	bx

	mov	bl,cl
	mov	cx,63h
	call	F000_Get_Ct
	mov	cl,bl
	shr	al,cl
	test	al,01h
;R02	xor	ax,ax
	mov	ax,0	;R02
	jz	short get_dmi_type_ok

	mov	cx,60h
	add	cl,bl
	call	F000_Get_Ct
	xchg	bx,ax			;save al to bl
	cmp	bl,11000000b
        mov     ax,MEM_TYPES_SDRAM	;SDRAM RAM Present
	jae	short get_dmi_type_ok
	cmp	bl,01000000b
;R02        mov     ax,MEM_TYPES_SDRAM	;EDO RAM Present
        mov     ax,MEM_TYPES_EDO	;R02EDO RAM Present
	jae	short get_dmi_type_ok
;R02        mov     ax,MEM_TYPES_SDRAM	;FAST_PAGE RAM Present
        mov     ax,MEM_TYPES_FAST_PAGE	;R02FAST_PAGE RAM Present
get_dmi_type_ok:
	pop	bx
	pop	cx
	ret

ct_get_dmi_memory_type endp

;=============================================================================
; FUNC: CT_GET_DMI_MEM_SIZE
;
; DESC: Return Memory Size for Bank
;
; IN:   CL  - Bank
; OUT:  AL  - Size (Memory size = 2 ^ AL MB)
;=============================================================================
ct_get_dmi_mem_size proc uses bx cx edx	    ;change dx to edx because SiS-551x
					    ;F000_call Get_Ct routin could destroy edx
	push	cx
	push	bx

	call	ct_get_dmi_memory_type
	or	ax,ax
	mov	ax,MEM_MAX_MODULE_000MB	;R02
	jz	short get_dmi_size_ok

	xor	ch,ch
	add	cl,60h
	call	F000_Get_Ct
	mov	cl,al			;save al to bl
	mov	bx,offset DRAM_tbl_FP_EDO
	cmp	al,11000000b
	jb	short trans_mem_size
	mov	bx,offset DRAM_tbl_SDRAM
trans_mem_size:
	and	al,0Fh
	db	2Eh			;CS:
	xlat
	test	cl,20h
	jz	short test_32bit
	inc	ax
test_32bit:
	test	cl,10h
	jz	short get_dmi_size_ok
	dec	ax
get_dmi_size_ok:
	pop	bx
	pop	cx
	ret

DRAM_tbl_FP_EDO:
	db	1,3,5,7,3,4,5,6,2,3,4
DRAM_tbl_SDRAM:
	db	3,5,5,6,4,6,6,7,5,7,7,8,4
ct_get_dmi_mem_size endp

;=============================================================================
; FUNC: CT_GET_DMI_MEM_SPEED
;
; DESC: Return Memory Speed for Bank
;
; IN:   CL  - Bank
; OUT:  AL  - Speed (unit is "ns")
;=============================================================================
ct_get_dmi_mem_speed proc uses edx

ifdef   DMI_SPEED_UNDETERMINEABLE

	mov	al,MEM_SPEEDS_UNKNOWN       ;replace if speed is determineable

endif;  DMI_SPEED_UNDETERMINEABLE

	ret

ct_get_dmi_mem_speed endp

;R03 - starts
;=============================================================================
; FUNC: CT_GET_DMI_MEM_INTERLEAVE
;
; DESC: Return Memory Current Interleave
;
; IN:   CL  - 
; OUT:  AL  - 01h : Other
;	      02h : Unknown
;	      03h : One Way Interleave
;	      04h : Two Way Interleave
;	      05h : Four Way Interleave
;	      06h : Eight Way Interleave
;	      07h : Sixteen Way Interleave
;=============================================================================
ct_get_dmi_mem_interleave proc
	mov	al,03h
	ret
ct_get_dmi_mem_interleave endp

;=============================================================================
; FUNC: CT_GET_DMI_MEM_ERROR_DETECTING
;
; DESC: Return Memory Error Detecting Method
;
; IN:   CL  - 
; OUT:  AL  - 01h : Other
;	      02h : Unknown
;	      03h : None
;	      04h : 8-bit Parity
;	      05h : 32-bit ECC
;	      06h : 64-bit ECC
;	      07h : 128-bit ECC
;	      08h : CRC		;for SMBIOS V2.1
;=============================================================================
ct_get_dmi_mem_error_detecting proc
	mov	al,04h
	ret
ct_get_dmi_mem_error_detecting endp

;=============================================================================
; FUNC: CT_GET_DMI_CACHE_CONFIG
;
; DESC: Get the cache config for this chipset
;
; IN:   CL  - Bank
; OUT:  AX  - Bits 15:10 Reserved, must be zero
;	      Bits  9:8  Operational Mode
;	      		00b Write Through
;	      		01b Write Back
;	      		10b Varies with Memory Address
;	      		11b Unknown
;	      Bit 7 Enable/Disable(at boot time)
;	      		1b Enable
;	      		0b Disable
;	      Bits 6:5 Location,relative to the CPU
;	      module:
;	      		00b Internal
;	      		01b External
;	      		10b Reserved
;	      		11b Unknown
;	      Bit 4 Reserved,must be zero
;	      Bit 3 Cache Socketed
;	      		1b Socketed
;	      		0b Not Socketed
;	      Bits 2:0 Cache Leave - 1 through 8,e.g
;	               an L1 cache would use value 000b and
;	               an L3 cache would use 010b.
;=============================================================================
ct_get_dmi_cache_config proc
	ret
ct_get_dmi_cache_config endp
;R03 - ends

