;	[]===========================================================[]
;
;	NOTICE: THIS PROGRAM BELONGS TO AWARD SOFTWARE INTERNATIONAL(R)
;	        INC. IT IS CONSIDERED A TRADE SECRET AND IS NOT TO BE 	
;	        DIVULGED OR USED BY PARTIES WHO HAVE NOT RECEIVED	
;	        WRITTEN AUTHORIZATION FROM THE OWNER.
;
; 	[]===========================================================[]
;

;   PAGE  60,132
;=============================================================================


;=============================================================================
;FILE:  DMI_MB.INC
;
;DESC:  Motherboard information for a specific motherboard
;
;BY:    Tim Markey
;DATE:  05/24/1996
;=============================================================================
;Rev  Date     Name  Description
;----------------------------------------------------------------------------
;R18  12/11/97 Stv Added SMBIOS V2.1 new functions (ifdef SMBIOS_v21_Support)
;R17  12/05/97 KVN Fixed up PCI data width to 32 bits in DMI type 9
;R16  10/01/97 KVN Added Type 15 (System Event Log) for LDCM v3.1
;R15  09/03/97 RCH Added five PCI slots	ID support.
;R14  09/02/97 RIC Add "VT586" for VIA request.
;R13  05/26/97 KVN Fix original coding mistake that cause running LDCM DMIBrow
;		   utility error in NT OS because our BIOS build a wrong NT DMI
;		   table about type 05
;R12  12/31/96 KVN Change DMI structure of BIOS date if update BIOS code
;R11  11/05/96 STV  1.Add system and system enclosure strings, rename base board strings
;		    2.Add support for more than one CPU
;		    3.New onboard device support
;		    4.Add support for more than variable memory slots
;		    5.Fixed equates for USB and PS/2 Mouse
;R10  10/04/96 KVN  Support DMI BIOS ROM size for Flash_2M_support to 2M
;R09  09/23/96 STV  Fixed bug that DMI structure PCI slot_id error
;R08  08/21/96 KVN  Added CPU brand auto detect for DMI processor component
;R07  08/16/96 KVN  Show 'Verify DMI Pool Data' during compare DMI pool data
;R06  07/23/96 TJM  Change enclosure type from 00h to 02h (unknown)
;R05  07/10/96 TJM  Fixed type05_size size equate (bug in MASM)
;R04  06/20/96 TJM  Move CPU struct to last struct, remove ifdef 
;                   (CPU MUST BE LAST AND PRESENT AS THIS IS ONLY STRUCTURE TO FLASH)
;R03  06/20/96 TJM  Added USB Port support
;R02  06/20/96 TJM  Added keyboard and ps/2 mouse support
;R01  05/24/96 TJM  Moved from dmipost.inc dmi_mb.inc
;----------------------------------------------------------------------------

;=============================================================================
; TYPE 0 - BIOS Information
;=============================================================================
type00_info label byte
  db  BIOS_INFORMATION                        ; type
ifdef SMBIOS_V21_Support		      ;R18
  db  13h                                     ;R18 len
else	;SMBIOS_V21_Support		      ;R18
  db  12h                                     ; len
endif	;SMBIOS_V21_Support		      ;R18
  dw  00h                                     ; handle
  db  01h                                     ; vendor string index
  db  02h                                     ; version string index
  dw  0E000h                                  ; segment
  db  03h                                     ; date string index
;R10 start
ifdef Flash_2M_support
  db  03h                                     ; rom size (256K)
else ;Flash_2M_support
;R10 end
  db  01h                                     ; rom size (128K)
endif ;Flash_2M_support				;R10
ifdef PRINT_SCREEN_SUPPORTED
  dd  BIOS_CHAR+PrintScreen                   ; bios characteristics byte low
else
  dd  BIOS_CHAR                               ; bios characteristics byte low
endif
  dd  00000000h                               ; bios characteristics byte high
ifdef SMBIOS_V21_Support		      ;R18
  db  BIOS_CHAR_EB 			      ;R18 bios characteristics extension bytes
endif	;SMBIOS_V21_Support		      ;R18
  db  'Award Software International, Inc.',0  ; vendor string
  db  '4.51 PG',0                             ; version string
  db  '00-00-0000',0                          ; string terminator
  db  0                                       ; string terminator
type00_size  equ  $-offset type00_info


;=============================================================================
; TYPE 01 - System Information
;=============================================================================
type01_info label byte
  db  SYSTEM_INFORMATION                      ; type
ifdef SMBIOS_V21_Support		      ;R18
  db  19h                                     ;R18 len
else	;SMBIOS_V21_Support		      ;R18
  db  08h                                     ; len
endif	;SMBIOS_V21_Support		      ;R18
  dw  00h                                     ; handle
  db  01h                                     ; manufacture string index
  db  02h                                     ; product string index
  db  03h                                     ; version string index
  db  04h                                     ; serial no. string index
ifdef SMBIOS_V21_Support		      ;R18
  db  16 dup(0)				      ;R18 16 Byters UUID
  db  Wake_up				      ;R18 Wake_up Type
endif	;SMBIOS_V21_Support		      ;R18
  db  Sys_Manufact_String                     ; manufacturer string  R11
  db  Sys_Product_String                      ; product string       R11
  db  Sys_Version_String                      ; version string       R11
;R11  db  0                                       ; manufacture string
;R11  db  0                                       ; product string
;R11  db  0                                       ; version string
  db  0                                       ; serial no. string 
  db  0                                       ; string terminator
type01_size  equ  $-offset type01_info

;=============================================================================
; TYPE 02 -  Base Board Information
;=============================================================================
type02_info label byte
  db  BASE_BOARD_INFORMATION                  ; type
  db  08h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; manufacture string index
  db  02h                                     ; product string index
  db  03h                                     ; version string index
  db  04h                                     ; serial no. string index
  db  MB_Manufact_String                      ; manufacturer string  R11
  db  MB_Product_String                       ; product string       R11
  db  MB_Version_String                       ; version string       R11
;R11  db  Manufact_String                         ; manufacturer string
;R11  db  Product_String                          ; product string
;R11  db  Version_String                          ; version string
  db  0                                       ; serial no. string 
  db  0                                       ; string terminator
type02_size  equ  $-offset type02_info

;=============================================================================
; TYPE 03 - Enclosure Information
;=============================================================================
type03_info label byte
  db  ENCLOSURE_INFORMATION                   ; type
ifdef SMBIOS_V21_Support		      ;R18
  db  0dh                                     ;R18 len
else	;SMBIOS_V21_Support		      ;R18
  db  09h                                     ; len
endif	;SMBIOS_V21_Support    		      ;R18	
  dw  00h                                     ; handle
  db  01h                                     ; manufacture string index
  db  TYPE03_UNKNOWN                          ; unknown R06
  db  02h                                     ; version string index
  db  03h                                     ; serial no. string index
  db  04h                                     ; asset tag string index
ifdef SMBIOS_V21_Support		      ;R18
  db  TYPE03_UNKNOWN			      ;R18 bootup state
  db  TYPE03_UNKNOWN			      ;R18 power supply state
  db  TYPE03_UNKNOWN			      ;R18 thermal state
  db  TYPE03_UNKNOWN			      ;R18 security state
endif	;SMBIOS_V21_Support		      ;R18
  db  Encl_Manufact_String                    ; manufacturer string  R11
  db  Encl_Product_String                     ; product string       R11
;R11  db  0                                       ; manufacture string
;R11  db  0                                       ; product string
  db  0                                       ; serial no. string 
  db  0                                       ; asset tag string
  db  0                                       ; string terminator
type03_size  equ  $-offset type03_info

;=============================================================================
; TYPE 4 - CPU Information
;=============================================================================
type04_info label byte
  db  CPU_INFORMATION                         ; type
ifdef SMBIOS_V21_Support		      ;R18
  db  20h                                     ;R18 len 
else	;SMBIOS_V21_Support		      ;R18
  db  1Ah                                     ; len
endif	;SMBIOS_V21_Support		      ;R18	
  dw  00h                                     ; handle
  db  01h                                     ; socket string index
  db  Central_Processor                       ; processor type0 enum
;R08  db  Pentium                                 ; processor family enum
  db  Unknown					;R08
  db  02h                                     ; manufacturer string index
  dd  00000000h                               ; processor ID
  dd  00000000h                               ; 
  db  03h                                     ; processor version string index
ifdef SMBIOS_V21_Support		      ;R18
  db  0A1h		                      ;R18 3.3 voltage for smbios v2.1
else	;SMBIOS_V21_Support		      ;R18
  db  TYPE04_VOLTAGE_3_3V                     ; voltage
endif	;SMBIOS_V21_Support		      ;R18
  dw  0000h                                   ; external clock
  dw  MAX_CPU_SPEED                           ; max_speed (fill at POST)
  dw  0000h                                   ; current speed (fill at POST) R05
  db  41h                                     ; status
  db  TYPE04_UPGRADE                          ; processor upgrade (DMI_MB.CFG)
ifdef SMBIOS_V21_Support		      ;R18
  dw  0000h				      ;R18 primary(Level1) cache
  dw  0000h				      ;R18 secondary(Level1) cache
  dw  0000h				      ;R18 tertiary(Level1) cache
endif	;SMBIOS_V21_Support		      ;R18
  db  'Socket 7',0                            ; socket string index
;R08  db  'Intel',0                               ; manufacturer string index
;R08  db  'Pentium',0                             ; processor string index
  db  0						;R08
  db  0						;R08
  db  0                                       ; string terminator
type04_size  equ  $-offset type04_info

;=============================================================================
; TYPE 05 - Memory Controller Information
;=============================================================================
type05_info label byte
  db  MEM_CTLR_INFORMATION                    ; type
;R13  db  15h+ (2*NUM_MEM_SLOTS)                  ; len
ifdef SMBIOS_V21_Support		      ;R18
  db  16+ (2*NUM_MEM_SLOTS)                   ;R18  len
else	;SMBIOS_V21_Support		      ;R18
  db  15+ (2*NUM_MEM_SLOTS)                   ;R13 len
endif	;SMBIOS_V21_Support		      ;R18
  dw  00h                                     ; handle
  db  MEM_ERROR_DETECTION                     ; error detection method
  db  MEM_ERROR_CORRECTION                    ; error correction capability
  db  MEM_INTERLEAVE                          ; supported interleave
  db  MEM_INTERLEAVE                          ; current interleave
  db  MEM_MAX_SIZE                            ; max mem module size
  dw  MEM_SUPPORTED_SPEEDS                    ; supported speeds
  dw  MEM_SUPPORTED_TYPES                     ; supported mem types
  db  MEM_SUPPORTED_VOLTAGE                   ; module voltage
  db  NUM_MEM_SLOTS                           ; number of associated slots
  dw  NUM_MEM_SLOTS dup (00h)                 ; Memory Module Handle
ifdef SMBIOS_V21_Support		      ;R18	
  db  0					      ;R18 Enabled Error Correcting Capabilities
endif	;SMBIOS_V21_Support		      ;R18	
;R13 type05_size  equ  $-offset type05_info + (2 * (NUM_MEM_SLOTS-1))   ; R05
type05_size  equ  $-offset type05_info		;R13


;=============================================================================
; TYPE 06 - Memory Module Information
;=============================================================================
type06_info label byte
  db  MEM_MOD_INFORMATION                     ; type
  db  0Ch                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; socket designation index
  db  00h                                     ; bank connections
  db  MEM_SPEEDS_UNKNOWN                      ; current speed
  dw  MEM_TYPES_UNKNOWN                       ; current memory type
  db  MEM_SIZE_NOT_DETERMINABLE               ; installed size
  db  00h                                     ; enabled size
  db  00h                                     ; error status
  db  'A1',0                                  ; socket designation
  db  0                                       ; string terminator
type06_size  equ  $-offset type06_info

;=============================================================================
; TYPE 7 - Internal Cache Information
;=============================================================================
type07_l1_info label byte
  db  CACHE_INFORMATION                       ; type
ifdef SMBIOS_V21_Support		      ;R18	
  db  13h                                     ;R18 len
else	;SMBIOS_V21_Support		      ;R18	
  db  0Fh                                     ; len
endif	;SMBIOS_V21_Support		      ;R18	
  dw  00h                                     ; handle
  db  01h                                     ; socket designation string index
  dw  TYPE07_CACHE_CFG_INTERNAL               ; cache config (fill at POST) R05
  dw  PENTIUM_INTERNAL_CACHE_SIZE             ; max cache size
  dw  PENTIUM_INTERNAL_CACHE_SIZE             ; installed size
  dw  SRAM_SYNCRHONOUS                        ; supported SRAM type (Get from BIOS.CFG)
  dw  SRAM_SYNCRHONOUS                        ; current SRAM type (Fill at POST) - R05
ifdef SMBIOS_V21_Support		      ;R18	
  db  0 				      ;R18 cache speed
  db  2					      ;R18 error correction type
  db  2					      ;R18 system cache type
  db  2					      ;R18 The associaivity of the cache
endif	;SMBIOS_V21_Support		      ;R18	
  db  'Internal Cache',0                      ; reference designation
  db  0                                       ; string terminator
type07_l1_size  equ  $-offset type07_l1_info

ifdef DMI_L2_ENABLED
;=============================================================================
; TYPE 7 - External Cache Information
;=============================================================================
type07_l2_info label byte
  db  CACHE_INFORMATION                       ; type
ifdef SMBIOS_V21_Support		      ;R18	
  db  13h                                     ;R18 len
else	;SMBIOS_V21_Support		      ;R18	
  db  0Fh                                     ; len
endif	;SMBIOS_V21_Support		      ;R18
  dw  00h                                     ; handle
  db  01h                                     ; socket designation string index
  dw  TYPE07_CACHE_CFG_EXTERNAL               ; cache config (fill at POST) R05
  dw  MAX_L2_SIZE                             ; max cache size
  dw  0000h                                   ; installed size
  dw  SRAM_SYNCRHONOUS                        ; supported SRAM type (Get from BIOS.CFG)
  dw  SRAM_SYNCRHONOUS                        ; current SRAM type (Fill at POST) R05
ifdef SMBIOS_V21_Support		      ;R18	
  db  0 				      ;R18 cache speed
  db  2					      ;R18 error correction type
  db  2					      ;R18 system cache type
  db  2					      ;R18 The associaivity of the cache
endif	;SMBIOS_V21_Support		      ;R18
  db  'External Cache',0                      ; reference designation
  db  0                                       ; string terminator
type07_l2_size  equ  $-offset type07_l2_info
endif

ifdef DMI_ONBOARD_IDE1
;=============================================================================
; TYPE 8 - Port Information - Onboard IDE
;=============================================================================
type08_ide1_info label byte
  db  PORT_CON_INFORMATION                    ; type
  db  09h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; internal ref string index
  db  CT_OB_IDE                               ; internal connector type
  db  02h                                     ; external ref string index
  db  CT_NONE                                 ; External connector type
  db  TYPE08_FUNC_OTHER                       ; Function of port
  db  'PRIMARY IDE',0                         ; internal reference
  db  0                                       ; external reference
  db  0                                       ; string terminator
type08_ide1_size  equ  $-offset type08_ide1_info
endif ;DMI_ONBOARD_IDE1

ifdef DMI_ONBOARD_IDE2
;=============================================================================
; TYPE 8 - Port Information - Onboard Secondary IDE
;=============================================================================
type08_ide2_info label byte
  db  PORT_CON_INFORMATION                    ; type
  db  09h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; internal ref string index
  db  CT_OB_IDE                               ; internal connector type
  db  02h                                     ; external ref string index
  db  CT_NONE                                 ; External connector type
  db  TYPE08_FUNC_OTHER                       ; Function of port
  db  'SECONDARY IDE',0                         ; internal reference
  db  0                                       ; external reference
  db  0                                       ; string terminator
type08_ide2_size  equ  $-offset type08_ide2_info
endif ; DMI_ONBOARD_IDE2

ifdef DMI_ONBOARD_COM1
;=============================================================================
; TYPE 8 - Port Information - Onboard COM 1
;=============================================================================
type08_com1_info label byte
  db  PORT_CON_INFORMATION                    ; type
  db  09h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; internal ref string index
  db  CT_9P_DIAL_INLINE                       ; internal connector type
  db  02h                                     ; external ref string index
  db  CT_DB9_MALE                             ; External connector type
  db  TYPE08_FUNC_SER_16450                   ; Function of port
  db  'COM1',0                                ; internal reference
  db  0                                       ; external reference
  db  0                                       ; string terminator
type08_com1_size  equ  $-offset type08_com1_info
endif ; DMI_ONBOARD_COM1

ifdef DMI_ONBOARD_COM2
;=============================================================================
; TYPE 8 - Port Information - Onboard COM 2
;=============================================================================
type08_com2_info label byte
  db  PORT_CON_INFORMATION                    ; type
  db  09h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; internal ref string index
  db  CT_9P_DIAL_INLINE                       ; internal connector type
  db  02h                                     ; external ref string index
  db  CT_DB9_MALE                             ; External connector type
  db  TYPE08_FUNC_SER_16450                   ; Function of port
  db  'COM2',0                         ; internal reference
  db  0                                       ; external reference
  db  0                                       ; string terminator
type08_com2_size  equ  $-offset type08_com2_info
endif ; DMI_ONBOARD_COM2

ifdef DMI_ONBOARD_FDD
;=============================================================================
; TYPE 8 - Port Information - Onboard FDD
;=============================================================================
type08_fdd_info label byte
  db  PORT_CON_INFORMATION                    ; type
  db  09h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; internal ref string index
  db  CT_OB_FLOPPY                            ; internal connector type
  db  02h                                     ; external ref string index
  db  CT_NONE                                 ; External connector type
  db  TYPE08_FUNC_8251_FIFO                   ; Function of port
  db  'FDD',0                                 ; internal reference
  db  0                                       ; external reference
  db  0                                       ; string terminator
type08_fdd_size  equ  $-offset type08_fdd_info
endif ; DMI_ONBOARD_FDD

ifdef DMI_ONBOARD_LPT1
;=============================================================================
; TYPE 8 - Port Information - Onboard LPT 1
;=============================================================================
type08_lpt1_info label byte
  db  PORT_CON_INFORMATION                    ; type
  db  09h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; internal ref string index
  db  CT_DB25_FEMALE                          ; internal connector type
  db  02h                                     ; external ref string index
  db  CT_DB25_FEMALE                          ; External connector type
  db  TYPE08_FUNC_PAR_ECP_EPP                 ; Function of port
  db  'LPT1',0                                ; internal reference
  db  0                                       ; external reference
  db  0                                       ; string terminator
type08_lpt1_size  equ  $-offset type08_lpt1_info
endif ; DMI_ONBOARD_LPT1

ifdef DMI_ONBOARD_KEYB
;=============================================================================
; TYPE 8 - Port Information - Keyboard
;=============================================================================
type08_keyb_info label byte
  db  PORT_CON_INFORMATION                    ; type
  db  09h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; internal ref string index
  db  CT_OTHER                                ; internal connector type
  db  02h                                     ; external ref string index
ifdef DMI_KEYBOARD_PS2
  db  CT_PS_2                                 ; External connector type
else
  db  CT_OTHER                                ; External connector type
endif
  db  TYPE08_FUNC_KEYBOARD                    ; Function of port
  db  'Keyboard',0                            ; internal reference
  db  0                                       ; external reference
  db  0                                       ; string terminator
type08_keyb_size  equ  $-offset type08_keyb_info
endif ; DMI_ONBOARD_KEYB

ifdef DMI_PS2M_ENABLED
;=============================================================================
; TYPE 8 - Port Information - PS/2 Mouse
;=============================================================================
type08_ps2m_info label byte
  db  PORT_CON_INFORMATION                    ; type
  db  09h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; internal ref string index
  db  CT_NONE                                 ; internal connector type
  db  02h                                     ; external ref string index
  db  CT_PS_2                                 ; External connector type
  db  TYPE08_FUNC_MOUSE                       ; Function of port
  db  'PS/2 Mouse',0                          ; internal reference
  db  0                                       ; external reference
  db  0                                       ; string terminator
type08_ps2m_size  equ  $-offset type08_ps2m_info
endif ; DMI_PS2M_ENABLED

ifdef DMI_USB_ENABLED
;=============================================================================
; TYPE 8 - Port Information - PS/2 Mouse
;=============================================================================
type08_usb_info label byte
  db  PORT_CON_INFORMATION                    ; type
  db  09h                                     ; len
  dw  00h                                     ; handle
  db  01h                                     ; internal ref string index
  db  CT_NONE                                 ; internal connector type
  db  02h                                     ; external ref string index
  db  CT_OTHER                                  ; External connector type
  db  TYPE08_FUNC_USB                         ; Function of port
  db  'USB',0                                 ; internal reference
  db  0                                       ; external reference
  db  0                                       ; string terminator
type08_usb_size  equ  $-offset type08_usb_info
endif ; DMI_USB_ENABLED


ifdef DMI_ISA_ENABLED
;=============================================================================
; TYPE 9 - System Slot Information (ISA) (One for each slot)
;=============================================================================
type09_isa_info label byte
  db  SYS_SLOTS_INFORMATION                   ; type
ifdef SMBIOS_V21_Support		      ;R18	
  db  0dh                                     ;R18 len
else	;SMBIOS_V21_Support		      ;R18
  db  0Ch                                     ; len
endif	;SMBIOS_V21_Support		      ;R18
  dw  00h                                     ; handle
  db  01h                                     ; slot design string
  db  TYPE09_SLOT_TYPE_ISA                    ; slot type - isa
  db  TYPE09_16BIT                            ; slot dbw - 16 bits
  db  02h                                     ; current usage - unknown
  db  TYPE09_LEN_FULL                         ; slot length - unknown
  dw  00h                                     ; slot id - 0
  db  02h                                     ; slot char - 5v
ifdef SMBIOS_V21_Support		      ;R18	
  db  0					      ;R18 PCI siot support Power Management Disable
endif	;SMBIOS_V21_Support		      ;R18
  db  'ISA',0
  db  0                                       ; string terminator
type09_isa_size  equ  $-offset type09_isa_info
endif ; DMI_ISA_ENABLED


ifdef DMI_PCI_ENABLED
;=============================================================================
; TYPE 9 - System Slot Information (PCI) (One for each slot)
;=============================================================================
type09_pci_info label byte
  db  SYS_SLOTS_INFORMATION                   ; type
ifdef SMBIOS_V21_Support		      ;R18	
  db  0dh                                     ;R18 len
else	;SMBIOS_V21_Support		      ;R18
  db  0Ch                                     ; len
endif	;SMBIOS_V21_Support		      ;R18
  dw  00h                                     ; handle
  db  01h                                     ; slot design string
  db  TYPE09_SLOT_TYPE_PCI                    ; slot type - pci
;R17  db  TYPE09_64BIT                            ; slot 64 bits
  db  TYPE09_32BIT                            ;R17 slot 32 bits
  db  02h                                     ; current usage - unknown
  db  TYPE09_LEN_FULL                         ; slot length - unknown
  dw  0000h                                   ; slot id - 0
  db  TYPE09_VOLTS_5                          ; slot char - 5v
ifdef SMBIOS_V21_Support		      ;R18
  db  0					      ;R18 PCI siot support Power Management Disable
endif	;SMBIOS_V21_Support		      ;R18
  db  'PCI',0
  db  0                                       ; string terminator
type09_pci_size  equ  $-offset type09_pci_info
endif

;;;R11 start
ifdef NUM_ONBOARD_DEVS 

STRING_INDEX  = 1
;=============================================================================
; TYPE 10 - Onboard Information
;=============================================================================
type10_info label byte
  db  ONBOARD_INFORMATION           ; type
  db  4+(NUM_ONBOARD_DEVS*2)        ; structure length
  dw  00h                           ; handle (filled in at POST)

ifdef DMI_ONBOARD_OTHER             ; device type other
  db  ONBOARD_OTHER
  db  STRING_INDEX
  STRING_INDEX = STRING_INDEX + 1
endif

ifdef DMI_ONBOARD_VIDEO             ; Video
  db  ONBOARD_VIDEO
  db  STRING_INDEX
  STRING_INDEX = STRING_INDEX + 1
endif

ifdef DMI_ONBOARD_SCSI              ; Onboard SCSI
  db  ONBOARD_SCSI
  db  STRING_INDEX
  STRING_INDEX = STRING_INDEX + 1
endif

ifdef DMI_ONBOARD_ETHERNET          ; Onboard Ethernet
  db  ONBOARD_ETHERNET
  db  STRING_INDEX
  STRING_INDEX = STRING_INDEX + 1
endif

ifdef DMI_ONBOARD_TOKEN_RING        ; Onboard Token Ring
  db  ONBOARD_TOKEN_RING
  db  STRING_INDEX
  STRING_INDEX = STRING_INDEX + 1
endif

ifdef DMI_ONBOARD_SOUND             ; Onboard Sound
  db  ONBOARD_SOUND
  db  STRING_INDEX
  STRING_INDEX = STRING_INDEX + 1
endif

ifdef DMI_ONBOARD_UNKNOWN             ; device type other
  db  ONBOARD_UNKNOWN
  db  STRING_INDEX
  STRING_INDEX = STRING_INDEX + 1
endif

ifdef DMI_ONBOARD_OTHER             ; device type other
  db  onboard_other_str
endif

ifdef DMI_ONBOARD_VIDEO             ; Video
  db  onboard_video_str
endif

ifdef DMI_ONBOARD_SCSI              ; Onboard SCSI
  db  onboard_scsi_str
endif

ifdef DMI_ONBOARD_ETHERNET          ; Onboard Ethernet
  db  onboard_ethernet_str
endif

ifdef DMI_ONBOARD_TOKEN_RING        ; Onboard Token Ring
  db  onboard_token_ring_str
endif

ifdef DMI_ONBOARD_SOUND             ; Onboard Sound
  db  onboard_sound_str
endif

  db  0                                       ; string terminator

type10_size  equ  $-offset type10_info
endif

;R11 end

;R16 start
;=============================================================================
; TYPE 15 - System Event Log
;=============================================================================
type15_info label byte
	db	SYSTEM_EVENT_LOG	; type
	db	14h			; len
	dw	00h			; handle
	dw	0h			; log area length
	dw	0h			; log header start offset
	dw	0h			; log data start offset
	db	4			; access method
	db	3			; log status
	dd	0			; log change token
	dd	0			; access method address
type15_size  equ  $-offset type15_info
;R16 end

;=============================================================================
; DMI struct table
;
;    Bios information
;    System information
;    Base board information
;    Enclosure information
;    Cpu information
;    Memory controller information
;    Memory module information
;    Cache information
;    Port connector information
;    Slot information
;    Onboard information
;
;=============================================================================


dmi_tbl  label dmi_entry
;---------------------------------------------------------------------------------------------
;         Num            Structure        Structure Size      Build Function   Update Function
;---------------------------------------------------------------------------------------------
ifdef BIOS_INFO_ENABLE
;R12 dmi_entry {1,            type00_info,      type00_size,       build_type00,    0}
dmi_entry {1,            type00_info,      type00_size,       build_type00,    update_type00}	;R12
endif

ifdef SYSTEM_INFO_ENABLE
dmi_entry {1,            type01_info,      type01_size,       0,               0}
endif

ifdef MB_INFO_ENABLE
dmi_entry {1,            type02_info,      type02_size,       0,               0}
endif

ifdef ENCLOSURE_INFO_ENABLE
dmi_entry {1,            type03_info,      type03_size,       0,               0}
endif


ifdef DMI_MEM_CTLR_ENABLED
dmi_entry {1,            type05_info,      type05_size,       build_type05,    update_type05}
endif

ifdef DMI_MEM_MOD_ENABLED
dmi_entry {NUM_MEM_SLOTS,type06_info,      type06_size,       build_type06,    update_type06}
endif

ifdef DMI_L1_ENABLED
dmi_entry {1,            type07_l1_info,   type07_l1_size,    0,               update_type07_l1}
endif

ifdef DMI_L2_ENABLED
dmi_entry {1,            type07_l2_info,   type07_l2_size,    0,               update_type07_l2}
endif

ifdef DMI_ONBOARD_IDE1
dmi_entry {1,            type08_ide1_info, type08_ide1_size,  0,               0}
endif

ifdef DMI_ONBOARD_IDE2
dmi_entry {1,            type08_ide2_info, type08_ide2_size,  0,               0}
endif

ifdef DMI_ONBOARD_FDD
dmi_entry {1,            type08_fdd_info,  type08_fdd_size,   0,               0}
endif

ifdef DMI_ONBOARD_COM1
dmi_entry {1,            type08_com1_info, type08_com1_size,  0,               0}
endif

ifdef DMI_ONBOARD_COM2
dmi_entry {1,            type08_com2_info, type08_com2_size,  0,               0}
endif

ifdef DMI_ONBOARD_LPT1
dmi_entry {1,            type08_lpt1_info, type08_lpt1_size,  0,               0}
endif

ifdef DMI_ONBOARD_KEYB
dmi_entry {1,            type08_keyb_info, type08_keyb_size,  0,               0}
endif

ifdef DMI_PS2M_ENABLED
dmi_entry {1,            type08_ps2m_info, type08_ps2m_size,  0,               0}
endif

ifdef DMI_ISA_ENABLED
dmi_entry {NUM_ISA,      type09_isa_info,  type09_isa_size,   0,               0}
endif

ifdef DMI_PCI_ENABLED
dmi_entry {NUM_PCI,      type09_pci_info,  type09_pci_size,   0,               update_type09_pci}
endif

;R11 start
ifdef NUM_ONBOARD_DEVS GE 1
dmi_entry {1,            type10_info,      type10_size,        0,              0}
endif

ifdef DMI_USB_ENABLED
dmi_entry {1,            type08_usb_info, type08_usb_size,  0,               0}
endif

;R11 end

;R11 start
;ifdef DMI_ONBOARD
;dmi_entry {0,            type10_info,     type10_size,        0,               0}
;endif
;R11 end

; R04
; NOTE CPU SUPPORT SHOULD ALWAYS BE LAST AS THIS IS THE ONLY UPDATE CODE
; WHICH IS SET TO ACTUALLY REFLASH THE BIOS IN ORDER TO SPEED UP THE BOOT
; PROCESS
;R11  dmi_entry {1,            type04_info,      type04_size,       0,               update_type04}
dmi_entry {NUM_CPU,      type04_info,      type04_size,       0,               update_type04}    ;R11

dmi_entry {1,            type15_info,     type15_size,        0,               0}	;R16

dmi_entry {0,            0FFFFh,          0FFFFh,             0,               0}
;---------------------------------------------------------------------------------------------


;---------------------------------------------------------------------------------------------
mem_bank_connection_table label byte
  db  MEM_BANK_CONNECT01            ; SIMM 01 banks
  db  MEM_BANK_CONNECT02            ; SIMM 02 banks
  db  MEM_BANK_CONNECT03            ; SIMM 03 banks
  db  MEM_BANK_CONNECT04            ; SIMM 04 banks

;R11 start
if NUM_MEM_SLOTS GE 6
  db  MEM_BANK_CONNECT05            ; SIMM 05 banks
  db  MEM_BANK_CONNECT06            ; SIMM 06 banks
endif
if NUM_MEM_SLOTS GE 8
  db  MEM_BANK_CONNECT07            ; SIMM 07 banks
  db  MEM_BANK_CONNECT08            ; SIMM 08 banks
endif

;R11 end


;=============================================================================
; Slot Device Number Equates.
;=============================================================================
slot_id_tbl label byte
;R09  db (SLOT1_ID SHL 3)       ; (upper 5 bits)
;R09  db (SLOT2_ID SHL 3)       ; (upper 5 bits)
  db SLOT1_ID       		; R09
  db SLOT2_ID        		; R09
ifndef TWO_PCI_SLOT_ONLY
;R09  db (SLOT3_ID SHL 3)       ; (upper 5 bits)
  db SLOT3_ID               ; R09
ifndef THREE_PCI_SLOT_ONLY
;R09  db (SLOT4_ID SHL 3)       ; (upper 5 bits)
  db SLOT4_ID               ; R09
endif ;THREE_PCI_SLOT_ONLY
endif ;TWO_PCI_SLOT_ONLY

ifdef	FIVE_PCI_SLOTS				;R15
	db	SLOT5_ID			;R15
endif;	FIVE_PCI_SLOTS				;R15			


;=============================================================================
; Memory string table
;=============================================================================
mem_string_table label byte
  dw mem_module_str1
  dw mem_module_str2
  dw mem_module_str3
  dw mem_module_str4

;R11 start
if NUM_MEM_SLOTS GE 6               
  dw mem_module_str5
  dw mem_module_str6
endif

if NUM_MEM_SLOTS GE 8               
  dw mem_module_str7
  dw mem_module_str8
endif

;R11 end

  dw 0
ifdef	VT586		;VIA chipset	;R14 - start
mem_module_str1  db  'BANK_0',0
mem_module_str2  db  'BANK_1',0
mem_module_str3  db  'BANK_2',0
mem_module_str4  db  'BANK_3',0

if NUM_MEM_SLOTS GE 6               
mem_module_str5  db  'BANK_4',0
mem_module_str6  db  'BANK_5',0
endif

if NUM_MEM_SLOTS GE 8               
mem_module_str7  db  'BANK_6',0
mem_module_str8  db  'BANK_7',0
endif
else;	VT586				;R14 - end
mem_module_str1  db  'A0',0
mem_module_str2  db  'A1',0
mem_module_str3  db  'A2',0
mem_module_str4  db  'A3',0

;R11 start
if NUM_MEM_SLOTS GE 6               
mem_module_str5  db  'A4',0
mem_module_str6  db  'A5',0
endif

if NUM_MEM_SLOTS GE 8               
mem_module_str7  db  'A6',0
mem_module_str8  db  'A7',0
endif
endif;	VT586				;R14
 

;R11 end

