;	[]===========================================================[]
;
;	NOTICE: THIS PROGRAM BELONGS TO AWARD SOFTWARE INTERNATIONAL(R)
;	        INC. IT IS CONSIDERED A TRADE SECRET AND IS NOT TO BE 	
;	        DIVULGED OR USED BY PARTIES WHO HAVE NOT RECEIVED	
;	        WRITTEN AUTHORIZATION FROM THE OWNER.
;
; 	[]===========================================================[]
;

;----------------------------------------------------------------------------
;Rev  	Date	  Name	Description
;----------------------------------------------------------------------------
;R04	10/23/97  RCH	Added Pentium II L2 cache ECC controlled by user
;			CMOS setup.
;R03	07/02/97  RCH	Update L2 cache sizing with revision 2.6
;			The old file is backuped as P6L2CACH.702
;R02	04/18/97  RCH	Update Pentium II L2 cache sizing for both klamath &
;			Deschutes CPU. The old file is backuped as P6L2CACH.418
;R00	--------  ----	Initial revision from Intel

;R03 - start

;---------------------------------------------------------------------------
; Processor ID(s).
;---------------------------------------------------------------------------

PENTIUM_II_PROCESSOR    EQU     0630h   ; Pentium(r) II Family/Model/Stepping.

;---------------------------------------------------------------------------
; Pentium(r) Pro/II Processor Power-on Configuration MSR.
;---------------------------------------------------------------------------

EBL_CR_POWERON          EQU     2Ah     ; Power-on configuration register.
                                                
;---------------------------------------------------------------------------
; L2 Controller Registers.
;---------------------------------------------------------------------------

L2_CR0                  EQU     00h     ; L2 controller device/stepping id.
L2_CR1                  EQU     01h     ; L2 controller manufacturer id.
L2_CR2                  EQU     02h     ; L2 controller cache size.
L2_CR3                  EQU     03h     ; L2 controler physical cacheable
                                        ; address space.
L2_CR4                  EQU     04h     ; L2 controller latency mode.
L2_CR5                  EQU     05h     ; L2 controller error type.
L2_CR6                  EQU     06h     ; L2 controller associativity.
L2_CR7                  EQU     07h     ; Reserved.
L2_CR8                  EQU     08h     ; L2 controller timing.

;---------------------------------------------------------------------------
; Pentium(r) Pro/II Processor L2 Cache Interface MSR's.
;---------------------------------------------------------------------------

BBL_CR_D0               EQU     88h     ; L2 chunk 0 data low.
BBL_CR_D1               EQU     89h     ; L2 chunk 1 data low.
BBL_CR_D2               EQU     8Ah     ; L2 chunk 2 data low.
BBL_CR_D3               EQU     8Bh     ; L2 chunk 3 data low.
BBL_CR_DECC             EQU     118h    ; L2 data ECC.
BBL_CR_ADDR             EQU     116h    ; L2 address.
BBL_CR_TRIG             EQU     11Ah    ; L2 trigger.
BBL_CR_BUSY             EQU     11Bh    ; L2 busy.
BBL_CR_CTL              EQU     119h    ; L2 control.
BBL_CR_CTL3             EQU     11Eh    ; L2 control 3.
BBL_CR_OVRD             EQU     17h     ; L2 cache latency override.

;---------------------------------------------------------------------------
; L2 Commands - Used in CTL0 [4:1].
;---------------------------------------------------------------------------

L2_CMD_RLU              EQU     00001100b ; Data read with LRU update.
L2_CMD_TRR              EQU     00001110b ; Tag read with data read.
L2_CMD_TWR              EQU     00001000b ; Tag write with data read.
L2_CMD_TWW              EQU     00011100b ; Tag write with data write.
L2_CMD_TW               EQU     00010000b ; Tag write.
L2_CMD_TR               EQU     00001111b ; Tag read.
L2_CMD_CR               EQU     00000010b ; L2 controller read.
L2_CMD_CW               EQU     00000011b ; L2 controller write.

;---------------------------------------------------------------------------
; L2 Outbound Way Bits, CTL0 [9:8].
;---------------------------------------------------------------------------

L2_WAY0                 EQU     00b     ; Way 0
L2_WAY1                 EQU     01b     ; Way 1
L2_WAY2                 EQU     10b     ; Way 2
L2_WAY3                 EQU     11b     ; Way 3

;---------------------------------------------------------------------------
; Control Parameters for the Cache Configure Code.
;---------------------------------------------------------------------------

BUSY_WAIT_COUNT         EQU     100h           ; Watchdog for cache bus.
BUSY_BIT_SET            EQU     1h             ; Still busy value.
HIGH_32_PATTERN_0       EQU     0aaaaaaaah     ; A data pattern.
LOW_32_PATTERN_0        EQU     0aaaaaaaah     ; A data pattern.
HIGH_32_PATTERN_ALIAS   EQU     055555555h     ; A data pattern.
LOW_32_PATTERN_ALIAS    EQU     055555555h     ; A data pattern.
MAX_CACHE_SIZE          EQU     8d*1024d*1024d ; Max cache size to look for.
MAX_L2_CONTROLLERS      EQU     4h             ; Maximum possible L2 cache
                                               ; controllers.
CSRAM_CONTROLLER_MSK    EQU     1800h          ; Mask for CSRAM L2 cache
                                               ; controllers (maximum of 4
                                               ; CSRAM controllers).
BSRAM_CONTROLLER_MSK    EQU     20000h         ; Mask for BSRAM L2 cache
                                               ; controllers (maximum of 2
                                               ; BSRAM controllers).

.XLIST
.LIST

;****************************************************************************
;                                   REV 2.6
;****************************************************************************
;
; Rev   Description
; ---   -------------------------------------------------------------
; 1.0   Initial Pentium(r) II processor release.
; 2.1   Fixes for Pentium(r) II processor CSER/ECC.
;       Adds Deschutes support.
; 2.2   Fix for CommandL2 bug - outbound way not copied into CTL.
; 2.3   Changed MTRR initialization recommendation.  No code changes.
; 2.4   Changed sense of 100/66# bit in EBL_CR_POWERON MSR.  Changed latency
;       tables.
; 2.5   Major revision for general CSRAM/BSRAM approach.
;       Fixed cache disable code - added wbinvd before clearing CD in CR0.
;       Added L2 code version signature.
;       Added CF flag to indicate errors and removed port 80h error loops.
;       Added support for safe config bypass.
;       Changed BSRAM 266MHz CR8 value to 0h.
;       Changed BBL_CR_LATENCY register address from 1Eh to 17h.
;       Changed EBL_POWERON ratio bits.
;       Renamed SetProcessorLatency to SetL2CacheLatency (more descriptive).
;       Renamed SetCacheLatency to SetL2LatencyMode (more descriptive).
;       Changed BBL_CR_LATENCY to BBL_CR_OVRD (more descriptive).
;       Changed compile option to 586P and some manual opcode DB's to ASM
;       instructions.
;       Replace ReadMSR, WriteMSR, and ChangeMSR macros with inline code.
; 2.6   Combine L2 disable checks.
;       Fixed masking out of new L2 cache latency.
;       Fixed masking out of default latency mode in SetL2LatencyMode.
;       A few optimizations.
;
;****************************************************************************
;                     Copyright (c) 1995-1997 Intel Corp.
;****************************************************************************
                             
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; This code provides L2 cache configuration, initialization, and sizing for
; the Pentium(r) II processor and future processors.  For execution time
; optimization this code enables the L1 cache, which require the MTRRs to be
; initialized.  The BIOS may either 1) initialize the appropriate F segment
; Fixed-Range MTRRs (Write Protect), enable the MTRRs through the Fixed/Global
; enable bit in the MTRRdefType MTRR, execute this routine, disable the MTRRs
; and execute the overall system MTRR initialization later or,
; 2) execute the overall system MTRR initialization first if the appropriate
; F segment is cached such that the exectution time reduction is achieved.
; Also, care must be taken to make sure that this code runs on all processors,
; if the system is Symmetric MultiProcessing (SMP) capable. 
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;---------------------------------------------------------------------------
; Assembler options
;---------------------------------------------------------------------------

;.386P
        
		public	ConfigurePProL2Cache 
ConfigurePProL2Cache 	proc	near

        pushad                          ; Save registers.

;       jmp     CheckGeniuneIntel       ; Jump over signature.

;---------------------------------------------------------------------------
; Intel Reference BIOS Version Signature.
;---------------------------------------------------------------------------

;       db      "_L2_", 26h             ; Signature to indicate which version
                                        ; of the INTEL REFERENCE BIOS CODE
                                        ; this L2 configuration routine is
                                        ; based on.  PLEASE keep this
                                        ; signature in your code.

;---------------------------------------------------------------------------
; Check for "GeniuneIntel" processor.
;---------------------------------------------------------------------------

;CheckGeniuneIntel:
;       xor     eax,eax                 ; CPUID signature parameter.
;       db      0fh,0a2h                ; Execute CPUID instruction.
;       cmp     ebx,756e6547h           ; Check signature "Genu".
;       jnz     ConfigureL2CacheFail    ; If not, return error.
;       cmp     edx,49656e69h           ; Check signature "ineI".
;       jnz     ConfigureL2CacheFail    ; If not, return error.
;       cmp     ecx,6c65746eh           ; Check signature "ntel".
;       jnz     ConfigureL2CacheFail    ; If not, return error.

;---------------------------------------------------------------------------
; Get feature bits now and check for processor type (family/model).
;---------------------------------------------------------------------------

CheckProcessorType:
        mov     eax,1h                  ; CPUID version/feature parameter.
        db      0fh,0a2h                ; Execute CPUID instruction.
        and     ax,0fff0h               ; Mask off stepping.
        test    ax,01000h               ; Check Overdrive bit.
        jnz     ConfigureL2CacheDone    ; If it's an Overdrive processor,
                                        ; we're done, no error returned.
        cmp     ax,PENTIUM_II_PROCESSOR ; Check for Pentium(r) II processor.
        jl      ConfigureL2CacheDone    ; It's a predecessor, which doesn't
                                        ; need L2 config, so we're done,
                                        ; again no error.
        mov     bx,ax                   ; Save CPU result for after L2
                                        ; disable check.

;---------------------------------------------------------------------------
; We now know it's at least a Pentium(r) II processors or greater, so it has
; an L2 cache.  Check to see if the L2 cache is disabled via hardware.  If
; it is, we don't need to do any L2 initialization.  This is an optimization
; change from the last release/BWG's.
;---------------------------------------------------------------------------

        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        and     eax,00800000h           ; CTL3, hardware disable bit 23.
        jnz     ConfigureL2CacheDone    ; Exit if disabled.

;---------------------------------------------------------------------------
; L2 Initialization for Pentium(r) II processors and future processors is
; different.  Handle accordingly.
;---------------------------------------------------------------------------

        cmp     bx,PENTIUM_II_PROCESSOR ; Check for Pentium(r) II processor.
        jg      BeyondPentiumIIProcessor; Handle processors after the 
                                        ; Pentium(r) II differently.

;---------------------------------------------------------------------------
; Initialize Pentium(r) II processor L2 cache.
;---------------------------------------------------------------------------

PentiumIIProcessor:

;---------------------------------------------------------------------------
; Check for safe config in EBL_CR_POWERON ratio bits [24:22].  A ratio of
; EBL_CR_POWERON [24:22] = 011b or 100b tells us to use the safe config.
;---------------------------------------------------------------------------

        mov     ecx,EBL_CR_POWERON      ; Read power on configuration MSR.
        rdmsr
        and     eax,01C00000h           ; We're interested in ratio only.
        cmp     eax,00C00000h           ; Check for ratio 011b.
        je      ConfigureL2CacheFail    ; If equal, it's a safe config and
                                        ; bypass L2 config and signal error.
        cmp     eax,01000000h           ; Check for ratio 100b.
        je      ConfigureL2CacheFail    ; If equal, it's a safe config and
                                        ; bypass L2 config and signal error.

;---------------------------------------------------------------------------
; If the BIOS would like to provide a setup option to disable the L2 cache,
; the code would be placed here.
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; Disable L2 cache first.  The L2 cache is disabled at the reset.  But, this
; step may be necessary, if control comes here from a soft reset.  This
; depends on your POST code sequence and organization.  The L2 cache is
; disabled by resetting CTL3 bit 8.  Also, we write CTL3 controlled L2
; features to default (excluding L2 cache latency) WITHOUT disturbing
; RESERVED or read only register bits.  We maintain the previous L2 cache
; latency since we don't know whether we got here from a hard or soft reset.
; If we got here from a soft reset, the L2 cache latency must remain the same
; otherwise the L2 cache latency and the L2 latency mode programmed into the
; BSRAM/CSRAM controller could be incompatible.  In the case of a hard reset,
; the L2 cache latency will be the processor's default, so we're ok.
;---------------------------------------------------------------------------

        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        and     eax,0ff88061eh          ; Clear address space, disable CSER 
                                        ; checking, clear cache size,
                                        ; set 1 bank, disable L2, disable
                                        ; CRTN parity checking, disable
                                        ; victim address parity checking,
                                        ; disable ECC, and L2 not configured.
        or      eax,000044000h          ; Enable CSER checking and set 512KB.
        wrmsr

;---------------------------------------------------------------------------
; Since Pentium(r) II processor only has one l2 cache latency setting (which
; it defaults to), we don't need to do anything else here.  Also, since
; Pentium(r) II processor L2 cache controllers do not require any changes to
; their default latency mode, bypass the latency mode code and jump ahead
; to special handling for L2 controller types.
;---------------------------------------------------------------------------

        jmp     CheckL2ControllerType   ; Skip ahead to L2 device handling.

;---------------------------------------------------------------------------
; Initialize Future Processor L2 caches.
;---------------------------------------------------------------------------

BeyondPentiumIIProcessor:

;---------------------------------------------------------------------------
; Check for safe config in EBL_CR_POWERON ratio bits [25:22].  A ratio of
; EBL_CR_POWERON [25:22] = 0011b or 1100b tells us to use the safe config.
;---------------------------------------------------------------------------

        mov     ecx,EBL_CR_POWERON      ; Read power on configuration MSR.
        rdmsr
        and     eax,03C00000h           ; We're interested in ratio only.
        cmp     eax,00C00000h           ; Check for ratio 0011b.
        je      ConfigureL2CacheFail    ; If equal, it's a safe config and
                                        ; bypass L2 config and signal error.
        cmp     eax,03000000h           ; Check for ratio 1100b.
        je      ConfigureL2CacheFail    ; If equal, it's a safe config and
                                        ; bypass L2 config and signal error.

;---------------------------------------------------------------------------
; If the BIOS would like to provide a setup option to disable the L2 cache,
; the code would be placed here.
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; Disable L2 cache first.  The L2 cache is disabled at the reset.  But, this
; step may be necessary, if control comes here from a soft reset.  This
; depends on your POST code sequence and organization.  The L2 cache is
; disabled by resetting CTL3 bit 8.  Also, we write CTL3 controlled L2
; features to default (excluding L2 cache latency) WITHOUT disturbing
; RESERVED or read only register bits.  We maintain the previous L2 cache
; latency since we don't know whether we got here from a hard or soft reset.
; If we got here from a soft reset, the L2 cache latency must remain the same
; otherwise the L2 cache latency and the L2 latency mode programmed into the
; BSRAM/CSRAM controller could be incompatible.  In the case of a hard reset,
; the L2 cache latency will be the processor's default, so we're ok.
;---------------------------------------------------------------------------

        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        and     eax,0ff88061eh          ; Clear address space, disable CSER 
                                        ; checking, clear cache size,
                                        ; set 1 bank, disable L2, disable
                                        ; CRTN parity checking, disable
                                        ; victim address parity checking,
                                        ; disable ECC, and L2 not configured.
        or      eax,000040000h          ; Enable CSER checking.
        wrmsr

        mov     bl,al                   ; Save original L2 cache latency.

;---------------------------------------------------------------------------
; Set L2 cache latency (CTL3) based on core speed.  This is only necessary
; for processors beyond the Pentium(r) II processor.
;---------------------------------------------------------------------------

        call    SetL2CacheLatency       ; Set L2 cache latency in CTL3.
        jc      ConfigureL2CacheFail    ; Check for cache/unknown speed error.

;---------------------------------------------------------------------------
; Set L2 controller latency mode.  For processors beyond the Pentium(r) II
; processor, once we set the proper L2 cache latency in CTL3 [4:1], the
; processor will return the correct L2 controller latency mode in 
; CTL3 [27:26] for that L2 cache latency.  We take this value and program it
; into the L2 control register 4 (L2_CR4).  However, in order to do this, we
; must first restore the original L2 cache latency in CTL3 [4:1], otherwise
; the write to L2 control register 4 (L2_CR4) may fail.  Then we will restore
; the new L2 cache latency in CTL3 [4:1].  Note, in the CSRAM case, the write
; to the L2 control register 4 (L2_CR4) will put the CSRAM in server timing
; mode.
;---------------------------------------------------------------------------

        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr                           ; Processors beyond the Pentium(r) II
                                        ; processor will return the latency
                                        ; mode setting in CTL3 [27:26] based
                                        ; upon the L2 cache latency setting
                                        ; programmed into CTL3 [4:1].  We
                                        ; copy these two bits [27:26] into
                                        ; CR4 bits [1:0].
        mov     bh,al                   ; Save new L2 cache latency.
        mov     al,bl                   ; Restore original L2 cache latency.
                                        ; Since we zeroed all but the cache
                                        ; latency bits of CTL3 [7:0] above,
                                        ; can safely assume bits 7-5 and 0
                                        ; are zero.
        wrmsr

        and     eax,0C000000h           ; Mask out latency mode from
                                        ; CTL3 [27:26].
        shr     eax,26d                 ; Align to bit 0 for SetL2LatencyMode.
        call    SetL2LatencyMode        ; Set all L2 cache controller latency
                                        ; modes.
        jc      ConfigureL2CacheFail    ; Check for cache error.

        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        mov     al,bh                   ; Restore new L2 cache latency.
                                        ; Since we zeroed all but the cache
                                        ; latency bits of CTL3 [7:0] above,
                                        ; can safely assume bits 7-5 and 0
                                        ; are zero.
        wrmsr

;---------------------------------------------------------------------------
; Special handling for the BSRAM/CSRAM devices.
;---------------------------------------------------------------------------

CheckL2ControllerType:

;---------------------------------------------------------------------------
; Read L2 cache control register CR0.  This contains device/stepping
; information.
;---------------------------------------------------------------------------

        mov     ecx,L2_CR0              ; L2 controller device/stepping id
                                        ; register.
        call    L2RegRead               ; Read device/stepping id register.
        jc      ConfigureL2CacheFail    ; Check for cache error.
        test    al,20h                  ; Bit 5 of this register indicates
                                        ; if we have a BSRAM or CSRAM device
                                        ; (a 1 is for CSRAM, a 0 for BSRAM).
        jnz     SetupCSRAM              ; Handle each device differently.

SetupBSRAM:
        jmp     CheckECCSupport

;---------------------------------------------------------------------------
; Since it's a CSRAM, it supports CRTN parity and victim address parity.
; Enable these features in CTL3, bits 7:6.
;---------------------------------------------------------------------------

SetupCSRAM:
        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        or      eax,0000000c0h          ; Enable CRTN and Victim address
        wrmsr                           ; parity, CTL3 [7:6].

;---------------------------------------------------------------------------
; Check for ECC support.  Write the ECC bits and see if they stick.  Enable
; ECC in CTL3 [5] if ECC is present.
;---------------------------------------------------------------------------

CheckECCSupport:
        call    CheckAndEnableECC       ; Enable ECC if supported.
        jc      ConfigureL2CacheFail    ; Check for cache error.

;---------------------------------------------------------------------------
; Read L2 cache control register CR3 for physical cacheable address space.
; Program this into CTL3 [22:20].
;---------------------------------------------------------------------------

        call    SetPhysicalCacheableRange ; Set physical cacheable range.
        jc      ConfigureL2CacheFail    ; Check for cache error.

;---------------------------------------------------------------------------
; Size the L2 cache.
;---------------------------------------------------------------------------

        call    SizeL2Cache             ; Size L2 cache.
        jc      ConfigureL2CacheFail    ; Check for cache error.

;---------------------------------------------------------------------------
; Now invalidate L2.  This is necessary for CSER and ECC support.
;---------------------------------------------------------------------------

InvalidateL2:

;---------------------------------------------------------------------------
; With all caching disabled, invalidating the entire L2 cache can take
; upwards of 20 seconds.  To speed this up, we enable caching via CR0 here.
; This is safe since the L2 enable bit in CTL3 (bit 8) is not set at this
; point.  Hence, only the L1 cache will be used.  We must also ensure that
; the MTRR's are setup to allow at least WT caching in the region this code
; lives.  Otherwise we've just done nothing.
;---------------------------------------------------------------------------

        mov     ebx,cr0                 ; EBX = Processor CR0.  Save in EBX
                                        ; for later restore.
        and     ebx,9fffffffh           ; Enable caching (CD=0, NW=0).
        mov     cr0,ebx                 ; Make it happen.

;---------------------------------------------------------------------------
; Determine L2 cache size from CTL3, bits 17:13.  Only 1 bank.
;---------------------------------------------------------------------------

        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        mov     ecx, eax                ; Use ECX as the size counter.
        and     ecx,0003E000h           ; Cache size only.
        shl     ecx,3h                  ; Convert to largest set address.
        mov     edx,0h                  ; Zero upper address.
        mov     edi,0h                  ; Zero upper data.
        mov     esi,0h                  ; Zero lower data.
        mov     al,L2_CMD_TWW           ; L2 write command.

InvalidateLoop:
        sub     ecx,20h                 ; Next L2 cache address.

        mov     ah,L2_WAY0              ; Way 0
        call    CommandL2               ; Write cache line.
        jc      ConfigureL2CacheFail    ; Check for cache error.

        mov     ah,L2_WAY1              ; Way 1
        call    CommandL2               ; Write cache line.
        jc      ConfigureL2CacheFail    ; Check for cache error.

        mov     ah,L2_WAY2              ; Way 2
        call    CommandL2               ; Write cache line.
        jc      ConfigureL2CacheFail    ; Check for cache error.

        mov     ah,L2_WAY3              ; Way 3
        call    CommandL2               ; Write cache line.
        jc      ConfigureL2CacheFail    ; Check for cache error.

        cmp     ecx,0h                  ; All of L2 written yet yet?
        jnz     InvalidateLoop          ; If not, continue.

;---------------------------------------------------------------------------
; Disable all caching now (i.e., L1).  It is very important to make sure
; all caching is disabled before enabling the L2 cache.
;---------------------------------------------------------------------------

        mov     eax,cr0                 ; Get CR0.
        or      eax,40000000h           ; Disable caching.
        mov     cr0,eax                 ; Make it happen.

        wbinvd                          ; Flush L1.

;---------------------------------------------------------------------------
; Set L2 configured in CTL3 [0].  Do an INVD to clear the L2 tags.  Finally,
; set L2 enabled in CTL3 [8].
;---------------------------------------------------------------------------

EnableL2:
        mov     ecx,BBL_CR_CTL3         ; Read L2 control register 3 (CTL3).
        rdmsr
        or      eax,000000001h          ; Set configured CTL3 [0].
        wrmsr
        invd                            ; Flush L2 tags.
        or      eax,000000101h          ; Set enabled CTL3 [8].
        wrmsr

;---------------------------------------------------------------------------
; Restore original caching state (i.e., CR0).
;---------------------------------------------------------------------------

        mov     cr0,ebx                 ; Original CR0 from above.

;---------------------------------------------------------------------------
; L2 initialization is either done or skipped.
;---------------------------------------------------------------------------

ConfigureL2CacheDone:

;R04 - start
ifdef	KLAMATH_CPU_ONLY
 ifdef	L2ECC_CMOS

	;Disable Pentium II's ECC if user select off in CMOS setup
	;and the CPU clock is below 333Mhz
	push	es			; save ES

	push	G_RAM			; Get BIOS data area
	pop	es

	test	es:[SYSTEM_FLAG],L2ECC_OFF;ECC allow to disable ?
	jz	short KeepEccEnabled	;no

        mov     ecx,BBL_CR_CTL3         ; MSR 011EH
        rdmsr
	and	al,NOT 20H		;disable ECC
	wrmsr

  KeepEccEnabled:

	pop	es			; restore ES
 endif;	L2ECC_CMOS
endif;	KLAMATH_CPU_ONLY
;R04 - end

        clc                             ; Clear CF to indicate success.
        jmp     ConfigureL2CacheExit    ; All done.

ConfigureL2CacheFail:
        stc                             ; Set CF to indicate error.

ConfigureL2CacheExit:
        popad                           ; Restore registers.
        ret                             ; Go home.

ConfigurePProL2Cache 	endp

;****************************************************************************
; CommandL2
;
; This procedure sends a Cache Interface Command to the L2 hardware.
;
; I/P = edx = Upper 32 bits of Address bits.
; I/P = ecx = Lower 32 bits of Address bits.
; I/P = edi = Upper 32 bits of Data pattern to be sent off.
; I/P = esi = Lower 32 bits of Data pattern to be sent off.
; I/P = al  = L2 transaction type (bits 4:0).
; I/P = ah  = Outbound way (bits 1:0).
; O/P       = Results will be in L2 cache MSRs.
; O/P = CY  = Set if L2 is permanently busy and is not accepting commands.
;
; Destroys no working register but for CY flag.
;
; Sends a Cache Interface command to L2 hardware and waits
; for the completion of the command. Returns completion status.
;****************************************************************************

CommandL2       PROC    NEAR

        pushad                          ; Save registers.
        mov     bp,sp                   ; Use register values off stack.

;----------------------------------------------------------------------------
; Step1: Set up 64 bit address.
;----------------------------------------------------------------------------

        mov     ecx,BBL_CR_ADDR         ; L2 Address register.      
        mov     eax,dword ptr [bp+18h]  ; Lower address.
        mov     edx,dword ptr [bp+14h]  ; Upper address.
        wrmsr

;----------------------------------------------------------------------------
; Step2: Set up Data pattern.
;----------------------------------------------------------------------------

        mov     eax,dword ptr [bp+4h]   ; Lower data pattern.
        mov     edx,dword ptr [bp]      ; Upper data pattern.
        mov     ecx,BBL_CR_D0           ; L2 Chunk 0 data register (D0).
        wrmsr
        inc     cx                      ; L2 Chunk 1 data register (D1).
        wrmsr
        inc     cx                      ; L2 Chunk 2 data register (D2).
        wrmsr
        inc     cx                      ; L2 Chunk 3 data register (D3).
        wrmsr

;----------------------------------------------------------------------------
; Step3: Set up Control Register0.  We must set/reset only command and
; outbound way bits of CTL register. We shall not disturb other bits.
;----------------------------------------------------------------------------

        mov     ecx,BBL_CR_CTL          ; L2 Control register.
        rdmsr
        xor     edx,edx                 ; No upper 32 bits here.
        and     ax,0fce0h               ; Remove previous command and
                                        ; outbound way.
        mov     bx,word ptr[bp+1ch]     ; Get L2 command and outbound way.
        or      ax,bx                   ; Add in new command and outbound way.
        wrmsr

;----------------------------------------------------------------------------
; Step4: Pull the trigger.
;----------------------------------------------------------------------------

        mov     ecx,BBL_CR_TRIG         ; L2 Trigger register to trigger
                                        ; command execution.
        xor     eax,eax                 ; No data.
        xor     edx,edx                 ; No data.
        wrmsr

;----------------------------------------------------------------------------
; Step5: Wait for L2 to execute command.  Poll the busy register, but use
; a software counter to prevent hanging.  Also, since (1) the time to execute
; an L2 command is a function of the processor's core speed, and (2) the
; time spent in the counter loop is a function of the processor's core speed,
; the counter loop is not processor speed sensitive (i.e. it will not break
; with faster future processors).
;----------------------------------------------------------------------------

        mov     bx,BUSY_WAIT_COUNT      ; Watchdog timer.

BusyLoop:
        mov     ecx,BBL_CR_BUSY         ; L2 Busy register.
        rdmsr
        test    al,BUSY_BIT_SET         ; Is it still busy?
        jz      DoneL2Command           ; If not, then we're done.
        dec     bx                      ; Decrement watchdog timer.
        jnz     short BusyLoop          ; Continue polling if watchdog hasn't expired.
                                                
;----------------------------------------------------------------------------
; Watchdog expired.  Set CF flag to return error.
;----------------------------------------------------------------------------

CacheInterfaceHang:
        stc                             ; Set error flag.
	  jmp	    CommandL2Exit           ; Exit.

;----------------------------------------------------------------------------
; All done.
;----------------------------------------------------------------------------

DoneL2Command:
	  clc                             ; Clear error flag.

CommandL2Exit:
        popad                           ; Restore registers.
        ret                             ; Go home.

CommandL2       ENDP

;****************************************************************************
; L2RegRead
;
; This procedure reads a control register in the L2 controller.  This
; procedure DOES NOT support multiple controllers (controller 0 is assumed).
;
; I/P = ECX = Address of the register being accessed.
; O/P = AL = 8 bit results.
; O/P = CY = Set if L2 is permanently busy and is not accepting commands.
;
; Destroys no working registers but for the ones used as O/P.
;
; Sends a CR command to L2 controller and waits for the completion of the
; command. Returns the read value.
;****************************************************************************

L2RegRead       PROC    NEAR

        pushad                          ; Save registers.
        mov     bp,sp                   ; Use register values off stack.

        xor     edx,edx                 ; Zero upper address.
        xor     edi,edi                 ; Zero upper data pattern.
        xor     esi,esi                 ; Zero lower data pattern.
        xor     eax,eax                 ; Zero command.
        shl     ecx,5h                  ; Align address to ADDR [10:5].
        mov     al,L2_CMD_CR            ; Control register read command.
        mov     ah,L2_WAY0              ; Way 0.
        call    CommandL2               ; Read it.
        jc      L2RegReadExit           ; Exit if cache error.

;----------------------------------------------------------------------------
; The read value will be in BBL_CR_ADDR (bits 21 thru 28).
;----------------------------------------------------------------------------

        mov     ecx,BBL_CR_ADDR         ; L2 Address register.
        rdmsr
        shr     eax,21d                 ; Return value is in ADDR [28:21].
        mov     byte ptr [bp+1ch],al    ; Store return value in AL on stack.

L2RegReadDone:
        clc                             ; Clear CF to indicate success.

;----------------------------------------------------------------------------
; All done.
;----------------------------------------------------------------------------

L2RegReadExit:
        popad                           ; Restore registers (including
                                        ; new AL).
        ret                             ; Go home.

L2RegRead       ENDP

;****************************************************************************
; L2RegWrite
;
; This procedure writes a control register in the L2 controller.  This
; procedure DOES support multiple controllers.
;
; I/P = ECX = Address of the register being accessed.
; I/P = BL = Controller number.
; I/P = AL = 8 bit value to be written to the L2 controller config register.
; O/P = CY = Set if L2 is permanently busy and is not accepting commands.
;
; Destroys no working registers but for the ones used as O/P.
;
; Sends a CW command to L2 controller and waits for the completion of command.
;****************************************************************************

L2RegWrite      PROC    NEAR

        pushad                          ; Save registers.

        xor     edx,edx                 ; Zero upper address.
        xor     edi,edi                 ; Zero upper data pattern.
        xor     esi,esi                 ; Zero lower data pattern.
        and     eax,000000ffh           ; EAX = ADDR value to write.  Start
                                        ; with data (from AL parameter).
        shl     eax,21d                 ; Align data to ADDR [28:21].
        shl     ebx,11d                 ; Align controller number to
                                        ; ADDR [12:11] for CSRAM devices
                                        ; (from BL parameter).
        and     ebx,CSRAM_CONTROLLER_MSK ; Mask out everything but CSRAM 
                                        ; controller number in EBX.
        or      eax,ebx                 ; Add CSRAM controller number into
                                        ; ADDR value.
        shl     ebx,6h                  ; Duplicate controller number in
                                        ; ADDR [17] for BSRAM devices.
        and     ebx,BSRAM_CONTROLLER_MSK ; Mask out everything but BSRAM 
                                        ; controller number in EBX.
        or      eax,ebx                 ; Add BSRAM controller number into
                                        ; ADDR value.
        shl     ecx,5h                  ; Align address to ADDR [10:5].
        or      ecx,eax                 ; Add address to ADDR value.
        xor     eax,eax                 ; Zero command.

        mov     al,L2_CMD_CW            ; Control register write command.
        mov     ah,L2_WAY0              ; Way 0.
        call    CommandL2               ; Write it.
        
        popad                           ; Restore registers.
        ret                             ; Go home.

L2RegWrite      ENDP

;****************************************************************************
; WriteAndCheckAlias
;
; Writes and check the data at an alias address.
;
; I/P = EDI = Start address of the 32 byte cache line to be written.
; I/P = ESI = Start address where aliasing of the pattern needs to be checked.
; I/P = ECX = low 32 bit data pattern used for writing.
; I/P = EDX = High 32 bit data pattern used for writing.
; O/P = ZF = True, if alias occured at the requested cache address.
; O/P = CY = Set if L2 is permanently busy and is not accepting commands.
;
; Destroys no working registers but for the ones used as O/P.
;
; This procedure writes a 64 bit pattern to the entire cache line
; of 32 bytes, starting at address specified in the input.  It then checks
; if an alias occured at the I/P requested alias address.
;****************************************************************************

WriteAndCheckAlias      PROC    NEAR

        pushad                          ; Store registers.
        mov     bp,sp                   ; Use register values off stack.

;----------------------------------------------------------------------------
; Write data pattern to specified address.
;----------------------------------------------------------------------------

        mov     ecx,edi                 ; Lower address (from EDI parameter).
        xor     edx,edx                 ; Upper address.
        mov     edi,dword ptr[bp+14h]   ; Upper data (from EDX parameter).
        mov     esi,dword ptr[bp+18h]   ; Lower data (from ECX parameter).
        mov     al,L2_CMD_TWW           ; Tag write with data write command.
        mov     ah,L2_WAY0              ; Way 0.
        call    CommandL2               ; Write the cache line.
        jc      WriteAndCheckAliasExit  ; Check for cache error.

;----------------------------------------------------------------------------
; Read back data pattern at specified aliasing address.
;----------------------------------------------------------------------------

        xor     edx,edx                 ; Upper address.
        xor     edi,edi                 ; Upper data.
        xor     esi,esi                 ; Lower data.
        mov     ecx,dword ptr [bp+4h]   ; Lower address (from ESI parameter).
        mov     al,L2_CMD_TRR           ; Tag read with data read command.
        mov     ah,L2_WAY0              ; Way 0.
        call    CommandL2               ; Read the cache line.
        jc      WriteAndCheckAliasExit  ; Check for cache error.

;----------------------------------------------------------------------------
; Check to see if data write overwrote aliasing address.  If so, we've
; wrapped around and we now know the cache size.
;----------------------------------------------------------------------------

        mov     ecx,BBL_CR_D0           ; L2 chunk 0 data register.

Read8ByteChunk:
        rdmsr

        cmp     eax,dword ptr [bp+18h]  ; Check for correct low pattern.
        jnz     WriteAndCheckAliasDone  ; If not equal, then we've wrapped
                                        ; around.
CompareUpper32Data:
        cmp     edx,dword ptr [bp+14h]  ; Check for correct high pattern.
        jnz     WriteAndCheckAliasDone  ; If not equal, then we've wrapped
                                        ; around.
NextChunk:
        inc     ecx                     ; Next L2 chunk data register.
        cmp     ecx,BBL_CR_D3           ; Check for beyond D3 data chunk.
        jbe     Read8ByteChunk          ; If not beyond, continue to compare.

;----------------------------------------------------------------------------
; If control comes here it means all reads and compares worked.
;----------------------------------------------------------------------------

        xor     ax,ax                   ; Set ZF for no wrap around.

WriteAndCheckAliasDone:
        clc                             ; Clear CF to indicate success.

WriteAndCheckAliasExit:
        popad                           ; Restore registers.
        ret                             ; Go home.

WriteAndCheckAlias      ENDP

;****************************************************************************
; SetL2CacheLatency
;
; This procedure determines and sets the L2 cache latency value in CTL3.
;
; I/P = None.
; O/P = Results will be in CTL3.
; O/P = CY = Set if L2 is permanently busy and is not accepting commands or
;            if current core speed is not in our latency table.
;
; Destroys no working registers but for the ones used as O/P.
;
; Determines and sets processor L2 latency in CTL3.
;****************************************************************************

SetL2CacheLatency     PROC    NEAR

        pushad                          ; Store registers.

;---------------------------------------------------------------------------
; Check for latency override.
;---------------------------------------------------------------------------

        mov     ecx,BBL_CR_OVRD         ; L2 cache latency override register.
        rdmsr
        and     edx,1E00000h            ; We want latency override info only.
        jnz     UseLatencyOverride      ; If it's non-zero, use it.

;---------------------------------------------------------------------------
; No latency override, so use latency table.
;---------------------------------------------------------------------------

UseLatencyTable:
        mov     ecx,L2_CR0              ; L2 controller device/stepping id
                                        ; register.
        call    L2RegRead               ; Read device/stepping id register.
        jc      SetL2CacheLatencyExit   ; Check for cache error.
        test    al,20h                  ; Bit 5: BSRAM/CSRAM.  A 0 indicates
        jnz     LoadCSRAMTable          ; BSRAM, a 1 is for CSRAM.
                                        ; Determines which latency table we
                                        ; use.
LoadBSRAMTable:
        mov     si,OFFSET bsramTable    ; SI = OFFSET of latency table.
        jmp     DetermineSpeed

LoadCSRAMTable:
        mov     si,OFFSET csramTable    ; SI = OFFSET of latency table.

DetermineSpeed:        
        mov     ecx,EBL_CR_POWERON      ; Read power on configuration MSR.
        rdmsr
        and     eax,03C80000h           ; We're interested in 66/100 and
                                        ; core ratio.
        shr     eax,19d                 ; AL = 66/100 and core ratio 
                                        ; (to determine core speed).
LookupLoop:
        mov     ah,BYTE PTR cs:[si]     ; AH = core speed from table entry.
        cmp     al,ah                   ; Compare against actual core speed.
        je      LookupDone              ; If same, then go write latency.
        cmp     ah,0FFh                 ; Check for table end marker (0FFh).
        je      UnknownSpeed            ; If end marker, then we don't have
                                        ; this speed in the table.
        add     si,2h                   ; Increment SI to next table entry.
        jmp     LookupLoop              ; Continue looking.

bsramTable:
        ;       SPEED     CTL3 [4:1]
        ;       =----==-= ==========
        db      00001000b,00000010b     ; 66/200MHz (1/3)
        db      00101000b,00000010b     ; 66/233MHz (2/7)
        db      00010000b,00000100b     ; 66/266MHz (1/4)
        db      00110000b,00000110b     ; 66/300MHz (2/9)
        db      00000000b,00001000b     ; 66/333MHz (1/5)
        db      00100000b,00001100b     ; 66/366MHz (2/11)
        db      00001001b,00000110b     ; 100/300MHz (1/3)
        db      00101001b,00001010b     ; 100/350MHz (2/7)
        db      00010001b,00001110b     ; 100/400MHz (1/4)
        db      00110001b,00010000b     ; 100/450MHz (2/9)
        db      00000001b,00010000b     ; 100/500MHz (1/5)
        db      11111111b,00000000b     ; End Marker.

csramTable:
        ;       SPEED     CTL3 [4:1]
        ;       ========= ==========
        db      00001001b,00010100b     ; 100/300MHz (1/3)
        db      00101001b,00010110b     ; 100/350MHz (2/7)
        db      00010001b,00011000b     ; 100/400MHz (1/4)
        db      00110001b,00011010b     ; 100/450MHz (2/9)
        db      11111111b,00000000b     ; End Marker.

LookupDone:
        mov     bl,BYTE PTR cs:[si+1]   ; Load L2 cache latency for this
                                        ; core speed.
        jmp     writeLatency            ; Go write L2 cache latency.

;---------------------------------------------------------------------------
; Use latency override from BBL latency register, bits 56:53.
;---------------------------------------------------------------------------

UseLatencyOverride:
        mov     ebx,edx                 ; Get override bits.
        shr     ebx,20d                 ; Align for CTL3.

WriteLatency:
        mov     ecx,BBL_CR_CTL3         ; Write L2 cache latency into
                                        ; CTL3 [4:1].
        rdmsr
        and     eax,0ffffffe1h          ; Replace L2 cache latency with value
        or      al,bl                   ; from either the table or override.
        wrmsr

SetL2CacheLatencyDone:
        clc                             ; Clear CF to indicate success.
        jmp     SetL2CacheLatencyExit   ; All done.

;---------------------------------------------------------------------------
; Control comes here if the core speed of the processor is not supported
; in our latency table.  We return the CF set to indicate error.
;---------------------------------------------------------------------------

UnknownSpeed:
        stc                             ; Set CF to indicate error.

SetL2CacheLatencyExit:
        popad                           ; Restore registers.
        ret                             ; Go home.

SetL2CacheLatency     ENDP

;****************************************************************************
; SetL2LatencyMode
;
; This procedure sets the latency in ALL L2 cache controllers.
;
; I/P = AL = Value to write into the cache latency registers.
; O/P = Results will be in L2 controllers latency mode registers.
; O/P = CY = Set if L2 is permanently busy and is not accepting commands.
;
; Destroys no working registers but for the ones used as O/P.
;
; Sets latency mode in ALL L2 controller CR4 registers.
;****************************************************************************

SetL2LatencyMode        PROC    NEAR
                        
        pushad                          ; Save registers.
        mov     ah,al                   ; Store latency mode value.
        and     ah,3h                   ; Only two bits valid.
        xor     bl,bl                   ; Start with controller 0.
        mov     ecx,L2_CR4              ; L2 controller latency mode register.
        call    L2RegRead               ; Read from current controller.
        jc      SetL2LatencyModeExit    ; Check for cache error.
        and     al,0fch                 ; Mask out lower two bits.
        or      al,ah                   ; Add in latency mode value.

WriteLatencyLoop:
        call    L2RegWrite              ; Write to current controller.
        jc      SetL2LatencyModeExit    ; Check for cache error.
        inc     bl                      ; Increment controller counter.
        cmp     bl,MAX_L2_CONTROLLERS   ; Check to see if we've exceeded 
                                        ; maximum controller number (we
                                        ; blindly write to all possible
                                        ; controllers).
        jl      WriteLatencyLoop        ; If not, continue looping.

;---------------------------------------------------------------------------
; BSRAM timing adjustment for 266MHz.  Write 00000000b to all possible
; L2 cache control register CR8's.  We may change this to a different value
; in the future.
;---------------------------------------------------------------------------

        mov     ecx,L2_CR0              ; L2 controller device/stepping id
                                        ; register.
        call    L2RegRead               ; Read device/stepping id register.
        jc      SetL2LatencyModeExit    ; Check for cache error.
        test    al,20h                  ; Bit 5 of this register indicates
                                        ; if we have a BSRAM or CSRAM device
                                        ; (a 1 is for CSRAM, a 0 for BSRAM).
        jnz     SetL2LatencyModeDone    ; Skip the following for CSRAM.

        mov     ecx,EBL_CR_POWERON      ; Power-on configuration register.
        rdmsr
        and     eax,03C80000h           ; We're interested in 66/100 and
                                        ; ratio.
        shr     eax,19d                 ; AL = 66/100 bit and ratio.
        cmp     al,00010000b            ; Is it 266MHz?
        jne     SetL2LatencyModeDone    ; If not, we're done.

        xor     bl,bl                   ; Start with controller 0.
        mov     ecx,L2_CR8              ; L2 controller timing register.

        call    L2RegRead               ; Read timing register.
        jc      SetL2LatencyModeExit    ; Check for cache error.
        and     al,0FCh                 ; Keep all but lower two bits.
        or      al,0h                   ; Add timing setting.

WriteCR8Loop:
        call    L2RegWrite              ; Write optimal timing to timing
                                        ; register.
        jc      SetL2LatencyModeExit    ; Check for cache error.
        inc     bl                      ; Increment controller counter.
        cmp     bl,MAX_L2_CONTROLLERS   ; Check to see if we've exceeded
                                        ; maximum controller number (we
                                        ; blindly write to all possible
                                        ; controllers).
        jl      WriteCR8Loop            ; If not, continue looping.

SetL2LatencyModeDone:
        clc                             ; Clear CF to indicate success.

SetL2LatencyModeExit:
        popad                           ; Restore registers.
        ret                             ; Go home.

SetL2LatencyMode        ENDP

;****************************************************************************
; SizeL2Cache routine follows.
; 
; This procedure determines the size of the L2 cache (for both CSRAM and
; BSRAM devices).
;
; I/P = None.
; O/P = Results will be in CTL3.
; O/P = CY = Set if L2 is permanently busy and is not accepting commands or
;            cache sizing error occured.
;
; Determines the size of the L2 cache and sets it in CTL3.
;****************************************************************************

SizeL2Cache     PROC    NEAR

        pushad                          ; Save registers.

;----------------------------------------------------------------------------
; BSRAM and CSRAM cache size algorithms differ.  Determine which L2 cache
; controller we have.
;----------------------------------------------------------------------------

        mov     ecx,L2_CR0              ; L2 controller device/stepping id
                                        ; register.
        call    L2RegRead               ; Read device/stepping id register.
        jc      SizeL2CacheExit         ; Check for cache error.
        test    al,20h                  ; AL = L2 cache control register CR0
                                        ; left over from above.  Bit 5 of
                                        ; this registers indicates whether
                                        ; we have a BSRAM or CSRAM device
                                        ; (a 1 is for CSRAM, a 0 for BSRAM).
        jnz     CsramSize               ; If it's 1, then do CSRAM algorithm.

;----------------------------------------------------------------------------
; BSRAM L2 cache sizing is straight forward.  We don't need to worry about
; the number of banks.  We simply determine the total cache size (by writing
; cache lines every power of 2 size and looking for aliasing to 0h) assuming
; only 1 bank.  We then program 1 bank into CTL3 [12:11] and use the total
; size to program the size per bank CTL3 [17:13].  Just assume 1 bank!
;----------------------------------------------------------------------------

BsramSize:

;----------------------------------------------------------------------------
; First write and check 256K.  No smaller caches exist.
;----------------------------------------------------------------------------

        xor     bx,bx                   ; BX = cache size.
        mov     bl,01h                  ; Initial cache size of 256k.
        xor     edi,edi                 ; Write address.
        xor     esi,esi                 ; Aliasing address.
        mov     edx,HIGH_32_PATTERN_0   ; Data high pattern.
        mov     ecx,LOW_32_PATTERN_0    ; Data low pattern.
        call    WriteAndCheckAlias      ; Write and check for aliasing.
        jc      SizeL2CacheExit         ; Check for cache error.
        jnz     SizeL2CacheFail         ; Since the smallest cache size is
                                        ; 256KB, this better work.  If it
                                        ; doesn't, we've got a critical error.

;----------------------------------------------------------------------------
; Now we find the top of the cache.
;----------------------------------------------------------------------------

        mov     edx,HIGH_32_PATTERN_ALIAS ; Data high pattern.
        mov     ecx,LOW_32_PATTERN_ALIAS ; Data low pattern.
        add     edi,64d*1024d           ; Next write address
                                        ; (64k, 4 ways).

FindCacheTop:
        call    WriteAndCheckAlias      ; Write and check for aliasing.
        jc      SizeL2CacheExit         ; Check for cache error.
        jz      CacheTopFound           ; If aliasing occured, we've found
                                        ; the top.
        shl     bl,1h                   ; Next Size indicator.
        add     edi,edi                 ; Double the write address (cache
                                        ; sizes only power of 2).
        cmp     edi,MAX_CACHE_SIZE/4    ; Check to make sure we haven't
                                        ; exceeded maximum possible size.
        ja      SizeL2CacheFail         ; If we have exceeded, we've got a
                                        ; critical failure.
        jmp short FindCacheTop          ; Continue looking for cache top.

;----------------------------------------------------------------------------
; We've found the top.  Now we load this into CTL3 [17:13].  We also 
; assume only 1 bank - hence size per bank is just size.
;----------------------------------------------------------------------------

CacheTopFound:
        and     ebx,0000001fh           ; Size only.
        shl     ebx,13d                 ; Align to CTL3 [17:13].
        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        and     eax,0fffc1fffh          ; Zero size per bank CTL3 [17:13].
        or      eax,ebx                 ; Replace size per bank CTL3 [17:13].
        wrmsr

;----------------------------------------------------------------------------
; We now need to write the cache size into each possible L2 cache control
; register CR2's.
;----------------------------------------------------------------------------

        shr     ebx,11d                 ; Align to the L2 controller register 
                                        ; CR2 [7:2].
        mov     ecx,L2_CR2              ; L2 controller cache size register.
        call    L2RegRead               ; Read current L2 controller register
                                        ; CR2.  We need to get the number of
                                        ; banks out of CR2 [1:0] to determine
                                        ; size/bank to program into CR2 [7:2].
        jc      SizeL2CacheExit         ; Check for cache error.
        and     al,03h                  ; Mask out all except number of banks.
        mov     cl,al
        shr     ebx,cl                  ; Divide cache size by number of
                                        ; banks to get size/bank.
        or      al,bl                   ; Merge size/bank value into CR2
                                        ; value.

        xor     bl,bl                   ; Start with controller 0.
        mov     ecx,L2_CR2              ; L2 controller cache size register.

WriteLoop:
        call    L2RegWrite              ; Write cache size to all controllers.
        jc      SizeL2CacheExit         ; Check for cache error.
        inc     bl                      ; Increment controller counter.
        cmp     bl,MAX_L2_CONTROLLERS   ; Check to see if we've exceeded
                                        ; maximum.
        jl      WriteLoop               ; If not, continue looping.
        jmp     SizeL2CacheDone         ; All done.

;----------------------------------------------------------------------------
; CSRAM L2 cache sizing is even more simple.  The CSRAM tells us the size per
; CSRAM and the total number of CSRAMs in the system.  The total cache size
; is size/CSRAM * Number of CSRAMs.  Also, just like in the BSRAM case, we
; assume only 1 bank.  We program 1 bank into CTL3 [12:11] and use the total
; size to program the size per bank CTL3 [17:13].  Just assume 1 bank!
;----------------------------------------------------------------------------

CsramSize:
        mov     ecx,L2_CR2              ; L2 controller cache size register.
        call    L2RegRead               ; Read cache size register.
        jc      SizeL2CacheExit         ; Check for cache error.

        mov     bl,al
        and     bl,7h                   ; BL = Number of CSRAM's only.

        mov     cl,al
        shr     cl,6h                   ; CL = Size/CSRAM.
        xor     eax,eax
        mov     ax,1h                   ; AX = 1 = 512KB value for
                                        ; CTL3 [17:14].  Minimum size of a
                                        ; CSRAM is 512KB.
        shl     al,cl                   ; Adjust CTL3 [17:14] value based on
                                        ; size/CSRAM.
        mul     bl                      ; Adjust CTL3 [17:14] value based on
                                        ; number of CSRAMs.
                                        ; Now we have total cache size.
        and     ax,0Fh                  ; Maximum of 4 bits in CTL3 [17:14].
        jz      SizeL2CacheFail         ; Cache size must be non-zero!

        shl     eax,14d                 ; Align to CTL3 [17:14].
        mov     ebx,eax

        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        and     eax,0fffc1fffh          ; Zero Size per bank CTL3 [17:13].
        or      eax,ebx                 ; Replace size per bank CTL3 [17:13].
        wrmsr

SizeL2CacheDone:
        clc                             ; Clear CF to indicate success.
        jmp     SizeL2CacheExit         ; All done.

SizeL2CacheFail:
        stc                             ; Set CF to indicate error.

SizeL2CacheExit:
        popad                           ; Restore registers.
        ret                             ; Go home.

SizeL2Cache     ENDP

;****************************************************************************
; SetPhysicalCacheableRange
;
; This procedure determines the physical cacheable range of memory based on
; the L2 controller.
;
; I/P = None.
; O/P = Results will be in CTL3.
; O/P = CY = Set if L2 is permanently busy and is not accepting commands.
;
; Determines the maximum cacheable address based on the L2 controller and
; puts this in CTL3, bits 20-22.
;****************************************************************************

SetPhysicalCacheableRange       PROC    NEAR

        pushad                          ; Save registers.

;****************************************************************************
; IMPORTANT NOTE: IF A SYSTEM HAS MORE MEMORY THAN THE SUPPORTED PHYSICAL
; ADDRESS SPACE, THEN IT IS NECESSARY TO PROGRAM MTRRs IN SUCH A WAY THAT ANY
; ADDRESS ABOVE THE SUPPORTED RANGE IS UNCACHEABLE. THIS MEANS THAT YOU HAVE
; TO COMMUNICATE THIS INFO FROM THIS MODULE TO THE MODULE THAT SETS MTRRs IN
; AN IMPLEMENTATION SPECIFIC WAY. THE CODE  THAT IS NECESSARY TO DO THIS IS
; IMPLEMENTATION DEPENDANT AND IS NOT INCLUDED HERE.
;****************************************************************************

;----------------------------------------------------------------------------
; Read physical cacheable address space register, L2 control register CR3.
;----------------------------------------------------------------------------

        xor     eax,eax
        mov     ecx,L2_CR3              ; L2 controller physical cacheable
                                        ; address space register.
        call    L2RegRead               ; Read physical cacheable address
                                        ; space.
        jc      SetPhysicalCacheableRangeExit ; Check for cache error.
        mov     ebx,eax                 ; EBX = Physical cacheable space.

;----------------------------------------------------------------------------
; Determine if it's BSRAM or CSRAM.  CSRAM physical cacheable address space
; values are inverted from BSRAM/CTL3.
;----------------------------------------------------------------------------

        mov     ecx,L2_CR0              ; L2 controller device/stepping id
                                        ; register.
        call    L2RegRead               ; Read device/stepping id.
        jc      SetPhysicalCacheableRangeExit ; Check for cache error.
        test    al,20h                  ; AL = L2 cache control register CR0
                                        ; left over from above.  Bit 5 of
                                        ; this registers indicates whether
                                        ; we have a BSRAM or CSRAM device
                                        ; (a 1 is for CSRAM, a 0 for BSRAM).
        jz      bypassInvert            ; If it's 0, skip invert.

;----------------------------------------------------------------------------
; CSRAM physical cacheable address space is inverted from BSRAM and CTL3.
; Hence, we must invert.
;----------------------------------------------------------------------------

        not     ebx                     ; Invert for CSRAM devices.

bypassInvert:
        and     ebx,00000007h           ; Mask out other bits.
        shl     ebx,20d                 ; Align to CTL3 [22:20].

        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        and     eax,0ff8fffffh          ; Zero physical address range,
                                        ; CTL3 [22:20].
        or      eax,ebx                 ; Replace physical address range,
                                        ; CTL3 [22:20].
        wrmsr

SetPhysicalCacheableRangeDone:
        clc                             ; Clear CF to indicate success.

SetPhysicalCacheableRangeExit:
        popad                           ; Restore registers.
        ret                             ; Go home.

SetPhysicalCacheableRange       ENDP

;****************************************************************************
; CheckAndEnableECC
;
; This procedure determines if the L2 supports ECC and enables it in CTL3
; if so.
;
; I/P = None.
; O/P = Results will be in CTL3 and L2.
; O/P = CY = Set if L2 is permanently busy and is not accepting commands.
;
; Determines if ECC is supported and enables it if so.  Also invalidates
; all ECC bits in the L2
;****************************************************************************

CheckAndEnableECC       PROC    NEAR

        pushad                          ; Save registers.

;----------------------------------------------------------------------------
; Enable user supplied ECC CTL [18] so that we may write our own ECC pattern.
;----------------------------------------------------------------------------

        mov     ecx,BBL_CR_CTL          ; L2 control register (CTL).
        rdmsr
        or      eax,000040000h          ; Enable user supplied ECC CTL [18].
        wrmsr

;----------------------------------------------------------------------------
; Write our pattern of ECC.  We do this by putting our pattern in the DECC
; register, writing data and our ECC pattern to tag address 0h, and then
; reading back the data and ECC from tag address 0h.  We use the 
; WriteAndCheckAlias routine here simply to save code space - we don't need
; to check any aliasing.
;----------------------------------------------------------------------------

        mov     ecx,BBL_CR_DECC         ; L2 data ECC register.
        xor     edx,edx                 ; Upper ECC pattern.
        mov     eax,0aa55aa55h          ; Lower ECC pattern.
        wrmsr

        xor     edi,edi                 ; Write address.
        xor     esi,esi                 ; Alias address.
        mov     edx,HIGH_32_PATTERN_0   ; High data pattern.
        mov     ecx,LOW_32_PATTERN_0    ; Low data pattern.
        call    WriteAndCheckAlias      ; Write and read back.
        jc      CheckAndEnableECCExit   ; Check for cache error.

        mov     ecx,BBL_CR_DECC         ; L2 data ECC register.
        rdmsr                           ; Read the ECC bits back.
        cmp     eax,0aa55aa55h          ; Compare it to the original ECC
                                        ; pattern we put in DECC.
        jnz     ECCDone                 ; If it's not equal, then ECC is
                                        ; not supported.  Otherwise it is.

;----------------------------------------------------------------------------
; ECC is supported.  So we set CTL3 [5] to enable ECC.
;----------------------------------------------------------------------------

EnableECC:
        mov     ecx,BBL_CR_CTL3         ; L2 control register 3 (CTL3).
        rdmsr
        or      eax,000000020h          ; Enable ECC checking CTL3 [5].
        wrmsr

;----------------------------------------------------------------------------
; Disable user supplied ECC CTL [18] for normal operation.
;----------------------------------------------------------------------------

ECCDone:
        mov     ecx,BBL_CR_CTL          ; L2 control register (CTL).
        rdmsr
        and     eax,0fffbffffh          ; Disable user supplied ECC CTL [18].
        wrmsr

CheckAndEnableECCDone:
        clc                             ; Clear CF to indicate success.

CheckAndEnableECCExit:
        popad                           ; Restore registers.
        ret                             ; Go home.

CheckAndEnableECC       ENDP

;R03 - end

