;	[]===========================================================[]
;
;	NOTICE:	THIS PROGRAM BELONGS TO AWARD SOFTWARE INTERNATIONAL(R)
;		INC. IT IS CONSIDERED A TRADE SECRET AND IS NOT TO BE
;		DIVULGED OR USED BY PARTIES WHO HAVE NOT RECEIVED
;		WRITTEN AUTHORIZATION FROM THE OWNER.
;
;	[]===========================================================[]
;

;************************************************************************
;*	This file will be included in PMUPOST.ASM, and compiled		*
;*	with 3 stages. All codes occupied at E000 segment, and		*
;*	some codes will be remove to SMI RAM for run-time execution.	*
;************************************************************************

;----------------------------------------------------------------------------
;Rev	Date	 Name	Description
;----------------------------------------------------------------------------
;R29A	12/16/97 KEN	Fixed coding mistake and cause some USB keyboard with
;			acceptable error at initialization stage can't work.
;			But it was working properbly with older BIOS.
;R32	12/16/97 KEN	Clear the bit-map flag of USB_RAM in LMEM_RESOURCE and
;			LMEM_RESERVED to release the UMB space when USB legacy
;			supporting is disabled. To correspond with the earlier
;			Usb_Final_Init before POST_82s.
;R31	12/16/97 KEN	Restore original method to allocate shadow RAM for
;			USB_RAM. That is, searching the available shadow RAM
;			upward from C800h to DC00h.
;			To fix that there are no sufficient UMB space for some
;			PCI devices that the size of option ROM is more than
;			48KB.
;R08A	12/15/97 KEN	Fixed bug for UHCI, the USB keyboard won't work after
;			seven times of hot-plugging. The causation is that
;			the flag of control buffer map is never cleared after
;			keyboard initailization.
;R30	11/28/97 JKY	When PMI_HANDLER move to XGROUP , need using F000_call
;R29	11/20/97 KEN	Added codes in Usb_Smi routine to process the error
;			recovery of USB host controller to patch some unstable
;			M/B.(e.g. GemLight 440LX+SMC672 M/B)
;R28	11/13/97 KEN	Fixed bug that there is incorrect handshaking when the
;			USB ownership is changed from the Windows98 to the
;			system BIOS.(e.g. select "Restart in MS-DOS mode" item
;			in Windows98 "Shut Down" menu)
;R27	11/11/97 KEN	Fixed bug that the SET_IDLE and SET_REPORT request
;			have no function with the newest USB keyboard(spec.
;			V1.0 final).
;R26	11/10/97 KEN	Added five keys' codes to support Japanese USB K/B.
;			Key codes translation:
;				USB Key Code	Translated Key Code
;				87h		73h
;				88h		70h
;				89h		7Dh
;				8Ah		79h
;				8Bh		7Bh
;R25	10/06/97 KEN	Get hub descriptor with the value 29h of descriptor
;			type byte to match the newest spec.
;R24	09/18/97 KEN	Get USB keyboard endpoint descriptor with more flexible
;			method to match the old and newest(V1.0 final) USB HID
;			specification.
;			To solve some new USB keyboards can't work with USB
;			BIOS legacy supporting.
;R23	08/28/97 KEN	Added codes to clear enabled status change of hub port
;			after device is disconnected and disabled, otherwise
;			the Intel Hub will always respond with hub port status
;			change.
;R22A	07/30/97 KEN	Modified I/O trapping codes to suit the current BIOS.
;			1. Added definition "IOTRAP_SUPPORT" to enable this
;			   module.
;			2. Got the device ID of host controller with wHostID
;			   that originally got with PIIX3_USB definition.
;			3. Call Ct_DisableUsbLegacySmi/Ct_EnableUsbLegacySmi
;			   to instead that direct setting registers according
;			   to PIIX_ID definition originally.
;			4. Four chipset hook routines added:
;				Ct_CheckUsbLegacySmi
;				Ct_ClearUsbLegacySmi
;				Ct_DisableUsbLegacySmi
;				Ct_EnableUsbLegacySmi
;R22	07/30/97 KEN	Added 60h/64h I/O trapping codes got from Award U.S.
;			Original files were backup and zipped as usb0730.zip
;			in oldfile directory.
;			Only one difference with original files in this
;			revision:
;			  Added pseudo code 'short' to some conditional or
;			  unconditional jumpping instructions.
;R21	06/25/97 RCH	Fixed SCT sometimes testing failure if "USB keyboard"
;			is enabled.
;R20	06/24/97 KEN	Fixed bugs:
;			 1. The system hang up while running the diagnostic
;			    program of Adaptec SCSI ROM.
;			 2. The system hang up when the boot-ROM of 3Com 3C900
;			    is executed.
;			The causation is that the USB kernal uses the base
;			memory at POST stage, and the base memory is conflicted
;			with these programs. To solve these problems, the USB
;			kernal will use the shadow RAM at whole time.
;R19A	06/16/97 KEN	Don't send any key code if over-run, otherwise the
;			system will hang-up when many keys are struke
;			simultaneously and the KEYB driver is installed.
;R19	06/16/97 KEN	Fixed bug for incorrect key code 00h of over-run, the
;			correct key code is 0FFh, and this will cause the KEYB
;			driver to translate key code 00h to 53h("S").
;R18	06/12/97 KEN	Added the definition "NO_UPDATE_USBKB_LED" to patch
;			that the SEJIN USB keyboard is inactive after updating
;			LED.
;R17	06/12/97 KEN	Added "GET_REPORT_DESCRIPTOR" to patch that the SEJIN
;			USB keyboard can't be activated.
;R16	06/11/97 KEN	Fixed bug for the CMD USB keyboard can't repeat and
;			abnormal keystroke. The causation seems to be that
;			the "SET_IDLE" request isn't set correctly.
;R15	06/10/97 KEN	Added 102-key supporting.
;			USB key number 32h, 64h are corresponding to AT key
;			number 42, 45.
;R14A	05/30/97 RCH	Fixed compiation error
;R14	05/27/97 KEN	Fixed bug that the testing program of CATC USB tester
;			report error at first time testing with 430VX M/B and
;			the SETUP item of "USB Keyboard Support" is disabled.
;R13	05/23/97 KEN	Fixed bug that the system hang-up at POST 31h in 430VX
;			system with CATC USB tester.
;R12	05/08/97 KEN	Fixed bug that the screen display abnormal in Win95
;			(no USB supporting) when press USB keyboard and move
;			PS/2 mouse at the same time.
;R11	04/28/97 KEN	Support USB always enabled at POST stage, and
;			initialize USB before memory testing.
;R10	04/16/97 KEN	Fixed bug that the system hang-up when cancel the
;			detection of PCI USB Controller at Win95(no USB
;			supporting). This condition occured when the USB Host
;			Controller shared IRQ with another PCI bus master card
;			and the BIOS USB legacy supporting is enabled.
;R09	04/09/97 KEN	Patched the abnormal keystroke of TATUNG USB keyboard.
;R08	04/09/97 KEN	Fixed bug for supporting multiple hub.
;R07	04/09/97 KEN	Saving all registers is done by Usb_Smi, it's done
;			by PMUPOST originally.
;R06	04/07/97 KEN	Changed the method of processing keystroke, to solve
;			the CMD keyboard problem.
;R05	04/01/97 KEN	Patch that some USB keyboards don't support the
;			SET_IDLE request.(e.g. BTC USB Keyboard)
;R04	03/26/97 KEN	Rename this file as UHCI.ASM, and is included by new
;			USBBIOS.ASM. Some host controller independent codes in
;			this file will be moved to new USBBIOS.ASM in future.
;R03	02/19/97 RCH	Fixed SCT 5.3 "unreport memory" error if USB legacy
;			support is enabled.
;R02	01/22/97 KEN	Updated the interval of system control task to 1024,
;			avoid to enter SMI frequently.
;R01	01/21/97 KEN	Fixed coding mistake.
;R00	01/16/97 KEN	New USBBIOS structure to support HUB and multiple
;			keyboard. The old files is backup as USB0116.zip.

;************************************************************************
;									*
; NOTES: If no special definition, DS and ES always point to USB_RAM	*
;									*
;************************************************************************

;R04IFDEF	USB_SUPPORT
IF	COMPILE_FOR_USBBIOS EQ 1

;****************************************************************
;*								*
;*	COMPILING STAGE 1					*
;*								*
;*	MISCELLANEOUS DEFINITION				*
;*								*
;****************************************************************

;R04	INCLUDE HCI.EQU
;R04	INCLUDE USB.EQU

	extrn	Get_Pci:near
	extrn	Set_Pci:near
	extrn	F000_Shadow_W:near
	extrn	F000_Shadow_R:near
	extrn	Ct_Shadow_RW:near
	extrn	Ct_Disable_Shadow:near		;R20
	extrn	USB_RAM_SEG:word
	extrn	LMEM_RESOURCE:abs
	extrn	LMEM_RESERVED:abs		;R20
	extrn	USB_STATUS:byte

;R04USB_RAM SEGMENT USE16	AT 0
;R04	INCLUDE HCDATA.INC
;R04USB_RAM ENDS

ENDIF	;COMPILE_FOR_USBBIOS EQ 1

IF	COMPILE_FOR_USBBIOS EQ 2

;R03 - start
ifdef	PNP_BIOS
	extrn	Build_UsbRamNode:near
endif;	PNP_BIOS
;R03 - start

;****************************************************************
;*								*
;*	COMPILING STAGE 2					*
;*								*
;*	POST INITIALIZATION CODES OCCUPIED AT E000 SEGMENT	*
;*								*
;****************************************************************

	assume	ds:USB_RAM

;************************************************************************
;*									*
;*	POST INTERFACE MODULE						*
;*									*
;************************************************************************

;[]========================================================================[]
;Procedure:	Usb_Init
;Function:	USB initialization
;Input:		none
;Output:	none
;[]========================================================================[]

	public	Usb_Init
Usb_Init	proc	near
	push	ds
	push	es
	pushad

	call	Ct_USB_Init
	jnc	short @f
	call	Record_USB_Disabled
	stc
	jmp	short Usb_Init_Exit
@@:

;R20	call	Allocate_Post_USBRAM
	call	Allocate_USBRAM			;R20
	call	Get_USBRAM
	mov	ds, ax
	mov	es, ax

	call	Host_Init

	call	System_Control_Init

	call	Resume_Tasks

	call	Host_Run

	clc

Usb_Init_Exit:
	popad
	pop	es
	pop	ds
	ret
Usb_Init	endp

;[]========================================================================[]
;Procedure:	Usb_Final_Init
;Function:	Re-initialize USB for reallocating USB_RAM to shadow
;Input:		none
;Output:	none
;[]========================================================================[]

	public	Usb_Final_Init
Usb_Final_Init	proc	near
	push	ds
	push	es
	pushad

	call	Check_USB_Disabled
;R14A	jnz	short Usb_Final_Init_Exit
	jnz	Usb_Final_Init_Exit			;R14A

	call	Get_USBRAM
	mov	ds, ax
	mov	es, ax

;R20	call	Host_Stop

;R11 - start
	call	Ct_Check_USB_Disabled
	jnc	short @f
	call	Record_USB_Disabled
	call	Host_Stop			;R20
	call	Host_Reset			;R14host controller reset
	mov	cx, wHostID
	add	cx, LEGACY_LO
	F000_CALL	Get_Pci
ifdef	IOTRAP_SUPPORT				;R22A
	and	al, not (USBSMIEN+0Fh)		;R22A
else	;IOTRAP_SUPPORT				;R22A
	and	al, not USBSMIEN
endif	;IOTRAP_SUPPORT				;R22A
	F000_CALL	Set_Pci
	mov	cx, wHostID
	add	cx, LEGACY_HI
	F000_CALL	Get_Pci
	or	al, USBPIRQDEN
	F000_CALL	Set_Pci
	mov	ax, 0FFFFh			;clear all pending status
	mov	dx, USBSTS
	call	Set_Host_Word
	call	Close_USBRAM			;R20
	jmp	short Usb_Final_Init_Exit
@@:
;R11 - end

;R20	call	Allocate_USBRAM
;R20	call	Get_USBRAM
;R03 - start
ifdef	PNP_BIOS
	xor	edi,edi
;R20	mov	di,ax
	mov	di,ds				;R20
	shl	edi,4
ifndef	SMIHANDLE_IN_XGROUP		;R30
	call	Build_UsbRamNode	;build device node for USB RAM
else	;SMIHANDLE_IN_XGROUP		;R30
	POST_func_call	Build_UsbRamNode;R30
endif	;SMIHANDLE_IN_XGROUP		;R30
endif;	PNP_BIOS
;R03 - start
;R20	mov	ds, ax
;R20	mov	es, ax
;R20
;R20	call	Host_Init
;R20
;R20	call	System_Control_Init
;R20
;R20	call	Resume_Tasks
;R20
;R20	call	Host_Run

Usb_Final_Init_Exit:
	popad
	pop	es
	pop	ds
	ret
Usb_Final_Init	endp

;[]========================================================================[]
;Procedure:	Report_USB_Keyboard
;Function:	Report the status of the USB keyboard's existence.
;Input:		none
;Output:	CF = 0 USB keyboard presented
;		CF = 1 no USB keyboard presented
;[]========================================================================[]
	public	Report_USB_Keyboard
Report_USB_Keyboard	proc	near
	pushad
	push	ds

	call	Check_USB_Disabled		;host or legacy support disabled?
	jnz	short Report_USB_Fail

	call	Get_USBRAM
	mov	ds, ax

	test	byte ptr ds:wKbdDevMap, 0FFFFh	;keyboard present?
	jz	short Report_USB_Fail

	clc					;keyboard present
	jmp	short Report_USB_Exit

Report_USB_Fail:
	stc					;keyboard absent

Report_USB_Exit:
	pop	ds
	popad
	ret
Report_USB_Keyboard	endp

;[]========================================================================[]
;Procedure:	Record_USB_Disabled
;Function:	Record USB legacy support disabled to USB_STATUS
;Input:		none
;Output:	none
;[]========================================================================[]

Record_USB_Disabled	proc	near
	push	ds
	push	ax

	F000_CALL	F000_Shadow_W
	mov	ax, 0F000h
	mov	ds, ax
	assume	ds:DGROUP
	mov	al, 01
	or	ds:USB_STATUS, al
	F000_CALL	F000_Shadow_R

	pop	ax
	pop	ds
	assume	ds:USB_RAM
	ret
Record_USB_Disabled	endp

;[]========================================================================[]
;Procedure:	Allocate_Post_USBRAM
;Function:	Allocate USB_RAM for POST usage
;Input:		none
;Output:	none
;[]========================================================================[]

;R20Allocate_Post_USBRAM	proc	near
;R20	push	es
;R20	push	ds
;R20	pushad
;R20
;R20	mov	ax, 9000h
;R20
;R20	call	Record_USBRAM
;R20
;R20	mov	ds, ax
;R20	mov	es, ax
;R20	call	Init_USBRAM
;R20
;R20	popad
;R20	pop	ds
;R20	pop	es
;R20	ret
;R20Allocate_Post_USBRAM	endp

;[]========================================================================[]
;Procedure:	Allocate_USBRAM
;Function:	Allocate USB_RAM for run-time usage, may be shadow RAM or
;		base memory.
;Input:		none
;Output:	none
;[]========================================================================[]

Allocate_USBRAM	proc	near
	push	ds
	push	es
	pushad

	mov	eax, LMEM_RESOURCE[bp]
;R20	mov	dx, 0C800h
;R31	mov	dx, 0DC00h			;R20;start from DC00h
;R31	mov	edi, 00F00000h			;R20;mask value
	mov	dx, 0C800h			;R31;start from C800h
	mov	edi, 0000000Fh			;R31;mask value
	mov	cx, 6
@@:
;R20	test	al, 0Fh
	test	eax, edi			;R20
	jz	short Use_Shadow
;R20	shr	eax, 4
;R20	add	dx, 400h
;R31	shr	edi, 4				;R20
;R31	sub	dx, 400h			;R20;next backward 16KB
	shl	edi, 4				;R31
	add	dx, 400h			;R31;next upward 16KB
	loop	short @b

Use_Base_Mem:
	mov	ax, G_RAM
	mov	ds, ax
	assume	ds:G_RAM
	mov	ax, ds:SIZE_SYSRAM
	sub	ax, 8				;top 8K basemem as temp USBRAM
	and	ax, 0FFFCh			;4K boundary
	mov	ds:SIZE_SYSRAM, ax
	shl	ax, 6				;translate to segment address
	jmp	short Move_USB_RAM

Use_Shadow:
	or	LMEM_RESOURCE[bp], edi		;R20
	mov	LMEM_RESERVED[bp], edi		;R20
	push	dx
	F000_CALL	Ct_Shadow_RW
	pop	ax

Move_USB_RAM:
	call	Record_USBRAM

	mov	ds, ax
	mov	es, ax
	call	Init_USBRAM

	popad
	pop	es
	pop	ds
	ret
Allocate_USBRAM	endp

;[]========================================================================[]
;Procedure:	Record_USBRAM
;Function:	Record USB_RAM segment address to USB_RAM_SEG at F000 segment.
;Input:		AX = segment address
;Output:	none
;[]========================================================================[]

Record_USBRAM	proc	near
	push	ds
	push	ax

	push	ax
	F000_CALL	F000_Shadow_W
	mov	ax, 0F000h
	mov	ds, ax
	assume	ds:DGROUP
	pop	ax
	mov	ds:USB_RAM_SEG, ax
	F000_CALL	F000_Shadow_R

	pop	ax
	pop	ds
	assume	ds:USB_RAM
	ret
Record_USBRAM	endp

;R20 - start
;[]========================================================================[]
;Procedure:	Close_USBRAM
;Function:	Close USB_RAM if Shadow RAM
;Input:		DS = point to USB_RAM
;Output:	none
;[]========================================================================[]

Close_USBRAM:
	mov	dx, ds
	cmp	dx, 0A000h			;base memory?
	jb	short Close_USBRAM_Exit
;R32 - start
	mov	cl, dh
	sub	cl, 0C8h
	mov	edi, 0FFFFFFF0h
	rol	edi, cl
	and	LMEM_RESOURCE[bp], edi
	and	LMEM_RESERVED[bp], edi
;R32 - end
	F000_call	Ct_Disable_Shadow
Close_USBRAM_Exit:
	ret
;R20 - end

;************************************************************************
;*									*
;*	UHCI POST INTERFACE MODULE					*
;*									*
;************************************************************************

;[]========================================================================[]
;Procedure:	Init_USBRAM
;Function:	Initialize USB_RAM to default value
;Input:		DS,ES=USB_RAM
;Output:	none
;[]========================================================================[]

Init_USBRAM:
;R08	xor	ax, ax
;R08	mov	cx, 1000h
;R08	xor	di, di
;R08	rep	stosw
	mov	cx, 2000h			;R08
	xor	si, si				;R08
	call	Clear_Buffer			;R08

;R21 - start
;set the upper 8Kb content of shadow RAM to prevent SCT testing error
	push	es
	push	di
	mov	ax,ds
	mov	es,ax
	mov	ax,0ffffh		;pattern to fill
	mov	di,2000h		;start from 8Kb address
	mov	cx,2000h		;8Kb to fill
	rep	stosw
	pop	di	
	pop	es
;R21 - end

	mov	word ptr ds:[0], 0AA55h
	mov	word ptr ds:[2], 0CB10h
	mov	dword ptr ds:[4], 'BSU$'

	mov	ax, ds
	shl	eax, 4
	mov	ds:dUsbRamAddr, eax
	add	eax, offset FrameList
	mov	ds:dFrameListAddr, eax

	call	Init_FL

	mov	word ptr wTaskLink, 0FFFFh	;task terminated
	mov	word ptr DevAddrMap+2, 0080h	;root hub device

	ret

;[]========================================================================[]
;Procedure:	System_Control_Init
;Function:	Initialize system control task
;Input:		none
;Output:	none
;[]========================================================================[]

System_Control_Init:

	; Initialize Root Hub task

	xor	ax, ax
	mov	wPort1Status, ax
	mov	wPort2Status, ax
	mov	bLedStatus, al

	mov	si, offset SysCtrlQH
	call	Init_QH
	mov	si, offset SysCtrlTD
	call	Init_TD
	or	byte ptr [si].bTdControl, ISS_IOC

	mov	si, offset SysCtrlTask
	call	Init_Task
	mov	byte ptr [si].bTkDevStatus, TK_WFP
	mov	byte ptr [si].bTkDevAddr, 1
;R02	mov	word ptr [si].wTkInterval, 512
	mov	word ptr [si].wTkInterval, 1024			;R02
	mov	word ptr [si].wTkQueueHead, offset SysCtrlQH
	mov	word ptr [si].wTkTransDesp, offset SysCtrlTD
	mov	word ptr [si].wTkProc, offset System_Control SMI_OFFSET
	call	Link_Task

;R06 - start
	; Initialize Timer task, and no link at this stage until
	; the USB keyboard is attached.

	mov	si, offset TimerQH
	call	Init_QH
	mov	si, offset TimerTD
	call	Init_TD
	or	byte ptr [si].bTdControl, ISS_IOC

	mov	si, offset TimerTask
	call	Init_Task
	mov	byte ptr [si].bTkDevStatus, TK_WFP
	mov	byte ptr [si].bTkDevAddr, 1
	mov	word ptr [si].wTkInterval, 64
	mov	word ptr [si].wTkQueueHead, offset TimerQH
	mov	word ptr [si].wTkTransDesp, offset TimerTD
	mov	word ptr [si].wTkProc, offset Proc_Timer SMI_OFFSET
;R06 - end

	; Link Control Transfer Queue Head for future using
	; and no TD link

	mov	si, offset ControlQH
	call	Init_QH
	mov	word ptr [si].wQhInterval, 1
	call	Link_QH_to_FL

	ret

;[]========================================================================[]
;Procedure:	Host_Init
;Function:	Initialize host controller
;Input:		none
;Output:	none
;[]========================================================================[]

Host_Init:

	call	Ct_USB_ID
	mov	wHostID, ax

	mov	cx, wHostID
	add	cx, LEGACY_LO
	F000_call	Get_PCI
ifdef	IOTRAP_SUPPORT				;R22A
	or	al, USBSMIEN+0Fh		;R22A;Enable Trap/SMI on USB IRQ
else	;IOTRAP_SUPPORT				;R22A
	or	al, USBSMIEN			;Enable Trap/SMI on USB IRQ
endif	;IOTRAP_SUPPORT				;R22A
	F000_call	Set_PCI

	mov	cx, wHostID
	add	cx, (USBBASE+1)
	F000_call	Get_PCI
	push	ax
	mov	cx, wHostID
	add	cx, USBBASE
	F000_call	Get_PCI
	pop	dx
	mov	ah, dl
	and	al, 0FCh
	mov	ds:wHostIoBase, ax

	; Global Reset

	call	Global_Reset

	; Clear USB Status Register

	mov	ax, 0FFFFh			;write clear
	mov	dx, USBSTS
	call	Set_Host_Word

	; USB Interrupt Enable Register

	mov	ax, IOC				;interrupt on complete
	mov	dx, USBINTR
	call	Set_Host_Word

	; Frame Number Register

	mov	ax, 0				;Starting from frame 0
	mov	dx, FRNUM
	call	Set_Host_Word

	; Frame List Base Address Register

	mov	ax, ds
	shl	eax, 4
	add	eax, offset FrameList
	mov	dx, FRBASEADD
	call	Set_Host_Dword

	ret

;[]========================================================================[]
;Procedure:	Global_Reset
;Function:	Global host controller reset
;Input:		none
;Output:	none
;[]========================================================================[]

Global_Reset:
	pusha

	; Issue global reset and set maximum packet size to 64

	mov	al, GRESET+MAX_PACKET		;Global reset/max packet=64
	mov	dx, USBCMD
	call	Set_Host_Byte

	; Minimum delay time 10ms

	call	Delay_12ms

	; Clear reset signal

	mov	al, MAX_PACKET			;max packet=64
	mov	dx, USBCMD
	call	Set_Host_Byte

	popa
	ret

;R14 - start
;[]========================================================================[]
;Procedure:	Host_Reset
;Function:	Host controller reset
;Input:		none
;Output:	none
;[]========================================================================[]

Host_Reset:
	pusha

	; Issue host controller reset

	mov	al, HCRESET			;Host controller reset
	mov	dx, USBCMD
	call	Set_Host_Byte

	popa
	ret
;R14 - end

	assume	ds:nothing

ENDIF	;COMPILE_FOR_USBBIOS EQ 2

IF	COMPILE_FOR_USBBIOS EQ 3

;****************************************************************
;*								*
;*	COMPILING STAGE 3					*
;*								*
;*	RUN-TIME EXECUTION CODES RESIDENT AT SMI RAM		*
;*								*
;****************************************************************

	assume	ds:USB_RAM

;[]========================================================================[]
;Procedure:	Usb_Smi
;Function:	Process SMI generated by USB Host
;Input:		none
;Output:	CF = 0 USB SMI
;		CF = 1 not USB SMI
;[]========================================================================[]

	public	Usb_Smi
Usb_Smi proc	near
	pushad					;R07
	push	ds				;R07
	push	es				;R07
	call	Check_USB_Disabled
	jnz	usb_smi_exit

	call	Get_USBRAM
	mov	ds, ax
	mov	es, ax
	test	byte ptr ds:bUsbFlag, 02
	jnz	usb_smi_exit

	call	Check_IoBase_Disabled		;R10
	jc	short Disable_Usb_Smi		;R10

;R22A - start
ifdef	IOTRAP_SUPPORT
	call	Ct_CheckUsbLegacySmi
	jc	short @f
	mov	cx, wHostID
	add	cx, LEGACY_HI
	SMI_F0CALL	Get_Pci
	test	al, 0Fh
	jz	short @f
	call	UsbLegSmi
	call	Ct_ClearUsbLegacySmi
	jmp	Usb_Smi_End
@@:
endif	;IOTRAP_SUPPORT
;R22A - end

;R28 - start
	mov	cx, wHostID
	add	cx, LEGACY_LO
	SMI_F0CALL	Get_Pci
	test	al, USBSMIEN
	jz	usb_smi_exit
;R28 - end

	mov	cx, wHostID
	add	cx, LEGACY_HI
	SMI_F0CALL	Get_Pci

	test	al, SMIBYUSB
	jz	usb_smi_exit

	mov	dx, USBSTS
	call	Get_Host_Byte
	IODELAY
;R13	test	al, USBINT			;int caused by Host
;R29	test	al, USBINT+UEI			;R13
	test	al, 00111011b			;R29;all interrupt
;R08	jz	short usb_smi_exit
	jz	usb_smi_exit			;R08

	call	Ct_USB_SMI_Entry

	mov	dx, FRBASEADD
	call	Get_Host_Dword
	and	ax, 0F000h			;discard reserved bits
	cmp	eax, ds:dFrameListAddr
	je	short @f
;R10	or	byte ptr ds:bUsbFlag, 02
;R10	mov	cx, wHostID
;R10	add	cx, LEGACY_LO
;R10	SMI_F0CALL	Get_Pci
;R10	and	al, not USBSMIEN
;R10	SMI_F0CALL	Set_Pci
	mov	cx, wHostID
	add	cx, LEGACY_HI
	SMI_F0CALL	Get_Pci
	or	al, USBPIRQDEN
	SMI_F0CALL	Set_Pci
Disable_Usb_Smi:				;R10
	mov	cx, wHostID			;R10
	add	cx, LEGACY_LO			;R10
	SMI_F0CALL	Get_Pci			;R10
ifdef	IOTRAP_SUPPORT				;R22A
	and	al, not (USBSMIEN+0Fh)		;R22A
else	;IOTRAP_SUPPORT				;R22A
	and	al, not USBSMIEN		;R10
endif	;IOTRAP_SUPPORT				;R22A
	SMI_F0CALL	Set_Pci			;R10
ifdef	IOTRAP_SUPPORT				;R22A
	call	Ct_DisableUsbLegacySmi		;R22A
endif	;IOTRAP_SUPPORT				;R22A
	or	byte ptr ds:bUsbFlag, 02	;R10
	jmp	short Toggle_SMI_Exit
@@:

	; clear INT

	mov	dx, USBSTS
	call	Get_Host_Word
	call	Set_Host_Word
;R29 - start
;R29A	test	al, 00111010b			;error interrupt
	test	al, 00111000b			;R29A;error interrupt
	jz	short Proc_UsbSmi
	mov	dx, PORTSC1
	call	Get_Host_Word
	test	al, CURR_CONN			;device connected
	jz	short @f
	call	Reset_RootPort
	mov	al, 1				;device 1
	mov	ah, 1				;port 1
	call	Release_ChildDev
@@:
	mov	dx, PORTSC2
	call	Get_Host_Word
	test	al, CURR_CONN			;device connected
	jz	short @f
	call	Reset_RootPort
	mov	al, 1				;device 1
	mov	ah, 2				;port 2
	call	Release_ChildDev
@@:
	call	Host_Run
Proc_UsbSmi:
;R29 - end

@@:						;R08
	and	byte ptr bUsbFlag, NOT 04h	;R08
	mov	si, offset wTaskLink
Usb_Smi_Loop:
	test	byte ptr bUsbFlag, 04		;R08
	jnz	short @b			;R08

	cmp	word ptr [si].wTkLink, 0FFFFh
	je	short Toggle_SMI_Exit

	mov	si, [si].wTkLink
	test	byte ptr [si].bTkDevStatus, TK_WFP
	jz	short Usb_Smi_Next

	pushad
	call	word ptr [si].wTkProc
	popad

Usb_Smi_Next:
	jmp	short Usb_Smi_Loop

Toggle_SMI_Exit:

	CALL	Ct_USB_SMI_Exit

Usb_Smi_End:					;R22A
	pop	es				;R07
	pop	ds				;R07
	popad					;R07
	clc
	ret

usb_smi_exit:
	pop	es				;R07
	pop	ds				;R07
	popad					;R07
	stc
	ret
Usb_Smi endp

;[]========================================================================[]
;Procedure:	Check_USB_Disabled
;Function:	Check USB legacy support disabled either by device disabled
;		or by setup disabled.
;Input:		none
;Output:	ZF = 0 enabled
;		ZF = 1 disabled
;[]========================================================================[]

Check_USB_Disabled	proc	near
	push	ds
	mov	ax, 0F000h
	mov	ds, ax
	assume	ds:DGROUP
	test	byte ptr ds:USB_STATUS, 01
	pop	ds
	assume	ds:USB_RAM
	ret
Check_USB_Disabled	endp

;R10 - start
;[]========================================================================[]
;Procedure:	Check_IoBase_Disabled
;Function:	Check the I/O base address of USB Host Controller disabled.
;Input:		none
;Output:	CF = 0 no disabled
;		CF = 1 disabled
;[]========================================================================[]

Check_IoBase_Disabled	proc	near
	push	ax
	push	cx
	mov	cx, wHostID
	mov	cl, 04
	SMI_F0CALL	Get_Pci
	test	al, 01
	stc
	jz	short @f
	clc
@@:
	pop	cx
	pop	ax
	ret
Check_IoBase_Disabled	endp
;R10 - end

;[]========================================================================[]
;Procedure:	Get_USBRAM
;Function:	Get USB_RAM segment address
;Input:		none
;Output:	AX = USB_RAM segment address
;[]========================================================================[]

Get_USBRAM	proc	near
	push	ds
	mov	ax, 0F000h
	mov	ds, ax
	assume	ds:DGROUP
	mov	ax, ds:USB_RAM_SEG
	pop	ds
	assume	ds:USB_RAM
	ret
Get_USBRAM	endp

;[]========================================================================[]
;Procedure:	Wait_Stable
;Function:	Delay for device to be stable
;Input:		none
;Output:	none
;Notes:		Delay timer refer to Cherry USB keyboard stable time after
;		attached.
;[]========================================================================[]

Wait_Stable:
	push	bx
	push	cx

	xor	bx, bx
	mov	cx, 6000			;180 ms
	call	Usb_Wait_Refresh

	pop	cx
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Delay_12ms
;Function:	Delay for 12 ms
;Input:		none
;Output:	none
;[]========================================================================[]

Delay_12ms:
	push	bx
	push	cx

       	xor	bx, bx
	mov	cx, 400				;bx:cx=wait period
						; (400 -> 12 msec)
	call	Usb_Wait_Refresh

	pop	cx
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Delay_1ms
;Function:	Delay for 1 ms
;Input:		none
;Output:	none
;[]========================================================================[]

Delay_1ms:
	push	bx
	push	cx

       	xor	bx, bx
	mov	cx, 40				;bx:cx=wait period
						; (40 -> 1.2 msec)
	call	Usb_Wait_Refresh

	pop	cx
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Usb_Wait_Refresh
;Function:	Delay time with system DRAM refresh period
;Input:		BX:CX = delay counter (*30us)
;Output:	none
;[]========================================================================[]

	align	4
Usb_Wait_Refresh:
	pusha

	align	4
Wait_Refresh_Loop:
@@:
	in	al, 61h
	test	al, 10h
	jz	short @b

	align	4
@@:
	in	al, 61h
	test	al, 10h
	jnz	short @b

	loop	short Wait_Refresh_Loop

	or	bx, bx
	jz	short @f
	dec	bx
	jnz	short Wait_Refresh_Loop

@@:
	popa
	ret

;[]========================================================================[]
;Procedure:	System_Control
;Function:	System control task process routine
;Input:		SI = point to SysCtrlTask
;Output:	none
;[]========================================================================[]

System_Control:

	; Check port 1 status changed

Check_Port1:
	mov	dx, PORTSC1
	call	Get_Host_Word

;R28	push	ax
;R28	xor	al, byte ptr wPort1Status
;R28	test	al, CURR_CONN
;R28	pop	ax
;R28	jz	short Check_Port2
;R28
;R28	mov	wPort1Status, ax
;R28 - start
	test	al, CONN_CHANGE
	jz	short Check_Port2
	call	Set_Host_Word
;R28 - end

	test	al, CURR_CONN
	jnz	short Enable_RootPort1

	; Port 1 disconnected

Disable_RootPort1:

	call	Disable_RootPort
	mov	al, 1				;device 1
	mov	ah, 1				;port 1
	call	Release_ChildDev

	jmp	short Check_Port2

	; Port 1 connected
	; DX = port address
	; AX = port value

Enable_RootPort1:
;R28 - start
	push	ax
	mov	al, 1				;device 1
	mov	ah, 1				;port 1
	call	Release_ChildDev
	pop	ax
;R28 - end

	call	Reset_RootPort
	call	Enable_RootPort

	call	Get_CtrlBuffer			;R08
	jc	short Disable_RootPort1		;R08

	call	Suspend_Tasks			;suspend all tasks
						;for bus enumeration

;R08	mov	si, offset ControlBuffer1
;R08	call	Init_CtrlBuffer

	test	ax, LOW_SPEED_DEV
	jz	short @f
	or	byte ptr [si].bCtrlByte, CP_LS
@@:
	mov	byte ptr [si].bParentAddr, 1	;device 1
	mov	byte ptr [si].bParentPort, 1	;port 1
	mov	byte ptr [si].bDevAddr, 0	;default address 0
	mov	byte ptr [si].bDevEndp, 0	;default endpoint 0

	mov	bx, si
	call	Init_Device
	jnc	short @f

	and	al, not PORT_ENABLE
	call	Set_Host_Word

	jmp	short Check_Port1_Exit

@@:
	call	Init_Config			;BX=point to control buffer

Check_Port1_Exit:

	call	Release_CtrlBuffer		;R08A

	call	Resume_Tasks			;resume all tasks
						;after bus enumeration

	; Check port 2 status changed

Check_Port2:
	mov	dx, PORTSC2
	call	Get_Host_Word

;R28	push	ax
;R28	xor	al, byte ptr wPort2Status
;R28	test	al, CURR_CONN
;R28	pop	ax
;R28	jz	short Check_Led			;System_Control_Exit
;R28
;R28	mov	wPort2Status, ax
;R28 - start
	test	al, CONN_CHANGE
	jz	short Check_Led
	call	Set_Host_Word
;R28 - end

	test	al, CURR_CONN
	jnz	short Enable_RootPort2

	; Port 2 disconnected

Disable_RootPort2:

	call	Disable_RootPort
	mov	al, 1				;device 1
	mov	ah, 2				;port 2
	call	Release_ChildDev

	jmp	short Check_Led			;System_Control_Exit

	; Port 2 connected
	; DX = port address
	; AX = port value

Enable_RootPort2:
;R28 - start
	push	ax
	mov	al, 1				;device 1
	mov	ah, 2				;port 2
	call	Release_ChildDev
	pop	ax
;R28 - end

	call	Reset_RootPort
	call	Enable_RootPort

	call	Get_CtrlBuffer			;R08
	jc	short Disable_RootPort2		;R08

	call	Suspend_Tasks			;suspend all tasks
						;for bus enumeration

;R08	mov	si, offset ControlBuffer1
;R08	call	Init_CtrlBuffer

	test	ax, LOW_SPEED_DEV
	jz	short @f
	or	byte ptr [si].bCtrlByte, CP_LS
@@:
	mov	byte ptr [si].bParentAddr, 1	;device 1
	mov	byte ptr [si].bParentPort, 2	;port 2
	mov	byte ptr [si].bDevAddr, 0	;default address 0
	mov	byte ptr [si].bDevEndp, 0	;default endpoint 0

	mov	bx, si
	call	Init_Device
	jnc	short @f

	and	al, not PORT_ENABLE
	call	Set_Host_Word

	jmp	short Check_Port2_Exit

@@:
	call	Init_Config			;BX=point to control buffer

Check_Port2_Exit:

	call	Release_CtrlBuffer		;R08A

	call	Resume_Tasks			;resume all tasks
						;after bus enumeration

	; Check keyboard LED status changed

Check_Led:
ifndef	IOTRAP_SUPPORT				;R22A
ifndef	NO_UPDATE_USBKB_LED			;R18
	test	word ptr wKbdDevMap, 0FFFFh
	jz	short System_Control_Exit

	push	ds
	mov	ax, 0040h
	mov	ds, ax
	mov	al, byte ptr ds:[0017h]		;BIOS key status flag
	pop	ds

	push	ax
	xor	al, bLedStatus
	test	al, 01110000b			;LED bits
	pop	ax
	jz	short System_Control_Exit

	mov	bLedStatus, al

	call	Suspend_Tasks			;suspend all tasks
						;for LED updated
	call	Update_Led			;update keyboard LED

	call	Resume_Tasks			;resume all tasks
						;after LED updated
endif	;NO_UPDATE_USBKB_LED			;R18
endif	;IOTRAP_SUPPORT				;R22A
System_Control_Exit:

	ret

;R06 - start
;[]========================================================================[]
;Procedure:	Proc_Timer
;Function:	Periodic timer task process routine
;Input:		SI = point to TimerTask
;Output:	none
;[]========================================================================[]

Proc_Timer:
	test	word ptr wKbdDevMap, 0FFFFh
	jz	short Proc_Timer_Exit

	mov	di, offset wTaskLink
Proc_Keybuffer_Loop:
	cmp	word ptr [di].wTkLink, 0FFFFh
	je	short Proc_Keybuffer_End

	mov	di, [di].wTkLink
	test	byte ptr [di].bTkDevStatus, TK_KBD
	jz	short Proc_Keybuffer_Loop

	mov	si, [di].wTkCtrlBuffer

	cmp	byte ptr [si].bKeyLength, 0
	je	short Proc_Keybuffer_Loop

	call	Proc_Key_Buffer

	jmp	short Proc_Keybuffer_Loop

Proc_Keybuffer_End:

Proc_Timer_Exit:
	ret
;R06 - end

ifndef	NO_UPDATE_USBKB_LED			;R18
;[]========================================================================[]
;Procedure:	Update_Led
;Function:	Update all USB keyboard LED
;Input:		bLedStatus = current BIOS flag(40:17)
;Output:	none
;[]========================================================================[]

Update_Led:

	movzx	bx, byte ptr bLedStatus
	shr	bx, 4
	and	bl, 00000111b
	add	bx, offset Led_Table SMI_OFFSET
	mov	al, byte ptr cs:[bx]
	mov	byte ptr MiscDataBuffer, al

;R08	mov	si, offset ControlBuffer1
;R08	call	Init_CtrlBuffer
	call	Get_CtrlBuffer			;R08
	jc	short Update_Led_Exit		;R08

	mov	byte ptr [si].wTotalSize, 1
	mov	byte ptr [si].wMaxSize, 1
	mov	word ptr [si].wDataBuffer, offset MiscDataBuffer

	mov	bx, si

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
;R27	mov	byte ptr [si].bmRequestType, 00100010b
	mov	byte ptr [si].bRequest, SET_REPORT
	mov	byte ptr [si+1].wValue, 2		;type output
	mov	byte ptr [si].wLength, 1

	mov	[bx].wReqBuffer, si

	mov	di, offset wTaskLink
Update_Led_Loop:
	cmp	word ptr [di].wTkLink, 0FFFFh
	je	short Update_Led_End

	mov	di, [di].wTkLink
	test	byte ptr [di].bTkDevStatus, TK_KBD
	jz	short Update_Led_Next

;R01	mov	al, [di].bDevEndp
;R27	mov	al, [di].bTkDevEndp			;R01
;R27	mov	byte ptr [si].wIndex, al
;R27 - start
	mov	byte ptr [si].bmRequestType, 00100010b
	mov	al, [di].bTkDevEndp
	test	byte ptr [di].bTkDevStatus, TK_NEWKBD
	jz	short @f
	mov	byte ptr [si].bmRequestType, 00100001b
	mov	al, [di].bTkDevInterface
@@:
	mov	byte ptr [si].wIndex, al
;R27 - end

	and	byte ptr [bx].bCtrlByte, not CP_LS
	test	byte ptr [di].bTkDevStatus, TK_LS
	jz	short @f
	or	byte ptr [bx].bCtrlByte, CP_LS
@@:

	mov	al, [di].bTkDevAddr
	mov	[bx].bDevAddr, al

	call	Control_Transfer

Update_Led_Next:
	jmp	short Update_Led_Loop


Update_Led_End:
	mov	si, bx				;R08
	call	Release_CtrlBuffer		;R08
Update_Led_Exit:				;R08
	ret

; LED translation table from USB keyboard format to AT keyboard format

Led_Table	label	byte
	db	000b, 100b, 001b, 101b, 010b, 110b, 011b, 111b
endif	;NO_UPDATE_USBKB_LED			;R18

;[]========================================================================[]
;Procedure:	Control_Transfer
;Function:	Control transfer
;Input:		BX = point to control parameter block
;Output:	CF = 0 successful
;		CF = 1 fail
;[]========================================================================[]

Control_Transfer:
	pushad

	mov	di, offset ControlQH

	mov	si, offset ControlTD
	call	Init_TD

	; SETUP stage

	mov	byte ptr [si].bTdControl, DEF_TDCTRL
	test	byte ptr [bx].bCtrlByte, CP_LS
	jz	short @f
	or	byte ptr [si].bTdControl, LS
@@:

	mov	edx, 00E00000h+PID_SETUP	;max length 8
	movzx	eax, byte ptr [bx].bDevAddr
	shl	eax, BIT_ADDR
	or	edx, eax
	movzx	eax, byte ptr [bx].bDevEndp
	shl	eax, BIT_EP
	or	edx, eax
	mov	[si].dTdToken, edx

	mov	ax, [bx].wReqBuffer
	call	Physical_Linear
	mov	[si].dTdPoint, eax

	mov	byte ptr [si].bTdStatus, STS_ACTIVE

	call	Delay_1ms			;wait device stable

	call	Link_TD_to_QH

	call	Wait_Ack
	jc	Control_Transfer_Fail

	; DATA stage

	cmp	word ptr [bx].wTotalSize, 0
	je	short Status_Stage

	mov	edx, [si].dTdToken
	mov	dl, PID_IN
	test	byte ptr [bx].bCtrlByte, CP_RD
	jnz	short @f
	mov	dl, PID_OUT
@@:
	and	edx, NOT MAX_LEN
	movzx	eax, word ptr [bx].wMaxSize
	dec	ax
	shl	eax, BIT_MAXL
	or	edx, eax
	mov	[si].dTdToken, edx

	xor	dx, dx
	mov	cx, [bx].wTotalSize

Data_Stage_Loop:
	xor	dword ptr [si].dTdToken, DATA_TOGGLE

	mov	ax, [bx].wDataBuffer
	add	ax, dx
	call	Physical_Linear
	mov	[si].dTdPoint, eax

	or	byte ptr [si].bTdControl, C_ERR		;error count 3
	mov	byte ptr [si].bTdStatus, STS_ACTIVE

	call	Delay_1ms			;wait device stable

	call	Link_TD_to_QH

	call	Wait_Ack
	jc	short Control_Transfer_Fail

	add	dx, [bx].wMaxSize
	cmp	dx, cx
	jb	short Data_Stage_Loop

	; STATUS stage

Status_Stage:
	mov	edx, [si].dTdToken
	mov	dl, PID_IN
	cmp	word ptr [bx].wTotalSize, 0
	je	short @f
	test	byte ptr [bx].bCtrlByte, CP_RD
	jz	short @f
	mov	dl, PID_OUT
@@:
	or	edx, DATA_TOGGLE			;terminated with DATA1
	or	edx, MAX_LEN				;null packet
	mov	[si].dTdToken, edx

	or	byte ptr [si].bTdControl, C_ERR		;error count 3
	mov	byte ptr [si].bTdStatus, STS_ACTIVE

	call	Delay_1ms			;wait device stable

	call	Link_TD_to_QH

	call	Wait_Ack
	jc	short Control_Transfer_Fail

	popad
	clc
	ret

Control_Transfer_Fail:

	call	Remove_TD_from_QH

	popad
	stc
	ret

;[]========================================================================[]
;Procedure:	Wait_Ack
;Function:	Wait transaction acknowledge
;Input:		SI = point to TD
;Output:	CF = 0 successful
;		CF = 1 fail (transaction error or time-out)
;[]========================================================================[]

Wait_Ack:
	push	cx

	mov	cx, WAIT_ACK_COUNT
@@:
	test	byte ptr [si].bTdStatus, STS_ACTIVE
	jz	short @f

	call	Delay_1ms

	loop	short @b

Wait_Ack_Fail:

	pop	cx
	stc
	ret

@@:
	test	byte ptr [si].bTdStatus, STS_ERROR
	jnz	short Wait_Ack_Fail

	pop	cx
	clc
	ret

;[]========================================================================[]
;Procedure:	Link_QH_to_FL
;Function:	Link Queue Head to Fram List
;Input:		SI = point to QH
;Output:	none
;[]========================================================================[]

Link_QH_to_FL:
	pushad

	mov	ax, [si].wQhInterval
;R08	call	Validate_Interval
;R08	mov	[si].wQhInterval, ax

	mov	cx, ax				;interval
	or	cx, cx				;avoid coding mistake
	jz	short Link_QF_Exit
	xor	dx, dx				;frame point

Link_QF_Loop:
	push	dx

	mov	ax, 4
	mul	dx
	mov	di, offset FrameList
	add	di, ax

Link_QF_Check:
	mov	eax, [di].dQhLink		;link pointer

	test	al, T				;terminate
	jnz	short Link_QF_Append

	call	Physical_Offset			;get offset in USB_RAM
	mov	bx, ax

	cmp	si, bx
	je	short Link_QF_Next

	cmp	cx, [bx].wQhInterval
	jae	short Link_QF_Insert

	mov	di, bx				;next queue head address
	jmp	short Link_QF_Check

Link_QF_Insert:
	mov	eax, [di].dQhLink
	mov	[si].dQhLink, eax
	jmp	short Link_QF_Previous

Link_QF_Append:
	or	byte ptr [si].dQhLink, T	;terminate

Link_QF_Previous:
	mov	ax, si
	call	Physical_Linear
	or	al, Q				;queue head
	and	al, not T			;no terminate
	mov	[di].dQhLink, eax

Link_QF_Next:
	pop	dx

	add	dx, cx				;next interval
	cmp	dx, 1024			;maximum interval
	jb	short Link_QF_Loop

Link_QF_Exit:
	popad
	ret

;[]========================================================================[]
;Procedure:	Link_TD_to_QH
;Function:	Link Transfer Descriptor to Queue Head
;Input:		SI = point to TD
;		DI = point to QH
;Output:	none
;[]========================================================================[]

Link_TD_to_QH:
	push	eax
	mov	ax, si
	call	Physical_Linear
	mov	[di].dQhElement, eax
	pop	eax
	ret

;[]========================================================================[]
;Procedure:	Remove_QH_from_FL
;Function:	Remove Queue Head from Fram List
;Input:		SI = point to QH
;Output:	none
;[]========================================================================[]

Remove_QH_from_FL:
	pushad

	mov	cx, [si].wQhInterval		;interval
	xor	dx, dx				;frame point

Remove_QF_Loop:
	push	dx

	mov	ax, 4
	mul	dx
	mov	di, offset FrameList
	add	di, ax

Remove_QF_Check:
	mov	eax, [di].dQhLink		;link pointer

	test	al, T
	jnz	short Remove_QF_Next

	call	Physical_Offset

	cmp	ax, si
	je	short Do_Remove_QF

	mov	di, ax
	jmp	short Remove_QF_Check

Do_Remove_QF:
	or	byte ptr [di].dQhLink, T	;terminate
	test	byte ptr [si].dQhlink, T	;terminate
	jnz	short Remove_QF_Next

	mov	eax, [si].dQhLink		;merge next task
	mov	[di].dQhLink, eax

Remove_QF_Next:
	pop	dx

	add	dx, cx				;next interval
	cmp	dx, 1024			;maximum interval
	jb	short Remove_QF_Loop

	popad
	ret

;[]========================================================================[]
;Procedure:	Remove_TD_from_QH
;Function:	Remove Transfer Descriptor from Queue Head
;Input:		SI = point to TD
;		DI = point to QH
;Output:	none
;[]========================================================================[]

Remove_TD_from_QH:
	or	byte ptr [di].dQhElement, T
	ret

;[]========================================================================[]
;Procedure:	Init_FL
;Function:	Initialize Frame List
;Input:		none
;Output:	none
;[]========================================================================[]

Init_FL:
	pushad

	; Clear frame list

	mov	cx, 1024			;Number of entries
	xor	esi, esi
@@:
	mov	dword ptr FrameList[esi*4], T
        inc	esi
	loop	short @b

	popad
	ret

;[]========================================================================[]
;Procedure:	Init_QH
;Function:	Initialize Queue Head data block
;Input:		SI = point to queue head
;Output:	none
;[]========================================================================[]

Init_QH:
	pusha
;R08	push	si
;R08	xor	al, al
;R08	mov	cx, SIZE QH
;R08@@:
;R08	mov	[si], al
;R08	inc	si
;R08	loop	short @b
;R08	pop	si
	mov	cx, SIZE QH			;R08
	call	Clear_Buffer			;R08
	or	byte ptr [si].dQhLink, T
	or	byte ptr [si].dQhElement, T
	popa
	ret

;[]========================================================================[]
;Procedure:	Init_TD
;Function:	Initialize Transfer Descriptor data block
;Input:		SI = point to transfer descriptor
;Output:	none
;[]========================================================================[]

Init_TD:
	pusha
;R08	push	si
;R08	xor	al, al
;R08	mov	cx, SIZE TD
;R08@@:
;R08	mov	[si], al
;R08	inc	si
;R08	loop	short @b
;R08	pop	si
	mov	cx, SIZE TD			;R08
	call	Clear_Buffer			;R08
	or	byte ptr [si].dTdLink, T
	popa
	ret

;[]========================================================================[]
;Procedure:	Host_Stop
;Function:	Stop host controller working
;Input:		none
;Output:	none
;[]========================================================================[]

Host_Stop:
	push	dx
	mov	dx, USBCMD
	call	Get_Host_Word
	and	ax, not RS
	call	Set_Host_Word
	pop	dx
	ret

;[]========================================================================[]
;Procedure:	Host_Run
;Function:	Start host controller working
;Input:		none
;Output:	none
;[]========================================================================[]

Host_Run:
	push	dx
	mov	dx, USBCMD
	call	Get_Host_Word
	or	ax, RS
	call	Set_Host_Word
	pop	dx
	ret

;[]========================================================================[]
;Procedure:	Reset_RootPort
;Function:	Reset root port
;Input:		DX = Port address to reset
;Output:	none
;[]========================================================================[]

Reset_RootPort:
	push	ax

	; Issue port reset

	call	Get_Host_Word			;ax=status of root port 1 or 2
	or	ax, PORT_RESET			;Issue RESET for about 10 ms
	call	Set_Host_Word

	; Delay for USB reset signaling is active

	call	Delay_12ms

	; Inactivate reset signal

       	and	ax, not PORT_RESET
	call	Set_Host_Word

	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Enable_RootPort
;Function:	Enable root port
;Input:		DX = Port address to enable
;Output:	none
;[]========================================================================[]

Enable_RootPort:
	push	ax

	call	Get_Host_Word			;ax=status of root port 1 or 2
	or	ax, PORT_ENABLE
	call	Set_Host_Word

	; Unknown reason must be set twice

	NEWIODELAY
	NEWIODELAY
	NEWIODELAY
	NEWIODELAY

	call	Set_Host_Word

	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Disable_RootPort
;Function:	Disable root port
;Input:		DX = Port address to disable
;Output:	none
;[]========================================================================[]

Disable_RootPort:
	push	ax

	call	Get_Host_Word			;ax=status of root port 1 or 2
	and	ax, not PORT_ENABLE
	call	Set_Host_Word

	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Get_Host_Byte
;Function:	Get host controller I/O register value with byte
;Input:		DX = Host I/O register offset
;Output:	AL = value
;[]========================================================================[]

Get_Host_Byte:
	push	dx
	add	dx, wHostIoBase
	in	al, dx
	pop	dx
	ret

;[]========================================================================[]
;Procedure:	Get_Host_Word
;Function:	Get host controller I/O register value with word
;Input:		DX = Host I/O register offset
;Output:	AX = value
;[]========================================================================[]

Get_Host_Word:
	push	dx
	add	dx, wHostIoBase
	in	ax, dx
	pop	dx
	ret

;[]========================================================================[]
;Procedure:	Get_Host_Dword
;Function:	Get host controller I/O register value with double word
;Input:		DX = Host I/O register offset
;Output:	EAX = value
;[]========================================================================[]

Get_Host_Dword:
	push	dx
	add	dx, wHostIoBase
	in	eax, dx
	pop	dx
	ret

;[]========================================================================[]
;Procedure:	Set_Host_Byte
;Function:	Set host controller I/O register value with byte
;Input:		DX = Host I/O register offset
;		AL = value
;Output:	none
;[]========================================================================[]

Set_Host_Byte:
	push	dx
	add	dx, wHostIoBase
	out	dx, al
	pop	dx
	ret

;[]========================================================================[]
;Procedure:	Set_Host_Word
;Function:	Set host controller I/O register value with word
;Input:		DX = Host I/O register offset
;		AX = value
;Output:	none
;[]========================================================================[]

Set_Host_Word:
	push	dx
	add	dx, wHostIoBase
	out	dx, ax
	pop	dx
	ret

;[]========================================================================[]
;Procedure:	Set_Host_Dword
;Function:	Set host controller I/O register value with double word
;Input:		DX = Host I/O register offset
;		EAX = value
;Output:	none
;[]========================================================================[]

Set_Host_Dword:
	push	dx
	add	dx, wHostIoBase
	out	dx, eax
	pop	dx
	ret


;************************************************************************
;*									*
;*	GENERAL DEVICE MANAGEMENT					*
;*									*
;************************************************************************

;[]========================================================================[]
;Procedure:	Init_Device
;Function:	Get device descriptor and set address for attached device
;Input:		BX = point to control parameter block
;			bCtrlByte contained low speed device information
;			bParentAddr conatined upstream device address
;			bParentPort contained upstream device port number
;Output:	CF = 0 successful
;			bDevAddr contained device address
;			DevDespBuffer contained device descriptor
;		CF = 1 fail
;[]========================================================================[]

Init_Device:
	pushad

	call	Wait_Stable			;wait device stable

	; First time to get device descriptor (8 bytes)

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	byte ptr [si].bmRequestType, 10000000b
	mov	byte ptr [si].bRequest, GET_DESCRIPTOR
	mov	byte ptr [si+1].wValue, DEVICE_DESCRIPTOR
	mov	word ptr [si].wLength, 8

	mov	word ptr [bx].wReqBuffer, si
	or	byte ptr [bx].bCtrlByte, CP_RD
	mov	word ptr [bx].wDataBuffer, offset DevDespBuffer
	mov	word ptr [bx].wTotalSize, 8
	mov	word ptr [bx].wMaxSize, 8
	call	Control_Transfer
	jc	short Init_Device_Fail

	; Set device address

	call	Query_Addr			;query available device address
	jc	short Init_Device_Fail

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	byte ptr [si].bmRequestType, 0
	mov	byte ptr [si].bRequest, SET_ADDRESS
	mov	byte ptr [si].wValue, al	;device address

	mov	[bx].wReqBuffer, si
	mov	word ptr [bx].wTotalSize, 0	;null data control transfer
	call	Control_Transfer
	jc	short Init_Device_Fail

	mov	byte ptr [bx].bDevAddr, al	;device address
	mov	dl, [bx].bParentAddr		;upstream device address
	mov	dh, [bx].bParentPort		;upstream device port
	call	Record_Addr			;record device address

	; Second time to get device descriptor (total bytes)

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	byte ptr [si].bmRequestType, 10000000b
	mov	byte ptr [si].bRequest, GET_DESCRIPTOR
	mov	byte ptr [si+1].wValue, DEVICE_DESCRIPTOR
	movzx	ax, byte ptr DevDespBuffer.DDbLength		;total length
	mov	word ptr [si].wLength, ax

	mov	word ptr [bx].wReqBuffer, si
	or	byte ptr [bx].bCtrlByte, CP_RD
	mov	word ptr [bx].wDataBuffer, offset DevDespBuffer
	mov	word ptr [bx].wTotalSize, ax
	movzx	ax, byte ptr DevDespBuffer.DDbMaxPackSize	;packet size
	mov	word ptr [bx].wMaxSize, ax
	call	Control_Transfer
	jc	short Init_Device_Fail

	popad
	clc
	ret

Init_Device_Fail:
	popad
	stc
	ret

;[]========================================================================[]
;Procedure:	Init_Config
;Function:	Get configuration descriptor and set configuration for BIOS
;		supported device
;Input:		BX = point to control parameter block
;			bCtrlByte contained low speed device information
;			bParentAddr conatined upstream device address
;			bParentPort contained upstream device port number
;			bDevAddr contained device address
;		DevDespBuffer contained device descriptor
;Output:	CF = 0 successful
;		CF = 1 fail
;[]========================================================================[]

Init_Config:
	pushad

	xor	cl, cl					;index

Init_Config_Loop:

	; First time to get configuration descriptor (8 bytes)

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	byte ptr [si].bmRequestType, 10000000b
	mov	byte ptr [si].bRequest, GET_DESCRIPTOR
	mov	byte ptr [si].wValue, cl
	mov	byte ptr [si+1].wValue, CONFIG_DESCRIPTOR
	mov	word ptr [si].wLength, 8

	mov	word ptr [bx].wReqBuffer, si
	or	byte ptr [bx].bCtrlByte, CP_RD
	mov	word ptr [bx].wDataBuffer, offset CfgDespBuffer
	mov	word ptr [bx].wTotalSize, 8
	mov	word ptr [bx].wMaxSize, 8
	call	Control_Transfer
	jc	Init_Config_Fail

	; Second time to get configuration descriptor (total bytes)

	mov	ax, word ptr CfgDespBuffer.CDwTotalLen		;total length
	cmp	ax, MAX_DESP_LEN
	jbe	short @f
	mov	ax, MAX_DESP_LEN
@@:
	mov	word ptr [si].wLength, ax

	mov	word ptr [bx].wTotalSize, ax
	movzx	ax, byte ptr DevDespBuffer.DDbMaxPackSize	;packet size
	mov	word ptr [bx].wMaxSize, ax
	call	Control_Transfer
	jc	short Init_Config_Fail

	push	cx

	mov	cx, BIOS_Device_Num
	mov	dx, offset Init_Proc SMI_OFFSET
	mov	di, offset BIOS_Device SMI_OFFSET
	mov	si, offset CfgDespBuffer.StdIDsp	;interface descriptor

Init_BIOS_Device_Loop:

	mov	al, cs:[di]
	cmp	al, [si].IDbClass
	jne	short Next_BIOS_Device
	mov	al, cs:[di+1]
	cmp	al, 0FFh			;R08
	je	short BIOS_Device_Found		;R08
	cmp	al, [si].IDbSubClass
	jne	short Next_BIOS_Device
	mov	al, cs:[di+2]
	cmp	al, 0FFh			;R08
	je	short BIOS_Device_Found		;R08
	cmp	al, [si].IDbProtocol
	jne	short Next_BIOS_Device

BIOS_Device_Found:				;R08

	pop	cx

	mov	di, dx

	; Set configuration

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	byte ptr [si].bmRequestType, 0
	mov	byte ptr [si].bRequest, SET_CONFIGURATION
	mov	al, CfgDespBuffer.CDbCfgValue
	mov	byte ptr [si].wValue, al

	mov	[bx].wReqBuffer, si
	mov	word ptr [bx].wTotalSize, 0	;null data control transfer
	call	Control_Transfer
	jc	short Init_Config_Exit

	; Call corresponding procedure to initialize BIOS devices

	pushad
	call	word ptr cs:[di]
	popad
	jmp	short Init_Config_Exit

Next_BIOS_Device:
	add	di, 3
	add	dx, 2
	loop	short Init_BIOS_Device_Loop

	pop	cx

Init_Config_Next:
	inc	cl				;next configuration
	cmp	cl, DevDespBuffer.DDbNumConfig	;number of config
	jb	Init_Config_Loop

Init_Config_Exit:
	popad
	clc
	ret

Init_Config_Fail:
	popad
	stc
	ret

BIOS_Device	label	byte
;		Class, SubClass, Protocol
;R08	db	09, 01, 00			;Hub
	db	09, 0FFh, 0FFh			;R08;Hub
	db	03, 01, 01			;Keyboard
BIOS_Device_Num	equ	($ - BIOS_Device)/3

Init_Proc	label	word
	dw	offset Init_Hub SMI_OFFSET
	dw	offset Init_Keyboard SMI_OFFSET

;[]========================================================================[]
;Procedure:	Release_ChildDev
;Function:	Release downstream device address and remove device task
;		if existed in Task Link
;Input:		AL = device number
;		AH = port number (0=all ports)
;Output:	none
;[]========================================================================[]

Release_ChildDev:
	pusha

	mov	cx, 127
	xor	dl, dl				;device address count
	mov	si, offset DevAddrMap+2

Release_ChildDev_Loop:

	inc	dl				;next device address

	push	ax
	xor	al, [si]
	cmp	al, 80h				;address occupied and
	jne	short Release_ChildDev_Next	;matched?

	or	ah, ah				;all ports?
	jz	short @f			;yes

	xor	ah, [si+1]			;port match?
	jnz	short Release_ChildDev_Next	;no

@@:
	and	byte ptr [si], 7Fh		;clear occupied bit

	mov	al, dl
	call	Remove_DevTask			;AL=dev addr

	call	Release_ChildDev		;AL=dev addr, AH=0:all ports

Release_ChildDev_Next:

	pop	ax

	inc	si				;next DevAddrMap
	inc	si
	loop	short Release_ChildDev_Loop

	popa
	ret

;[]========================================================================[]
;Procedure:	Link_Task
;Function:	Link (Append/Insert) one task to the Task Link
;Input:		SI = point to task
;Output:	none
;[]========================================================================[]

Link_Task:
	pusha

	or	byte ptr bUsbFlag, 04		;R08;task link updated

	; Link Task to Task Chain

	mov	ax, [si].wTkInterval
;R08	call	Validate_Interval
;R08	mov	[si].wTkInterval, ax

	mov	cx, ax				;interval
	or	cx, cx				;avoid coding mistake
	jz	short Link_TK_Exit

	mov	di, offset wTaskLink		;task link point

Link_TK_Loop:
	cmp	word ptr [di].wTkLink, 0FFFFh
	je	short Link_TK_Append

	mov	bx, [di].wTkLink

	cmp	cx, [bx].wTkInterval
	jb	short Link_TK_Insert

	mov	di, bx
	jmp	short Link_TK_Loop

Link_TK_Insert:
	mov	ax, [di].wTkLink
	mov	[si].wTkLink, ax
	jmp	short Link_TK_Previous

Link_TK_Append:
	mov	word ptr [si].wTkLink, 0FFFFh	;terminate

Link_TK_Previous:
	mov	[di].wTkLink, si

	; Link Queue Head to Frame List

	mov	ax, [si].wTkInterval
	mov	di, [si].wTkTransDesp
	mov	si, [si].wTkQueueHead
	mov	[si].wQhInterval, ax
	call	Link_QH_to_FL

;R06 - start
	test	word ptr wKbdDevMap, 0FFFFh
	jz	short Link_TK_Exit

	test	bUsbFlag, 08
	jnz	short Link_TK_Exit

	or	bUsbFlag, 08
	mov	si, offset TimerTask
	call	Link_Task
;R06 - end

Link_TK_Exit:
	popa
	ret

;[]========================================================================[]
;Procedure:	Remove_Task
;Function:	Remove one task from the Task Link
;Input:		SI = point to task
;Output:	none
;[]========================================================================[]

Remove_Task:
	pusha

	; Remove Task from Task Chain

	mov	di, offset wTaskLink

Remove_TK_Loop:
	cmp	word ptr [di].wTkLink, 0FFFFh
	je	short Remove_TK_Exit

	cmp	si, [di].wTkLink
	je	short Go_Remove_TK

	mov	di, [di].wTkLink
	jmp	short Remove_TK_Loop

Go_Remove_TK:
	mov	ax, [si].wTkLink
	mov	[di].wTkLink, ax

	call	Do_Remove_Task

Remove_TK_Exit:
	popa
	ret

;[]========================================================================[]
;Procedure:	Remove_DevTask
;Function:	Remove one task from the Task Link according to device address
;Input:		AL = device address
;Output:	none
;[]========================================================================[]

Remove_DevTask:
	pusha

	; Remove Task from Task Chain

	mov	di, offset wTaskLink

Remove_DTK_Loop:
	cmp	word ptr [di].wTkLink, 0FFFFh
	je	short Remove_DTK_Exit

	mov	si, [di].wTkLink

	cmp	al, [si].bTkDevAddr
	je	short Go_Remove_DTK

	mov	di, si
	jmp	short Remove_DTK_Loop

Go_Remove_DTK:
	mov	ax, [si].wTkLink
	mov	[di].wTkLink, ax

	call	Do_Remove_Task

Remove_DTK_Exit:
	popa
	ret

;[]========================================================================[]
;Procedure:	Do_Remove_Task
;Function:	Remove TD and QH, release BIOS supported device map
;Input:		SI = point to task be removed
;Output:	none
;[]========================================================================[]

Do_Remove_Task:

	or	byte ptr bUsbFlag, 04		;R08;task link updated

	; Remove Transfer Descriptor from Queue Head

	push	si

	mov	di, [si].wTkQueueHead
	mov	si, [si].wTkTransDesp
	call	Remove_TD_from_QH

	; Remove Queue Head from Frame List

	mov	si, di
	call	Remove_QH_from_FL

	pop	si

	; Release index number of BIOS supported devices

	mov	ah, byte ptr [si].bTkDevStatus
	test	ah, (TK_HUB+TK_KBD+TK_MS)
	jz	short Do_Remove_Exit

	mov	al, [si].bTkDevIndex
	mov	bx, offset Release_Hub_Index SMI_OFFSET
	test	ah, TK_HUB
	jnz	short @f
	mov	bx, offset Release_Kbd_Index SMI_OFFSET
	test	ah, TK_KBD
	jnz	short @f
	mov	bx, offset Release_Ms_Index SMI_OFFSET
@@:
	call	bx

Do_Remove_Exit:

;R06 - start
	test	word ptr wKbdDevMap, 0FFFFh
	jnz	short @f

	test	bUsbFlag, 08
	jz	short @f

	and	bUsbFlag, 0F7h
	mov	si, offset TimerTask
	call	Remove_Task
@@:
;R06 - end

	ret

;[]========================================================================[]
;Procedure:	Init_Task
;Function:	Initialize task data block
;Input:		SI = point to task
;Output:	none
;[]========================================================================[]

Init_Task:
	pusha
;R08	xor	al, al
;R08	mov	cx, SIZE TASK
;R08@@:
;R08	mov	[si], al
;R08	inc	si
;R08	loop	short @b
	mov	cx, SIZE TASK			;R08
	call	Clear_Buffer			;R08
	popa
	ret

;[]========================================================================[]
;Procedure:	Suspend_Tasks
;Function:	Suspend all tasks for Control Transfer
;Input:		none
;Output:	none
;[]========================================================================[]

Suspend_Tasks:
	pusha

	mov	bx, offset wTaskLink
Suspend_TK_Loop:
	cmp	word ptr [bx].wTkLink, 0FFFFh
	je	short Suspend_TK_Exit

	mov	bx, [bx].wTkLink
	mov	si, [bx].wTkTransDesp
	mov	di, [bx].wTkQueueHead
	call	Remove_TD_from_QH
	jmp	short Suspend_TK_Loop

Suspend_TK_Exit:
	popa
	ret

;[]========================================================================[]
;Procedure:	Resume_Tasks
;Function:	Resume all tasks after Control Transfer
;Input:		none
;Output:	none
;[]========================================================================[]

Resume_Tasks:
	pusha

	mov	bx, offset wTaskLink
Resume_TK_Loop:
	cmp	word ptr [bx].wTkLink, 0FFFFh
	je	short Resume_TK_Exit

	mov	bx, [bx].wTkLink
	mov	si, [bx].wTkTransDesp
	mov	di, [bx].wTkQueueHead
	call	Link_TD_to_QH
	jmp	short Resume_TK_Loop

Resume_TK_Exit:
	popa
	ret

;[]========================================================================[]
;Procedure:	Validate_Interval
;Function:	Validate the interval value
;		Allowed interval value in Award BIOS is the dimension of 2
;		e.g.1,2,4,8,16,32,64,128,256,512,1024
;Input:		AX = interval value reported by device
;Output:	AX = allowed interval value
;[]========================================================================[]

Validate_Interval:
	push	bx
	push	cx
	push	dx
	mov	bx, ax
	mov	ax, 1
	mov	cx, 10
@@:
	cmp	bx, ax
	jbe	short @f
	mov	dx, 2
	mul	dx
	loop	short @b
@@:
	pop	dx
	pop	cx
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Physical_Linear
;Function:	Calculate FL, QH and TD physical 32-bit linear address
;Input:		AX = offset value in USB_RAM
;Output:	EAX = physical 32-bit linear address value
;[]========================================================================[]

Physical_Linear:
	push	edx
	xor	edx, edx
	mov	dx, ds
	shl	edx, 4
	and	eax, 0000FFFFh
	add	eax, edx
	pop	edx
	ret

;[]========================================================================[]
;Procedure:	Physical_Offset
;Function:	Calculate 32-bit linear address to offset of USB_RAM
;Input:		EAX = physical 32-bit linear address value
;Output:	EAX = offset value in USB_RAM
;[]========================================================================[]

Physical_Offset:
	push	edx
	xor	edx, edx
	mov	dx, ds
	shl	edx, 4
	sub	eax, edx
	and	al, 0F0h			;don't care bit
	pop	edx
	ret

;[]========================================================================[]
;Procedure:	Query_Addr
;Function:	Query availible device address
;Input:		none
;Output:	CF = 1 no device address availible
;		CF = 0 availible device address found
;			AL = availible device address
;[]========================================================================[]

Query_Addr:
	push	cx
	push	si

	mov	cx, 127
	xor	ah, ah
	mov	si, offset DevAddrMap+2
Query_Addr_Loop:
	inc	ah
	lodsb
	test	al, 80h
	jz	short Query_Addr_Success
	inc	si
	loop	short Query_Addr_Loop
	stc
	jmp	short Query_Addr_Exit

Query_Addr_Success:
	mov	al, ah
	clc
Query_Addr_Exit:
	pop	si
	pop	cx
	ret

;[]========================================================================[]
;Procedure:	Record_Addr
;Function:	Record device address
;Input:		AL = device address to record
;		DL = parent device address
;		DH = parent device port
;Output:	none
;[]========================================================================[]

Record_Addr:
	push	bx
	push	dx
	movzx	bx, al
	shl	bx, 1
	or	dl, 80h
	mov	word ptr DevAddrMap[bx], dx
	pop	dx
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Release_Addr
;Function:	Release device address
;Input:		AL = device address to release
;Output:	none
;[]========================================================================[]

Release_Addr:
	push	bx
	movzx	bx, al
	shl	bx, 1
	and	byte ptr DevAddrMap[bx], 7Fh
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Query_Index
;Function:	Query availible hub/keyboard/mouse index number
;Input:		none
;Output:	CF = 1 no index number availible
;		CF = 0 availible index number found
;			AL = availible index number
;[]========================================================================[]

Query_Hub_Index:
	push	cx
	push	dx
	mov	cx, MAX_HUB
	mov	dx, wHubDevMap
	jmp	short Query_Index

Query_Kbd_Index:
	push	cx
	push	dx
	mov	cx, MAX_KBD
	mov	dx, wKbdDevMap
	jmp	short Query_Index

Query_Ms_Index:
	push	cx
	push	dx
	mov	cx, MAX_MS
	mov	dx, wMsDevMap

Query_Index:
	xor	al, al
@@:
	shr	dx, 1
	jnc	short @f
	inc	al
	loop	short @b

	pop	dx
	pop	cx
	stc
	ret
@@:
	pop	dx
	pop	cx
	clc
	ret

;[]========================================================================[]
;Procedure:	Record_Index
;Function:	Record hub/keyboard/mouse index number
;Input:		AL = index number
;Output:	none
;[]========================================================================[]

Record_Hub_Index:
	push	bx
	mov	bx, offset wHubDevMap
	jmp	short Record_Index

Record_Kbd_Index:
	push	bx
	mov	bx, offset wKbdDevMap
	jmp	short Record_Index

Record_Ms_Index:
	push	bx
	mov	bx, offset wMsDevMap

Record_Index:
	push	ax
	push	cx
	mov	cl, al
	mov	ax, 1
	rol	ax, cl
	or	[bx], ax
	pop	cx
	pop	ax
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Release_Index
;Function:	Release hub/keyboard/mouse index number
;Input:		AL = index number
;Output:	none
;[]========================================================================[]

Release_Hub_Index:
	push	bx
	mov	bx, offset wHubDevMap
	jmp	short Release_Index

Release_Kbd_Index:
	push	bx
	mov	bx, offset wKbdDevMap
	jmp	short Release_Index

Release_Ms_Index:
	push	bx
	mov	bx, offset wMsDevMap

Release_Index:
	push	ax
	push	cx
	mov	cl, al
	mov	ax, 0FFFEh
	rol	ax, cl
	and	[bx], ax
	pop	cx
	pop	ax
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Init_ReqBuffer
;Function:	Initialize device request data buffer
;Input:		SI = point to device request data buffer
;Output:	none
;[]========================================================================[]

Init_ReqBuffer:
	pusha
;R08	xor	al, al
;R08	mov	cx, SIZE DEVREQ
;R08@@:
;R08	mov	[si], al
;R08	inc	si
;R08	loop	short @b
	mov	cx, SIZE DEVREQ			;R08
	call	Clear_Buffer			;R08
	popa
	ret

;R08 - start
;[]========================================================================[]
;Procedure:	Get_CtrlBuffer
;Function:	Get availible control parameter block
;Input:		none
;Output:	CF = 1 no availible control buffer
;		CF = 0 found availible control buffer
;			SI = point to control parameter block
;[]========================================================================[]

Get_CtrlBuffer:
	push	ax
	push	cx
	push	dx

	mov	cx, MAX_CTLBUFF
	mov	dx, 1
	xor	ax, ax
@@:
	test	wCtrlBuffMap, dx
	jz	short @f
	shl	dx, 1
	inc	ax
	loop	short @b
	jmp	short Get_CtrlBuff_Fail

@@:
	or	wCtrlBuffMap, dx

	mov	si, SIZE CTLPARA
	mul	si
	mov	si, ax
	add	si, offset ControlBuffer
	call	Init_CtrlBuffer

	clc
	jmp	short Get_CtrlBuff_Exit

Get_CtrlBuff_Fail:
	stc

Get_CtrlBuff_Exit:
	pop	dx
	pop	cx
	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Release_CtrlBuffer
;Function:	Release control parameter block
;Input:		SI = point to control parameter block
;Output:	none
;[]========================================================================[]

Release_CtrlBuffer:
	pusha
	cmp	si, (offset ControlBuffer+((SIZE CTLPARA)*MAX_CTLBUFF))
	jae	short @f
	mov	ax, si
	sub	ax, offset ControlBuffer
	mov	cx, SIZE CTLPARA
	xor	dx, dx
	div	cx
	or	dx, dx
	jnz	short @f
	mov	cl, al
	mov	ax, 0FFFEh
	rol	ax, cl
	and	wCtrlBuffMap, ax
@@:
	popa
	ret
;R08 - end

;[]========================================================================[]
;Procedure:	Init_CtrlBuffer
;Function:	Initialize control parameter block
;Input:		SI = point to control parameter block
;Output:	none
;[]========================================================================[]

Init_CtrlBuffer:
	pusha
;R08	xor	al, al
;R08	mov	cx, SIZE CTLPARA
;R08@@:
;R08	mov	[si], al
;R08	inc	si
;R08	loop	short @b
	mov	cx, SIZE CTLPARA		;R08
	call	Clear_Buffer			;R08
	popa
	ret

;R08 - start
;[]========================================================================[]
;Procedure:	Clear_Buffer
;Function:	Clear miscellaneous data buffer
;Input:		SI = point to data buffer
;		CX = length
;Output:	none
;[]========================================================================[]

Clear_Buffer:
	pusha
	xor	al, al
@@:
	mov	[si], al
	inc	si
	loop	short @b
	popa
	ret
;R08 - end

;R24 - start
;[]========================================================================[]
;Procedure:	Find_EndpDesp
;Function:	Find endpoint descriptor
;Input:		DI = point to configuration descriptor
;Output:	CF = 0 descriptor found
;			DI = point to endpoint descriptor
;		CF = 1 descriptor not found
;[]========================================================================[]

Find_EndpDesp:
	push	ax
	push	cx
	mov	cx, [di].CDwTotalLen
Find_EndpDesp_Loop:
	cmp	byte ptr [di+1], 05		;descriptor type byte
	jne	short Find_EndpDesp_Next
	pop	cx
	pop	ax
	clc
	ret
Find_EndpDesp_Next:
	movzx	ax, byte ptr [di+0]		;descriptor length
	add	di, ax				;next descriptor
	sub	cx, ax
	jnb	short Find_EndpDesp_Loop
	pop	cx
	pop	ax
	stc
	ret
;R24 - end


;************************************************************************
;*									*
;*	HUB MANAGEMENT							*
;*									*
;************************************************************************

;[]========================================================================[]
;Procedure:	Init_Hub
;Function:	Initialize HUB
;Input:		BX = point to control parameter block
;Output:	none
;[]========================================================================[]

Init_Hub:

	; First time to get hub descriptor (8 bytes)

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	byte ptr [si].bmRequestType, 10100000b
	mov	byte ptr [si].bRequest, GET_DESCRIPTOR
	mov	byte ptr [si+1].wValue, HUB_DESCRIPTOR	;R25
	mov	word ptr [si].wLength, 8

	mov	word ptr [bx].wReqBuffer, si
	or	byte ptr [bx].bCtrlByte, CP_RD
	mov	word ptr [bx].wDataBuffer, offset ClassDespBuffer
	mov	word ptr [bx].wTotalSize, 8
	mov	word ptr [bx].wMaxSize, 8
	call	Control_Transfer
	jc	Init_Hub_Exit

	; Second time to get hub descriptor (total bytes)

	movzx	ax, byte ptr ClassDespBuffer.UDbLength		;total length
	cmp	ax, MAX_DESP_LEN
	jbe	short @f
	mov	ax, MAX_DESP_LEN
@@:
	mov	word ptr [si].wLength, ax

	mov	word ptr [bx].wTotalSize, ax
	movzx	ax, byte ptr DevDespBuffer.DDbMaxPackSize	;packet size
	mov	word ptr [bx].wMaxSize, ax
	call	Control_Transfer
	jc	Init_Hub_Exit

	call	Query_Hub_Index
	jc	Init_Hub_Exit

	call	Record_Hub_Index

	mov	dl, al					;hub index number

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	byte ptr [si].bmRequestType, 00100011b
	mov	byte ptr [si].bRequest, SET_FEATURE	;set port feature
	mov	word ptr [si].wValue, S_PORT_POWER	;set port power
	mov	word ptr [bx].wTotalSize, 0
	xor	ax, ax
	movzx	cx, byte ptr ClassDespBuffer.UDbNumPort
@@:
	inc	ax
	mov	[si].wIndex, ax
	call	Control_Transfer
	loop	short @b

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	byte ptr [si].bmRequestType, 10100011b
	mov	byte ptr [si].bRequest, GET_STATUS	;get port status
	mov	word ptr [si].wLength, 4

	or	byte ptr [bx].bCtrlByte, CP_RD
	mov	word ptr [bx].wDataBuffer, offset MiscDataBuffer
	mov	word ptr [bx].wTotalSize, 4
	mov	word ptr [bx].wMaxSize, 4
;R08	xor	ax, ax
;R08	xor	dl, dl
;R08	mov	dh, 02
	movzx	cx, byte ptr ClassDespBuffer.UDbNumPort

	push	cx
	push	dx				;R08;hub index number
	xor	ax, ax				;R08
	xor	dl, dl				;R08
	mov	dh, 02				;R08;start with port 1

Get_HubPort_Loop:
	inc	ax
	mov	[si].wIndex, ax
	call	Control_Transfer
	jc	short Get_HubPort_Next
	test	byte ptr MiscDataBuffer.HPSbStatus1, HPS_CONNECT
	jz	short Get_HubPort_Next
	or	dl, dh
Get_HubPort_Next:
	shl	dh, 1
	loop	short Get_HubPort_Loop

	pop	ax				;R08;hub index number
	push	dx
	mov	dx, ax				;R08;hub index number

	; Configure HUB interrupt transaction task
	; DL = hub index number

	mov	al, SIZE TASK
	mul	dl
	add	ax, offset HubIntTask
	mov	si, ax
	call	Init_Task

	mov	[si].bTkDevStatus, TK_WFP+TK_HUB

	mov	byte ptr [si].bTkDevIndex, dl

	mov	word ptr [si].wTkProc, offset Proc_Hub SMI_OFFSET

	mov	al, [bx].bDevAddr
	mov	[si].bTkDevAddr, al

	mov	al, [bx].bParentAddr
	mov	[si].bTkParentAddr, al

	mov	al, [bx].bParentPort
	mov	[si].bTkParentPort, al

	mov	di, offset CfgDespBuffer.StdEDsp

	movzx	ax, byte ptr [di].EDbInterval
	call	Validate_Interval		;R08
	mov	[si].wTkInterval, ax

	mov	ax, [di].EDwMaxPackSize
	mov	[si].wTkPacketSize, ax

	mov	al, byte ptr [di].EDbEndpAddr
	and	al, 7Fh
	mov	[si].bTkDevEndp, al

	push	si				;R08
	mov	al, 8
	movzx	cx, al
	mul	dl
	add	ax, offset HubDataBuffer
;R08	mov	di, ax
;R08	xor	al, al
;R08	rep	stosb
;R08	mov	[si].wTkDataBuffer, di
	mov	si, ax				;R08
	call	Clear_Buffer			;R08
	pop	si				;R08
	mov	[si].wTkDataBuffer, ax		;R08

	push	si				;R08
	mov	al, SIZE HUBCTRL
	movzx	cx, al
	mul	dl
	add	ax, offset HubCtrlBuffer
;R08	mov	di, ax
;R08	xor	al, al
;R08	rep	stosb
;R08
;R08	mov	al, ClassDespBuffer.UDbNumPort
;R08	mov	[di].bHcNumPort, al
;R08
;R08	mov	[si].wTkCtrlBuffer, di
	mov	si, ax				;R08
	call	Clear_Buffer			;R08
	mov	cl, ClassDespBuffer.UDbNumPort	;R08
	mov	[si].bHcNumPort, cl		;R08
	pop	si				;R08
	mov	[si].wTkCtrlBuffer, ax		;R08

	push	si
	mov	al, 16
	mul	dl
	add	ax, offset HubIntQH
	mov	si, ax
	call	Init_QH
	pop	si
	mov	[si].wTkQueueHead, ax

	push	si
	mov	al, SIZE TD
	mul	dl
	add	ax, offset HubIntTD
	mov	si, ax
	call	Init_TD
;R08	mov	di, si
	pop	si
;R08	mov	[si].wTkTransDesp, di
	mov	[si].wTkTransDesp, ax		;R08
	mov	di, ax				;R08

	mov	al, DEF_TDCTRL+ISS_IOC		;hub always high speed
	mov	[di].bTdControl, al

	mov	edx, PID_IN
	movzx	eax, word ptr [si].wTkPacketSize
	dec	eax
	shl	eax, BIT_MAXL
	or	edx, eax
	movzx	eax, byte ptr [si].bTkDevAddr
	shl	eax, BIT_ADDR
	or	edx, eax
	movzx	eax, byte ptr [si].bTkDevEndp
	shl	eax, BIT_EP
	or	edx, eax
	mov	[di].dTdToken, edx

	mov	ax, [si].wTkDataBuffer
	call	Physical_Linear
	mov	[di].dTdPoint, eax

	mov	byte ptr [di].bTdStatus, STS_ACTIVE

	call	Link_Task

	pop	ax
	pop	cx

	or	al, al
	jz	short Init_Hub_Exit

	; SI = point to hub task
	; AL = hub port status

	xor	ah, ah

Init_HubPort_Loop:
	inc	ah
	ror	al, 1
	test	al, 1
	jz	short Init_HubPort_Next

	call	Check_HubPort

Init_HubPort_Next:
	loop	short Init_HubPort_Loop

Init_Hub_Exit:
	ret

;[]========================================================================[]
;Procedure:	Proc_Hub
;Function:	HUB interrupt transaction process routine
;Input:		SI = point to task
;Output:	none
;[]========================================================================[]

Proc_Hub:
	mov	bx, [si].wTkTransDesp
	test	byte ptr [bx].bTdStatus, STS_ACTIVE
	jnz	short Proc_Hub_Exit

	test	byte ptr [bx].bTdStatus, STS_ERROR
	jnz	short Proc_Hub_End

	mov	bx, [si].wTkCtrlBuffer
	movzx	cx, byte ptr [bx].bHcNumPort

	mov	bx, [si].wTkDataBuffer
	mov	al, [bx]
	or	al, al
	jz	short Proc_Hub_End
	xor	ah, ah
Proc_HubPort_Loop:
	inc	ah
	ror	al, 1
	test	al, 1
	jz	short Proc_HubPort_Next

	call	Suspend_Tasks

	call	Check_HubPort

	call	Resume_Tasks

Proc_HubPort_Next:
	loop	short Proc_HubPort_Loop

Proc_Hub_End:
	mov	bx, [si].wTkTransDesp
	or	byte ptr [bx].bTdControl, DEF_TDCTRL
	xor	dword ptr [bx].dTdToken, DATA_TOGGLE
	mov	byte ptr [bx].bTdStatus, STS_ACTIVE

	mov	di, [si].wTkQueueHead
	mov	si, [si].wTkTransDesp
	call	Link_TD_to_QH

Proc_Hub_Exit:
	ret

;[]========================================================================[]
;Procedure:	Check_HubPort
;Function:	Check and configure hub port device attached
;Input:		SI = point to hub task
;		AH = hub port number to enumerate
;Output:	none
;[]========================================================================[]

Check_HubPort:
	pusha

	mov	di, si

	; Get Hub port status

;R08	mov	si, offset ControlBuffer1
;R08	call	Init_CtrlBuffer
	call	Get_CtrlBuffer				;R08
	jc	Check_HubPort_End			;R08

	mov	bx, si

	mov	al, [di].bTkDevAddr
	mov	[bx].bDevAddr, al
	mov	byte ptr [bx].bCtrlByte, CP_RD		;hub always high speed
	mov	word ptr [bx].wDataBuffer, offset MiscDataBuffer
	mov	byte ptr [bx].wTotalSize, 4
	mov	byte ptr [bx].wMaxSize, 4

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	[bx].wReqBuffer, si

	mov	byte ptr [si].bmRequestType, 10100011b
	mov	byte ptr [si].bRequest, GET_STATUS	;get port status
	mov	byte ptr [si].wLength, 4
	mov	byte ptr [si].wIndex, ah

	call	Control_Transfer
	jc	Check_HubPort_Exit

	; Clear connection change status of hub port

	mov	word ptr [bx].wTotalSize, 0

	mov	byte ptr [si].bmRequestType, 00100011b
	mov	byte ptr [si].bRequest, CLEAR_FEATURE
	mov	byte ptr [si].wValue, C_PORT_CONNECTION
	mov	byte ptr [si].wLength, 0

	call	Control_Transfer
	jc	Check_HubPort_Exit

	; Check hub port connection status

	test	byte ptr MiscDataBuffer.HPSbStatus1, HPS_CONNECT
	jnz	short Enable_HubPort

	; Disable hub port

Disable_HubPort:

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer

	mov	byte ptr [si].bmRequestType, 00100011b
	mov	byte ptr [si].bRequest, CLEAR_FEATURE
	mov	byte ptr [si].wValue, S_PORT_ENABLE
	mov	byte ptr [si].wIndex, ah	;R01

	mov	word ptr [bx].wTotalSize, 0

	call	Control_Transfer
;R23	jc	Check_HubPort_Exit

;R23 - start
	mov	byte ptr [si].wValue, C_PORT_ENABLE
	call	Control_Transfer
;R23 - end

	mov	al, [di].bTkDevAddr		;device address
						;AH = port number
	call	Release_ChildDev
;R08	jmp	short Check_HubPort_Exit
	jmp	Check_HubPort_Exit		;R08

Enable_HubPort:

	; Reset port

	mov	byte ptr [si].bmRequestType, 00100011b
	mov	byte ptr [si].bRequest, SET_FEATURE
	mov	byte ptr [si].wValue, S_PORT_RESET

	call	Control_Transfer
;R08	jc	short Check_HubPort_Exit
	jc	Check_HubPort_Exit		;R08

	; Wait reset completed

	mov	byte ptr [bx].wTotalSize, 4

	mov	byte ptr [si].bmRequestType, 10100011b
	mov	byte ptr [si].bRequest, GET_STATUS	;get port status
	mov	byte ptr [si].wValue, 0			;selector field
	mov	byte ptr [si].wLength, 4

	call	Delay_12ms			;wait reset process

Wait_HubPort_Reset:
	call	Control_Transfer
	jc	short Check_HubPort_Exit

	test	byte ptr MiscDataBuffer.HPSbStatus1, HPS_RESET
	jnz	short Wait_HubPort_Reset

	mov	word ptr [bx].wTotalSize, 0

	mov	byte ptr [si].bmRequestType, 00100011b
	mov	byte ptr [si].bRequest, CLEAR_FEATURE
	mov	byte ptr [si].wValue, C_PORT_RESET
	mov	byte ptr [si].wLength, 0

	call	Control_Transfer
	jc	short Check_HubPort_Exit

	mov	byte ptr [si].bmRequestType, 00100011b	;R08
	mov	byte ptr [si].bRequest, CLEAR_FEATURE	;R08
	mov	byte ptr [si].wValue, C_PORT_ENABLE	;R08
	mov	byte ptr [si].wLength, 0		;R08
	call	Control_Transfer			;R08
	jc	short Check_HubPort_Exit		;R08

	; Enumerate device attached

	call	Get_CtrlBuffer				;R08
	jc	Disable_HubPort				;R08

	push	bx

;R08	mov	si, offset ControlBuffer2
;R08	call	Init_CtrlBuffer
	mov	bx, si

	test	byte ptr MiscDataBuffer.HPSbStatus2, HPS_LS
	jz	short @f
	or	byte ptr [bx].bCtrlByte, CP_LS
@@:
	mov	al, [di].bTkDevAddr
	mov	byte ptr [bx].bParentAddr, al	;upstream device address
	mov	byte ptr [bx].bParentPort, ah	;upstream port number
	mov	byte ptr [bx].bDevAddr, 0	;default address 0
	mov	byte ptr [bx].bDevEndp, 0	;default endpoint 0

	call	Init_Device
	jnc	short @f

	mov	si, bx				;R08
	call	Release_CtrlBuffer		;R08
	pop	bx

	jmp	Disable_HubPort

@@:
	call	Init_Config			;BX=point to control buffer

	mov	si, bx				;R08
	call	Release_CtrlBuffer		;R08
	pop	bx

Check_HubPort_Exit:
	mov	si, bx				;R08
	call	Release_CtrlBuffer		;R08
Check_HubPort_End:				;R08
	popa
	ret

;************************************************************************
;*									*
;*	KEYBOARD MANAGEMENT						*
;*									*
;************************************************************************

;[]========================================================================[]
;Procedure:	Init_Keyboard
;Function:	Initialize USB keyboard
;Input:		BX = point to control parameter block
;Output:	none
;[]========================================================================[]

Init_Keyboard:

	; Query/record available keyboard index number

	call	Query_Kbd_Index
	jc	Init_Kbd_Exit

	call	Record_Kbd_Index

;R27	mov	dl, al				;keyboard index number

;R24 - start
	mov	di, offset CfgDespBuffer
	call	Find_EndpDesp
;R24 - end

	call	Init_Keyboard_Task		;R27

	; Set idle rate

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
;R27	mov	byte ptr [si].bmRequestType, 00100010b
	mov	byte ptr [si].bRequest, SET_IDLE
;R16	mov	word ptr [si].wValue, 0201h
	mov	word ptr [si].wValue, 0400h		;R16
;R24	mov	al, CfgDespBuffer.HidEDsp.EDbEndpAddr
;R27	mov	al, [di].EDbEndpAddr			;R24
;R27	and	al, 07Fh				;mask direction bit
;R27 - start
	mov	byte ptr [si].bmRequestType, 00100010b
	mov	al, [di].bTkDevEndp
	test	byte ptr [di].bTkDevStatus, TK_NEWKBD
	jz	short @f
	mov	byte ptr [si].bmRequestType, 00100001b
	mov	al, [di].bTkDevInterface
@@:
;R27 - end
	mov	byte ptr [si].wIndex, al

	mov	word ptr [bx].wReqBuffer, si
	mov	word ptr [bx].wTotalSize, 0

	call	Control_Transfer
;R05	jc	Init_Kbd_Exit
;R17 - start
	; Get report descriptor

	mov	si, offset RequestBuffer
	call	Init_ReqBuffer
	mov	byte ptr [si].bmRequestType, 10000010b
	mov	byte ptr [si].bRequest, GET_DESCRIPTOR
	mov	byte ptr [si+1].wValue, 22h		;report descriptor
;R24	mov	al, CfgDespBuffer.HidEDsp.EDbEndpAddr
;R27	mov	al, [di].EDbEndpAddr			;R24
;R27	and	al, 07Fh				;mask direction bit
	mov	al, [di].bTkDevEndp			;R27
	mov	byte ptr [si].wIndex, al
	mov	word ptr [si].wLength, 003Fh		;descriptor length

	mov	word ptr [bx].wReqBuffer, si
	or	byte ptr [bx].bCtrlByte, CP_RD		;read from device
	mov	word ptr [bx].wDataBuffer, offset MiscDataBuffer
	mov	word ptr [bx].wTotalSize, 3Fh			;total length
	movzx	ax, byte ptr DevDespBuffer.DDbMaxPackSize	;packet size
	mov	word ptr [bx].wMaxSize, ax
	call	Control_Transfer
;R17 - end

;R27	; Configure Keyboard interrupt transaction task
;R27	; DL = keyboard index number
;R27
;R27	mov	al, SIZE TASK
;R27	mul	dl
;R27	add	ax, offset KbdIntTask
;R27	mov	si, ax
;R27	call	Init_Task
;R27
;R27;R01	mov	[si].bTkDevStatus, TK_WFP+TK_KBD
;R27	mov	al, TK_WFP+TK_KBD			;R01
;R27	test	byte ptr [bx].bCtrlByte, CP_LS		;R01
;R27	jz	short @f				;R01
;R27	or	al, TK_LS				;R01
;R27@@:							;R01
;R27	mov	[si].bTkDevStatus, al			;R01
;R27
;R27	mov	byte ptr [si].bTkDevIndex, dl
;R27
;R27	mov	word ptr [si].wTkProc, offset Proc_Keyboard SMI_OFFSET
;R27
;R27	mov	al, [bx].bDevAddr
;R27	mov	[si].bTkDevAddr, al
;R27
;R27	mov	al, [bx].bParentAddr
;R27	mov	[si].bTkParentAddr, al
;R27
;R27	mov	al, [bx].bParentPort
;R27	mov	[si].bTkParentPort, al
;R27
;R27;R24	mov	di, offset CfgDespBuffer.HidEDsp
;R27
;R27	movzx	ax, byte ptr [di].EDbInterval
;R27	call	Validate_Interval		;R08
;R27	mov	[si].wTkInterval, ax
;R27
;R27	mov	ax, [di].EDwMaxPackSize
;R27	mov	[si].wTkPacketSize, ax
;R27
;R27	mov	al, byte ptr [di].EDbEndpAddr
;R27	and	al, 7Fh				;mask direction bit
;R27	mov	[si].bTkDevEndp, al
;R27
;R27	push	si				;R08
;R27	mov	al, 8
;R27	movzx	cx, al
;R27	mul	dl
;R27	add	ax, offset KbdDataBuffer
;R27;R08	mov	di, ax
;R27;R08	xor	al, al
;R27;R08	rep	stosb
;R27;R08	mov	[si].wTkDataBuffer, di
;R27	mov	si, ax				;R08
;R27	call	Clear_Buffer			;R08
;R27	pop	si				;R08
;R27	mov	[si].wTkDataBuffer, ax		;R08
;R27
;R27	push	si				;R08
;R27	mov	al, SIZE KBDCTRL
;R27	movzx	cx, al
;R27	mul	dl
;R27	add	ax, offset KbdCtrlBuffer
;R27;R08	mov	di, ax
;R27;R08	xor	al, al
;R27;R08	rep	stosb
;R27;R08	mov	byte ptr [di].bDelayRate,  12
;R27;R08	mov	byte ptr [di].bRepeatRate, 4
;R27;R08
;R27;R08	mov	[si].wTkCtrlBuffer, di
;R27	mov	si, ax				;R08
;R27	call	Clear_Buffer			;R08
;R27	mov	byte ptr [si].bDelayRate,  12	;R08
;R27	mov	byte ptr [si].bRepeatRate, 4	;R08
;R27	pop	si				;R08
;R27	mov	[si].wTkCtrlBuffer, ax		;R08
;R27
;R27	push	si
;R27	mov	al, 16
;R27	mul	dl
;R27	add	ax, offset KbdIntQH
;R27	mov	si, ax
;R27	call	Init_QH
;R27	pop	si
;R27	mov	[si].wTkQueueHead, ax
;R27
;R27	push	si
;R27	mov	al, SIZE TD
;R27	mul	dl
;R27	add	ax, offset KbdIntTD
;R27	mov	si, ax
;R27	call	Init_TD
;R27;R08	mov	di, si
;R27	pop	si
;R27;R08	mov	[si].wTkTransDesp, di
;R27	mov	[si].wTkTransDesp, ax		;R08
;R27	mov	di, ax				;R08
	mov	si, di				;R27;task control pointer
	mov	di, [si].wTkTransDesp		;R27;TransDescriptor pointer

	mov	al, DEF_TDCTRL+ISS_IOC
	test	byte ptr [bx].bCtrlByte, CP_LS
	jz	short @f
	or	al, LS
@@:
	mov	[di].bTdControl, al

	mov	edx, PID_IN
	movzx	eax, word ptr [si].wTkPacketSize
	dec	eax
	shl	eax, BIT_MAXL
	or	edx, eax
	movzx	eax, byte ptr [si].bTkDevAddr
	shl	eax, BIT_ADDR
	or	edx, eax
	movzx	eax, byte ptr [si].bTkDevEndp
	shl	eax, BIT_EP
	or	edx, eax
	mov	[di].dTdToken, edx

	mov	ax, [si].wTkDataBuffer
	call	Physical_Linear
	mov	[di].dTdPoint, eax

	mov	byte ptr [di].bTdStatus, STS_ACTIVE

	call	Link_Task

ifndef	NO_UPDATE_USBKB_LED			;R18
	call	Update_Led			;update all keyboards' LED
endif	;NO_UPDATE_USBKB_LED			;R18

Init_Kbd_Exit:
	ret

;R27 - start
;[]========================================================================[]
;Procedure:	Init_Keyboard_Task
;Function:	Initialize keyboard task data structure
;Input:		BX = point to control parameter block
;		DI = point to endpoint descriptor
;		AL = keyboard index number
;Output:	DI = point to task
;[]========================================================================[]

Init_Keyboard_Task:

	mov	dl, al

	mov	al, SIZE TASK
	mul	dl
	add	ax, offset KbdIntTask
	mov	si, ax
	call	Init_Task

	mov	al, TK_WFP+TK_KBD
	cmp	di, offset CfgDespBuffer.HidEDsp
	je	short @f
	or	al, TK_NEWKBD
@@:
	test	byte ptr [bx].bCtrlByte, CP_LS
	jz	short @f
	or	al, TK_LS
@@:
	mov	[si].bTkDevStatus, al

	mov	byte ptr [si].bTkDevIndex, dl

	mov	word ptr [si].wTkProc, offset Proc_Keyboard SMI_OFFSET

	mov	al, [bx].bDevAddr
	mov	[si].bTkDevAddr, al

	mov	al, [bx].bParentAddr
	mov	[si].bTkParentAddr, al

	mov	al, [bx].bParentPort
	mov	[si].bTkParentPort, al

	movzx	ax, byte ptr [di].EDbInterval
	call	Validate_Interval
	mov	[si].wTkInterval, ax

	mov	ax, [di].EDwMaxPackSize
	mov	[si].wTkPacketSize, ax

	mov	al, byte ptr [di].EDbEndpAddr
	and	al, 7Fh				;mask direction bit
	mov	[si].bTkDevEndp, al

	mov	al, CfgDespBuffer.StdIDsp.IDbAltSet
	mov	[si].bTkDevInterface, al

	push	si				;task control pointer
	mov	al, 8
	movzx	cx, al
	mul	dl
	add	ax, offset KbdDataBuffer
	mov	si, ax
	call	Clear_Buffer
	pop	si				;task control pointer
	mov	[si].wTkDataBuffer, ax

	push	si				;task control pointer
	mov	al, SIZE KBDCTRL
	movzx	cx, al
	mul	dl
	add	ax, offset KbdCtrlBuffer
	mov	si, ax
	call	Clear_Buffer
	mov	byte ptr [si].bDelayRate,  12
	mov	byte ptr [si].bRepeatRate, 4
	pop	si				;task control pointer
	mov	[si].wTkCtrlBuffer, ax

	push	si				;task control pointer
	mov	al, 16
	mul	dl
	add	ax, offset KbdIntQH
	mov	si, ax
	call	Init_QH
	pop	si				;task control pointer
	mov	[si].wTkQueueHead, ax

	push	si				;task control pointer
	mov	al, SIZE TD
	mul	dl
	add	ax, offset KbdIntTD
	mov	si, ax
	call	Init_TD
	pop	si				;task control pointer
	mov	[si].wTkTransDesp, ax

	mov	di, si

	ret
;R27 - end

;[]========================================================================[]
;Procedure:	Proc_Keyboard
;Function:	Keyboard interrupt transaction process routine
;Input:		SI = point to task
;Output:	none
;[]========================================================================[]

Proc_Keyboard:
;R06	mov	bx, [si].wTkTransDesp
;R06	test	byte ptr [bx].bTdStatus, STS_ACTIVE
;R06	jnz	short Proc_Kbd_Exit
;R06
;R06	mov	bx, [si].wTkCtrlBuffer
;R06
;R06	cmp	byte ptr [bx].bKeyLength, 0
;R06	je	short Check_Kbd_Data
;R06
;R06	push	si
;R06
;R06	mov	si, bx
;R06	call	Proc_Key_Buffer
;R06
;R06	pop	si
;R06
;R06	jmp	short Proc_Kbd_End
;R06
;R06Check_Kbd_Data:
	mov	bx, [si].wTkTransDesp
	test	byte ptr [bx].bTdStatus, STS_ACTIVE	;R06
	jnz	short Proc_Kbd_Exit			;R06
	test	byte ptr [bx].bTdStatus, STS_ERROR
	jnz	short Proc_Kbd_End

	mov	ax, [bx].wTdActLen
	and	ax, 07FFh
	cmp	ax, 07FFh
	je	short Proc_Kbd_End

	push	si

	mov	di, [si].wTkDataBuffer
	mov	si, [si].wTkCtrlBuffer
	call	Proc_Kbd_Data

;R06 - start
	cmp	byte ptr [si].bKeyLength, 0
	je	short @f

	call	Proc_Key_Buffer
@@:
;R06 - end
	pop	si

Proc_Kbd_End:
	mov	bx, [si].wTkTransDesp
	or	byte ptr [bx].bTdControl, DEF_TDCTRL
	xor	dword ptr [bx].dTdToken, DATA_TOGGLE
	mov	byte ptr [bx].bTdStatus, STS_ACTIVE

	mov	di, [si].wTkQueueHead
	mov	si, [si].wTkTransDesp
	call	Link_TD_to_QH

Proc_Kbd_Exit:
	ret

;[]========================================================================[]
;Input:		SI = point to control buffer
;		DI = point to data buffer
;[]========================================================================[]

; Key map process algorithm
; Current Map	Last Map
; 0		0		no key press
; 1		0		make key
; 1		1		repeat key
; 0		1		break key

Proc_Kbd_Data:
	call	Check_RollOver			;R09
	jz	Roll_Over_Exit			;R09

	call	Clear_Current_Buffer		;clear current key map

;R09	call	Check_RollOver
;R09	jz	Roll_Over_Exit

	call	Check_Modifier			;check empty modifier bytes
	jz	short Proc_MainKey		;skip if empty

Proc_Modifier:
	mov	al, [di]			;modifier

	mov	cx, 8				;8 bits
	mov	bx, offset Modifier_Table SMI_OFFSET
Proc_Modifier_Loop:
	test	al, 1
	jz	short Proc_Modifier_Next
	push	cx
	push	bx
	push	ax
	mov	al, cs:[bx]
	call	Mark_Current_Buffer		;mark current key map
	pop	ax
	pop	bx
	pop	cx
Proc_Modifier_Next:
	inc	bx
	shr	al, 1
	loop	short Proc_Modifier_Loop

Proc_MainKey:
	call	Check_MainKey			;check empty main key buffer?
	jz	short Proc_KeyCode		;skip if empty

	mov	cx, 6				;6 bytes
	lea	bx, [di+2]			;main key start offset
Proc_MainKey_Loop:
	mov	al, [bx]
	push	cx
	push	bx
	or	al, al
	jz	short Proc_MainKey_Next
	call	Mark_Current_Buffer		;mark current key map
Proc_MainKey_Next:
	pop	bx
	pop	cx
	inc	bx
	loop	short Proc_MainKey_Loop

Proc_KeyCode:
	call	Check_Current_Buffer		;check current key map empty?
	jnz	short @f			;jmp if not empty
	call	Check_Last_Buffer		;check last key map empty?
	jz	short Proc_Kbd_Buff_Exit	;jmp if also empty
@@:
	push	ds
	mov	ax, 0040h
	mov	ds, ax
	mov	al, byte ptr ds:[0017h]		;BIOS key status flag
	pop	ds
	mov	[si].bBiosFlag, al		;save for status reference

	xor	dx, dx
	lea	bx, [si].CurrentBuffer
	lea	di, [si].LastBuffer
	mov	cx, 32

Proc_Byte_Loop:
	push	cx
	push	bx
	push	di

	mov	al, ds:[bx]
	mov	ah, ds:[di]
	mov	cx, 8
Proc_Bit_Loop:
	push	ax
	push	dx
	push	cx

	mov	[si].bCurrentIndex, dl

	test	al, 1
	jz	short Check_Break		;current=0 (no make)
	test	ah, 1
	jnz	short Check_Repeat		;current/last=1 (repeat)

	mov	[si].bLastIndex, dl

	mov	al, [si].bDelayRate		;typematic delay rate
	mov	[si].bDelayCount, al

	call	Proc_MakeKey			;translate make codes

	jmp	short Proc_Next_Bit

Check_Repeat:
	cmp	dl, [si].bLastIndex		;just repeat with last key
	jne	short Proc_Next_Bit

	dec	byte ptr [si].bDelayCount
	jnz	short Proc_Next_Bit

	mov	al, [si].bRepeatRate		;typematic repeat rate
	mov	[si].bDelayCount, al

	call	Proc_RepeatKey			;translate repeat codes

	jmp	short Proc_Next_Bit

Check_Break:
	test	ah, 1
	jz	short Proc_Next_Bit		;current/last=0 (no key)

	;current=0, last=1 (break key)

	call	Proc_BreakKey			;translate break codes

Proc_Next_Bit:
	pop	cx
	pop	dx
	pop	ax
	inc	dx
	shr	al, 1
	shr	ah, 1
	loop	short Proc_Bit_Loop

	pop	di
	pop	bx
	pop	cx
	inc	bx
	inc	di
	loop	short Proc_Byte_Loop

Proc_Kbd_Buff_Exit:
	call	Update_Last_Buffer		;update last map to current map
Roll_Over_Exit:
	ret

;[]========================================================================[]
; Modifier translation table from modifier bit map to USB key number(-70h)
;[]========================================================================[]

Modifier_Table:
;		0E0h, 0E1h, 0E2h, 0E3h, 0E4h, 0E5h, 0E6h, 0E7h
	db	070h, 071h, 072h, 073h, 074h, 075h, 076h, 077h

;[]========================================================================[]
; Key codes translation table from USB key number to AT key code
;[]========================================================================[]

Xlat_Code_Table:
; Scan code set 1 (USB key number mapping)
;   0	 1    2    3	4    5	  6    7    8	 9    A    B	C    D	  E    F
 db 000h,000h,000h,000h,01Eh,030h,02Eh,020h,012h,021h,022h,023h,017h,024h,025h,026h ;0
 db 032h,031h,018h,019h,010h,013h,01Fh,014h,016h,02Fh,011h,02Dh,015h,02Ch,002h,003h ;1
 db 004h,005h,006h,007h,008h,009h,00Ah,00Bh,01Ch,001h,00Eh,00Fh,039h,00Ch,00Dh,01Ah ;2
;R15 db 01Bh,02Bh,000h,027h,028h,029h,033h,034h,035h,03Ah,03Bh,03Ch,03Dh,03Eh,03Fh,040h ;3
 db 01Bh,02Bh,02Bh,027h,028h,029h,033h,034h,035h,03Ah,03Bh,03Ch,03Dh,03Eh,03Fh,040h ;3;R15
 db 041h,042h,043h,044h,057h,058h,037h,046h,000h,052h,047h,049h,053h,04Fh,051h,04Dh ;4
 db 04Bh,050h,048h,045h,035h,037h,04Ah,04Eh,01Ch,04Fh,050h,051h,04Bh,04Ch,04Dh,047h ;5
;R15 db 048h,049h,052h,053h,000h,05Dh,000h,000h,054h,000h,000h,000h,000h,000h,000h,000h ;6
 db 048h,049h,052h,053h,056h,05Dh,000h,000h,054h,000h,000h,000h,000h,000h,000h,000h ;6;R15
 db 01Dh,02Ah,038h,05Bh,01Dh,036h,038h,05Ch,000h,000h,000h,000h,000h,000h,000h,000h ;7
 db 000h,000h,000h,000h,000h,000h,000h,073h,070h,07Dh,079h,07Bh,000h,000h,000h,000h ;8;R26

;[]========================================================================[]
; Convenient flag codes translation table
;[]========================================================================[]

Xlat_Flag_Table:
; Flag code for process conveniently (USB key number mapping)
; bit7=0 status key
; bit6=0 send extended E0
; 0000.0100 - 04 : RtCtrl
; 0000.1000 - 08 : RtAlt
; 0100.0001 - 41 : RtShft
; 0100.0010 - 42 : LtShft
; 0100.0100 - 44 : LtCtrl
; 0100.1000 - 48 : LtAlt
; 1000.0000 - 80 : RtEnter
; 1000.1000 - 88 : /
; 1001.0000 - 90 : PrntScrn
; 1010.0000 - A0 : Ins, Del, Home, End, PgUp, PgDn, UpA, DnA, LtA, RtA
; 1100.0000 - C0 : NSK (No Special Keys)
; 1100.0100 - C4 : Pause
; 1100.0111 - C7 : CapsLock
; 1100.1011 - CB : NumLock
; 1100.1101 - CD : ScrollLock
;   0	 1    2    3	4    5	  6    7    8	 9    A    B	C    D	  E    F
 db 0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h ;0
 db 0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h ;1
 db 0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h ;2
 db 0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C7h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h ;3
 db 0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,090h,0CDh,0C4h,0A0h,0A0h,0A0h,0A0h,0A0h,0A0h,0A0h ;4
 db 0A0h,0A0h,0A0h,0CBh,088h,0C0h,0C0h,0C0h,080h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h ;5
 db 0C0h,0C0h,0C0h,0C0h,0C0h,0A0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h ;6
 db 044h,042h,048h,0A0h,004h,041h,008h,0A0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h ;7
 db 0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h,0C0h ;8;R26

;[]========================================================================[]
;Procedure:	Proc_MakeKey
;Function:	Translate make codes
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Proc_MakeKey:
	call	Reloc_PrnScrn			;relocate <PrnScrn> key number

	mov	al, [si].bCurrentIndex
	call	Get_Key_Num

	call	Proc_Shift			;process enhance <Shift> code

	call	Send_Ext_Shift

	mov	al, [si].bCurrentKey
	call	Get_Flag_Code

	cmp	al, 0C4h
	je	short Proc_Pause

	test	al, 80h
	jnz	short Not_Make_FKP

; Setting RtShft, LtShft, RtCtrl, LtCtrl, AltGr, LtAlt flag

	and	al, 0Fh
	or	[si].bBiosFlag, al

Not_Make_FKP:
	call	Send_Ext_E0

	movzx	bx, byte ptr [si].bCurrentKey
	add	bx, offset Xlat_Code_Table SMI_OFFSET
	mov	al, cs:[bx]
	call	Store_Code

Proc_MakeKey_Exit:
	ret

; <Pause> process

Proc_Pause:
	mov	bx, offset Pause_Code SMI_OFFSET
	mov	cx, 6
	test	byte ptr [si].bBiosFlag, 04	;check <Ctrl> status
	jz	short @f			;jump if no <Ctrl> depress
	add	bx, 6
	mov	cl, 4
@@:
	mov	al, cs:[bx]
	call	Store_Code
	inc	bx
	loop	short @b
	jmp	short Proc_MakeKey_Exit

Pause_Code:
; Normal <Pause> code
	DB	0E1H, 01DH, 045H, 0E1H, 09DH, 0C5H

; <Ctrl+Pause> code
	DB	0E0h, 046h, 0E0h, 0C6h

;[]========================================================================[]
;Procedure:	Proc_RepeatKey
;Function:	Translate repeat codes
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Proc_RepeatKey:
	mov	al, [si].bCurrentIndex
	call	Get_Key_Num

	call	Send_Ext_Shift			;send extended <Shift> code
						;if necessary
	call	Send_Ext_E0

	movzx	bx, byte ptr [si].bCurrentKey
	add	bx, offset Xlat_Code_Table SMI_OFFSET
	mov	al, cs:[bx]
	call	Store_Code

	ret

;[]========================================================================[]
;Procedure:	Proc_BreakKey
;Function:	Translate break codes
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Proc_BreakKey:
	mov	al, [si].bCurrentIndex
	call	Get_Key_Num

	call	Get_Flag_Code			;get flag code
	cmp	al, 0C4h			;check <Pause> break
	je	short Proc_BreakKey_Exit

	test	al, 80h 			;check FKP break
	jnz	short Normal_Break

; RtCtrl, AltGr, RtShft, LtShft, LtCtrl, LtAlt

	and	al, 0Fh
	xor	[si].bBiosFlag, al		;toggle status flag

	not	al
	or	al, 0FCh			;reserved <Shift> status
	and	[si].bStatusFlag, al

Normal_Break:
	call	Send_Ext_E0

	movzx	bx, byte ptr [si].bCurrentKey
	add	bx, offset Xlat_Code_Table SMI_OFFSET
	mov	al, cs:[bx]
	or	al, 80h
	call	Store_Code

	mov	al, [si].bCurrentIndex
	cmp	al, [si].bLastIndex
	jne	short Proc_BreakKey_Exit

	call	Proc_Shift

Proc_BreakKey_Exit:
	ret

;[]========================================================================[]
;Procedure:	Get_Key_Num
;Function:	Get AT key codes and relocate PrnScrn code if necessary
;Input:		SI = point to control buffer
;		AL = Current_Index
;Output:	AL = Current_Key
;[]========================================================================[]

Get_Key_Num:
	cmp	al, 46h 			;check <PrnScrn> key number
	jne	short @f
	mov	al, [si].bRelocKey		;get <PrnScrn> relocate in make
@@:
	mov	[si].bCurrentKey, al
	ret

;[]========================================================================[]
;Procedure:	Reloc_PrnScrn
;Function:	Relocate PrnScrn code if ALT pressed
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Reloc_PrnScrn:
	mov	byte ptr [si].bRelocKey, 46h 	;normal <PrnScrn> key number

	test	byte ptr [si].bBiosFlag, ALT 	;<Alt> status
	jz	short @f

	mov	byte ptr [si].bRelocKey, 68h 	;code 54H key number
@@:
	ret

;[]========================================================================[]
;Procedure:	Proc_Shift
;Function:	Process virtual shift codes
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Proc_Shift:
	test	byte ptr [si].bStatusFlag, VLTSHFT
	jz	short Chk_RtShft		;skip if virtual <LtShft> off

	test	byte ptr [si].bBiosFlag, LTSHFT
	jnz	short Proc_LtShft		;jump if <LtShft> on

; Send Enhance <LtShft> break code (VLTSHFT-on, LTSHFT-off)

	call	LtShft_Break
	jmp	short Chk_RtShft

; Send Enhance <LtShft> make code (VLTSHFT-on, LTSHFT-on)

Proc_LtShft:
	call	LtShft_Make

Chk_RtShft:
	test	byte ptr [si].bStatusFlag, VRTSHFT
	jz	short Proc_Shift_Ret		;skip if virtual <RtShft> off

	test	byte ptr [si].bBiosFlag, RTSHFT
	jnz	short Proc_RtShft		;jump if <RtShft> on

; Send Enhance <RtShft> break code (VRTSHFT-on, RTSHFT-off)

	call	RtShft_Break
	jmp	short Proc_Shift_Ret

; Send Enhance <RtShft> make code (VRTSHFT-on, RTSHFT-on)

Proc_RtShft:
	call	RtShft_Make

Proc_Shift_Ret:
	and	byte ptr [si].bStatusFlag, 0FCh	;clear virtual shift flag
	ret

; Send Enhance <RtShft> make code

RtShft_Make:
	mov	al, 0E0h			;Code E0h
	call	Store_Code
	mov	al, 036h			;Code <RtShft>
	call	Store_Code
	ret

; Send Enhance <RtShft> break code

RtShft_Break:
	mov	al, 0E0h			;Code E0h
	call	Store_Code
	mov	al, 0B6h			;Code <RtShft> break
	call	Store_Code
	ret

; Send Enhance <LtShft> make code

LtShft_Make:
	mov	al, 0E0h			;Code E0h
	call	Store_Code
	mov	al, 02Ah			;Code <LtShft>
	call	Store_Code
	ret

; Send Enhance <LtShft> break code

LtShft_Break:
	mov	al, 0E0h			;Code E0h
	call	Store_Code
	mov	al, 0AAh			;Code <LtShft> break
	call	Store_Code
	ret

;[]========================================================================[]
;Procedure:	Send_Ext_Shift
;Function:	Translate extended shift code for special keys
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Send_Ext_Shift:
	mov	al, [si].bCurrentKey
	call	Get_Flag_Code			;get flag code
	test	al, 10000000b
	jz	short SES_Ret
	test	al, 00000001b
	jnz	short SES_Ret
	test	al, 00100000b
	jnz	short Chk_NumLock		;jump if FKP
	test	al, 00010000b
	jnz	short SES_PrnScrn		;jump if <PrnScrn>
	test	al, 00001000b
	jnz	short FKP_NumLock_Off		;jump if "/"

SES_Ret:
	ret

; FKP : Function Keys Pad : Ins,Del,Home,End,PgUp,PgDn,UpA,DnA,LtA,RtA

Chk_NumLock:
	test	byte ptr [si].bBiosFlag, NUMLOCK
	jnz	short FKP_NumLock_On		;jump if <NumLock> On

; Key < / >
; FKP and <NumLock> off

FKP_NumLock_Off:
	mov	al, [si].bBiosFlag
	xor	al, [si].bStatusFlag 		;toggle virtual shift state
	and	al, 03
	jz	short SES_Ret

	and	byte ptr [si].bStatusFlag, 0FCh
	or	byte ptr [si].bStatusFlag, al

	test	byte ptr [si].bStatusFlag, VLTSHFT
	jz	short Chk_VRTSHFT		;skip if virtual <LtShft> off

	call	LtShft_Break			;send enhance LtShft break code

Chk_VRTSHFT:
	test	byte ptr [si].bStatusFlag, VRTSHFT
	jz	short SES_Ret			;skip if virtual <RtShft> off

	call	RtShft_Break			;send enhance RtShft break code
	jmp	short SES_Ret

; FKP and <NumLock> on

FKP_NumLock_On:
	mov	al, [si].bBiosflag
	or	al, [si].bStatusFlag
	and	al, 03
	jnz	short SES_Ret			;skip if <Shift> on

	and	byte ptr [si].bStatusFlag, 0FCh
	or	byte ptr [si].bStatusFlag, VLTSHFT	;set <LtShft> flag
	call	LtShft_Make			;send enhance LtShft make code
	jmp	short SES_Ret

; <PrnScrn>

SES_PrnScrn:
	mov	al, [si].bStatusFlag
	and	al, 03
	or	al, [si].bBiosFlag

	and	al, 07h
	jnz	short SES_Ret			;skip if <Ctrl> or <Shift> on

	and	byte ptr [si].bStatusFlag, 0FCh
	or	byte ptr [si].bStatusFlag, VLTSHFT	;set <LtShft> flag
	call	LtShft_Make
	jmp	short SES_Ret

;[]========================================================================[]
;Procedure:	Send_Ext_E0
;Function:	Translate extended E0 code for special keys
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Send_Ext_E0:
	mov	al, [si].bCurrentKey
	call	Get_Flag_Code			;get flag code

	test	al, 01000000b
	jnz	short @f

; Send E0H : RtCtrl, RtAlt, RtEnter, /, PrnScrn, FKP, LtGUI, RtGUI, APP

	mov	al, 0E0h
	call	Store_Code
@@:
	ret

;[]========================================================================[]
;Procedure:	Get_Flag_Code
;Function:	Get key convenient flag code
;Input:		AL = USB key number
;Output:	AL = flag code
;[]========================================================================[]

Get_Flag_Code:
	movzx	bx, al
	add	bx, offset Xlat_Flag_Table SMI_OFFSET
	mov	al, cs:[bx]
	ret

;[]========================================================================[]
;Procedure:	Proc_Key_Buffer
;Function:	Process key codes buffer and output to keyboard controller
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Proc_Key_Buffer:
	push	bx
	push	di

	mov	al, 0Bh 			;read ISR
	out	20h, al
	NEWIODELAY
	in	al, 20h
	test	al, 02				;IRQ1 in service
	jnz	short Proc_Key_Buffer_Exit

	mov	al, 0Bh 			;R12;read ISR
	out	0A0h, al			;R12
	NEWIODELAY				;R12
	in	al, 0A0h			;R12
	test	al, 10h				;R12;IRQ12 in service
	jnz	short Proc_Key_Buffer_Exit	;R12

	movzx	bx, byte ptr [si].bKeyPoint
	lea	di, [si].KeyBuffer
	mov	al, [bx+di]
	call	Send_KeyCode
	jc	short Proc_Key_Buffer_Exit

;R19A	cmp	byte ptr [si].bKeyLength, 17
;R19A	jne	short @f
;R19A	movzx	bx, byte ptr [si].bKeyPoint
;R19A	lea	di, [si].KeyBuffer
;R19A;R19	mov	byte ptr [bx+di], 0		;roll over key
;R19A	mov	byte ptr [bx+di], 0FFh		;R19;roll over key
;R19A@@:
	inc	byte ptr [si].bKeyPoint
	and	byte ptr [si].bKeyPoint, 0Fh
	dec	byte ptr [si].bKeyLength

Proc_Key_Buffer_Exit:
	pop	di
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Store_Code
;Function:	Store key codes to bKeyBuffer
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Store_Code:
	push	bx
	push	di
	cmp	byte ptr [si].bKeyLength, 16
	jae	short Store_Code_Exit		;R19A
;R19A	ja	short Store_Code_Exit
;R19A	jb	short @f
;R19A	inc	byte ptr [si].bKeyLength	;length=17:roll over
;R19A	jmp	short Store_Code_Exit
;R19A@@:
	movzx	bx, byte ptr [si].bKeyPoint
	movzx	di, byte ptr [si].bKeyLength
	add	bx, di
	and	bx, 0Fh
	lea	di, [si].KeyBuffer
	mov	[bx+di], al
	inc	byte ptr [si].bKeyLength
Store_Code_Exit:
	pop	di
	pop	bx
	ret

;[]========================================================================[]
;Procedure:	Check_Modifier
;Function:	check modifier empty
;Input:		DI = point to data buffer
;Output:	ZF = 1 empty
;		ZF = 0 not empty
;[]========================================================================[]

Check_Modifier:
	test	byte ptr [di], 11111111b	;modifier
	ret

;[]========================================================================[]
;Procedure:	Check_MainKey
;Function:	check mainkey buffer empty
;Input:		DI = point to data buffer
;Output:	ZF = 1 empty
;		ZF = 0 not empty
;[]========================================================================[]

Check_MainKey:
	push	ax
	push	cx
	push	di
	xor	al, al
	mov	cx, 6
	add	di, 2
	repe	scasb
	pop	di
	pop	cx
	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Check_RollOver
;Function:	check keyboard roll-over error
;Input:		DI = point to data buffer
;Output:	ZF = 1 roll-over error
;		ZF = 0 no error
;[]========================================================================[]

Check_RollOver:
	push	ax
	push	cx
	push	di
	mov	al, 01
;R09	mov	cx, 8
;R09	repe	scasb
	mov	cx, 6				;R09
	add	di, 2				;R09
	repe	scasb				;R09
	pop	di
	pop	cx
	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Mark_Current_Buffer
;Function:	Mark current key bit map with key code
;Input:		SI = point to control buffer
;		AL = USB key code
;Output:	none
;[]========================================================================[]

Mark_Current_Buffer:
	push	ax
	push	bx
	push	cx
	cmp	al, 90h				;R26
	jae	short @f			;R26
	movzx	bx, al
	shr	bx, 3				;byte position
	lea	cx, [si].CurrentBuffer
	add	bx, cx
	mov	cl, al
	and	cl, 07				;bit position
	mov	al, 01
	shl	al, cl
	or	[bx], al
@@:						;R26
	pop	cx
	pop	bx
	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Check_Current_Buffer
;Function:	check current key bit map empty
;Input:		SI = point to control buffer
;Output:	ZF = 1 empty
;		ZF = 0 not empty
;[]========================================================================[]

Check_Current_Buffer:
	push	ax
	push	cx
	push	di
	xor	al, al
	mov	cx, 32
	lea	di, [si].CurrentBuffer
	repe	scasb
	pop	di
	pop	cx
	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Check_Last_Buffer
;Function:	check last key bit map empty
;Input:		SI = point to control buffer
;Output:	ZF = 1 empty
;		ZF = 0 not empty
;[]========================================================================[]

Check_Last_Buffer:
	push	ax
	push	cx
	push	di
	xor	al, al
	mov	cx, 32
	lea	di, [si].LastBuffer
	repe	scasb
	pop	di
	pop	cx
	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Clear_Current_Buffer
;Function:	clear current key bit map
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Clear_Current_Buffer:
	push	ax
	push	cx
	push	di
	mov	cx, 32
	xor	al, al
	lea	di, [si].CurrentBuffer
	rep	stosb
	pop	di
	pop	cx
	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Clear_Last_Buffer
;Function:	clear last key bit map
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Clear_Last_Buffer:
	push	ax
	push	cx
	push	di
	mov	cx, 32
	xor	al, al
	lea	di, [si].LastBuffer
	rep	stosb
	pop	di
	pop	cx
	pop	ax
	ret

;[]========================================================================[]
;Procedure:	Update_Last_Buffer
;Function:	update last key map with current key map
;Input:		SI = point to control buffer
;Output:	none
;[]========================================================================[]

Update_Last_Buffer:
	push	cx
	push	si
	push	di
	mov	cx, 32
	lea	di, [si].LastBuffer
	lea	si, [si].CurrentBuffer
	rep	movsb
	pop	di
	pop	si
	pop	cx
	ret

;[]========================================================================[]
;Procedure:	Send_KeyCode
;Function:	Send key code to keyboard controller with D2 command
;Input:		AL = key code
;Output:	CF = 0 successful
;		CF = 1 fail
;[]========================================================================[]

Send_KeyCode:
	push	cx
	push	ax
ifdef	IOTRAP_SUPPORT				;R22A
        call    DisableUsbLegacySmi             ;R22A
endif	;IOTRAP_SUPPORT				;R22A
	in	al, 64h
	test	al, 03				;IBF/OBF
	jz	short @f
kc_write_fail:
	pop	ax
	stc
	jmp	short kc_write_exit
@@:
	mov	al, 0D2h			;write output buffer command
	out	64h, al
	NEWIODELAY
	xor	cx, cx
@@:
	in	al, 64h
	NEWIODELAY
	test	al, 01				;R12
	jnz	short kc_write_fail		;R12
	test	al, 02				;IBF
	jz	short @f
	loop	short @b
	jmp	short kc_write_fail
@@:
	pop	ax
	out	60h, al
	clc
kc_write_exit:
;R22A - start
ifdef	IOTRAP_SUPPORT
	pushf
        mov     al, 0fh
	mov	cx, wHostID
	add	cx, LEGACY_HI
	SMI_F0CALL	Set_Pci
        call    EnableUsbLegacySmi
	popf
endif	;IOTRAP_SUPPORT
;R22A - end
	pop	cx
	ret

;R22 - start
ifdef	IOTRAP_SUPPORT
;************************************************************************
;*									*
;*	60h/64h I/O TRAPPING MODULE					*
;*									*
;************************************************************************

UsbLegSmi       proc	near
	call	GetTrapBy
	test	TrapByStatus, 0fh
	jnz	short CkTrap

IsSmiByUsb:
	clc
	ret

CkTrap:
	call	GetTrapData
@@:
        cmp     UsbWfp, 0
        je      short Is64W
        call    ds:KbcProc
        jc      short Is64W
        jmp     short UsbLegSmi_exit

Is64W:
;R22A	mov	cx, (PIIX3_USB SHL 8)+LEGACY_LO
	mov	cx, wHostID			;R22A
	add	cx, LEGACY_LO			;R22A
	SMI_F0CALL	Get_Pci

        test    al, PORT64WEN
        jz      short @f
        test    TrapByStatus, TRAPBY64W
        jnz	short do64W

@@:
        test    al, PORT60WEN
        jz      short @f

        test    TrapByStatus, TRAPBY60W
	jnz	short do60W

@@:
        test    al, PORT64REN
        jz      short @f

	test	TrapByStatus, TRAPBY64R
	jnz	short do64R

@@:
        test    al, PORT60REN
        jz      short @f

	test	TrapByStatus, TRAPBY60R
	jnz	short do60R
@@:						;R22A
	clc
	ret

do60R:
	call	SmiIn60
	call	SetTrapData
	jmp	short UsbLegSmi_exit

do64R:
	call	SmiIn64
	call	SetTrapData
	jmp	short UsbLegSmi_exit

do64W:
	call	SmiOut64
        mov     ds:SearchTbl, offset cs:KbcCmdTbl SMI_OFFSET
	mov	ds:CmdCnt, KBC_CMD_CNT
	jmp	short UsbLegSmiGo

do60W:
	call	SmiOut60
	mov	ds:CmdCnt, KBD_CMD_CNT
        mov     ds:SearchTbl, offset cs:KbdCmdTbl SMI_OFFSET

UsbLegSmiGo:
        call    SearchCmdTbl
	cmp	CmdProc, 0
	je	short UsbLegSmi_exit
        call    ds:CmdProc

UsbLegSmi_exit:
        mov     al, 0fh
;R22A	mov	cx, (PIIX3_USB SHL 8)+LEGACY_HI
	mov	cx, wHostID			;R22A
	add	cx, LEGACY_HI			;R22A
	SMI_F0CALL	Set_Pci
	stc
	ret
UsbLegSmi	endp

;[]========================================================================[]
;Procedure:	SmiOut60
;Function:      Output data to port 60 in USB Legacy SMM
;
;Input:		IoTrapData
;Output:        None
;
;[]========================================================================[]
SmiOut60	proc	near
	push	ax
	call	DisableUsbLegacySmi
	mov	al, IotrapData
	out	60h, al
	call	EnableUsbLegacySmi
	pop	ax
	ret
SmiOut60	endp

;[]========================================================================[]
;Procedure:	SmiOut64
;Function:      Output data to port 64 in USB Legacy SMM
;
;Input:		IoTrapData
;Output:        None
;
;[]========================================================================[]
SmiOut64	proc	near
	push	ax
	call	DisableUsbLegacySmi
	mov	al, IotrapData
	out	64h, al
	call	EnableUsbLegacySmi
	pop	ax
	ret
SmiOut64	endp

;[]========================================================================[]
;Procedure:	SmiIn60
;Function:      Input data from port 60 in USB Legacy SMM
;
;Input:		None
;Output:        IoTrapData = Data read from port 60
;
;[]========================================================================[]
SmiIn60	proc	near
	push	ax
	call	DisableUsbLegacySmi
	in	al, 60h
	IODELAY
	mov	IoTrapData, al
	call	EnableUsbLegacySmi
	pop	ax
	ret
SmiIn60	endp

;[]========================================================================[]
;Procedure:	SmiIn64
;Function:      Input data from port 64 in USB Legacy SMM
;
;Input:		None
;Output:        IoTrapData = Data read from port 64
;
;[]========================================================================[]
SmiIn64	proc	near
	push	ax
	call	DisableUsbLegacySmi
        in      al, 64h
        IODELAY
	mov	IoTrapData, al
	call	EnableUsbLegacySmi
	pop	ax
	ret
SmiIn64	endp


;[]========================================================================[]
;Procedure:	GetTrapBy
;Function:      Read C1 register of function 2 of PIIX3/4
;
;Input:		None
;Output:        TrapByStatus = Contents of C1 register of function 2 of
;		PIIX3/4
;[]========================================================================[]
GetTrapBy	proc	near
;R22A	mov	cx, (PIIX3_USB SHL 8)+LEGACY_HI
	mov	cx, wHostID			;R22A
	add	cx, LEGACY_HI			;R22A
	SMI_F0CALL	Get_Pci
	mov	TrapByStatus, al
	ret
GetTrapBy	endp


;[]========================================================================[]
;Procedure:	GetTrapData
;Function:      Retrieve the contents of AL register from the SMM storage
;               area.
;Input:		None
;Output:        IoTrapData = Contents of AL register when SMI is triggered.
;[]========================================================================[]
GetTrapData    proc     near
        push    ax
        push    si
        mov     si, 7fd0h
        mov     al, cs:[si]
        mov     IoTrapData, al
        pop     si
        pop     ax
        ret
GetTrapData   endp

;[]========================================================================[]
;Procedure:	SetTrapData
;Function:      Retrieve the contents of AL register from the SMM storage
;               area.
;Input:		IoTrapData = Contents of AL register when SMI is resumed
;Output:        None
;[]========================================================================[]
SetTrapData    proc     near
        push    ax
        push    si
	mov	al, IoTrapData
        mov     si, 7fd0h
        mov     cs:[si], al
        pop     si
        pop     ax
        ret
SetTrapData   endp

DisableUsbLegacySmi	proc	near
;R22A	mov	cx, (PIIX3_USB SHL 8)+LEGACY_LO
	mov	cx, wHostID			;R22A
	add	cx, LEGACY_LO			;R22A
	SMI_F0CALL	Get_Pci
	and	al, NOT USBSMIEN	;Disable Trap/SMI on USB IRQ
        and     al, not 0fh
	SMI_F0CALL	Set_Pci
;R22A        mov	cx, (PIIX_ID SHL 8)+0A3h
;R22A	SMI_F0CALL	Get_PCI
;R22A        and     al, not 01
;R22A	SMI_F0CALL	Set_PCI
	call	Ct_DisableUsbLegacySmi		;R22A

	ret
DisableUsbLegacySmi	endp

EnableUsbLegacySmi	proc	near
;R22A	mov	cx, (PIIX3_USB SHL 8)+LEGACY_LO
	mov	cx, wHostID			;R22A
	add	cx, LEGACY_LO			;R22A
	SMI_F0CALL	Get_Pci
	or	al, USBSMIEN+A20PASSEN	;Enable Trap/SMI on USB IRQ
        or      al, PORT64REN+PORT60REN+PORT64WEN+PORT60WEN
	SMI_F0CALL	Set_Pci
;R22A        mov	cx, (PIIX_ID SHL 8)+0A3h
;R22A	SMI_F0CALL	Get_PCI
;R22A        or      al, 01
;R22A	SMI_F0CALL	Set_PCI
	call	Ct_EnableUsbLegacySmi		;R22A

	ret
EnableUsbLegacySmi	endp

;[]========================================================================[]
;Procedure:	SearchCmdTbl
;Function:	Set the variable CmdPorc to pointer to a KbdCmd handler or
;		a KbcCmd handler
;Input:		SearchTbl = pointer to either the KbcCmdTbl or KbdCmdTbl
;       	IoTrapData = Kbd or Kbc command
;Output:        CmdProc = pointer to KbdCmd or KbcCmd handler or a null
;		pointer if KbdCmd or KbcCmd handler is not implemented.
;[]========================================================================[]
SearchCmdTbl	proc	near
	push	ax
	push	cx
	push	si
	mov	si, SearchTbl
	mov	al, IoTrapData
        mov     cx, ds:CmdCnt

SearchNextCmd:
	cmp	al, cs:[si]
	je	short SetPtr
	add	si, 3
	loop	SearchNextCmd
	mov	CmdProc, 0
	jmp	short SearchCmdTbl_exit

SetPtr:
	mov	ax, cs:[si+1]
	mov	CmdProc, ax

SearchCmdTbl_exit:
	pop	si
	pop	cx
	pop	ax
	ret
SearchCmdTbl	endp

write_cmd_proc	proc	near
	mov	dx, offset write_cmd_proc2 SMI_OFFSET
	mov	KbcProc, dx
	mov	UsbWfp, 1
	call	DisableIrq1

	ret
write_cmd_proc	endp

write_cmd_proc2	proc	near
        test    TrapByStatus, TRAPBY60W
	jnz	short Retrive60W
	test	TrapByStatus, TRAPBY64R
	jnz	short Retrive64R
	test	TrapByStatus, TRAPBY60R
	jnz	short Retrive60R
	mov	UsbWfp, 0
	stc
	ret

Retrive60R:
	call	SmiIn60
	jmp	short SetTrap_64R

Retrive64R:
	call	SmiIn64

SetTrap_64R:
	call	SetTrapData
	mov	UsbWfp, 1
	jmp	short write_cmd_proc2_exit

Retrive60W:
	call	SmiOut60
	call	UsbKbdAction
	call	UsbMsAction
	mov	UsbWfp, 0
	call	EnableIrq1
write_cmd_proc2_exit:
	clc

	ret

write_cmd_proc2	endp

read_cmd_proc	proc	near
	mov	dx, offset read_cmd_proc2 SMI_OFFSET
	mov	KbcProc, dx
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
read_cmd_proc	endp

DisableIrq1	proc
	in	al, 21h
	IODELAY
	IODELAY
	or	al, 02
	out	21h, al
	IODELAY
	ret
DisableIrq1	endp

EnableIrq1	proc
	in	al, 21h
	IODELAY
	IODELAY
	and	al, not 02
	out	21h, al
	IODELAY
	ret
EnableIrq1	endp

read_cmd_proc2	proc	near
	test	TrapByStatus, TRAPBY64R
	jnz	short Out64_readcmd
	test	TrapByStatus, TRAPBY60R
	jnz	short Out60_readcmd
	mov	UsbWfp, 0
	test	TrapByStatus, TRAPBY64W ;debug
	jz	@f                      ;debug
        mov     al, 11h                 ;debug
        out     80h, al                 ;debug
@@:                                     ;debug
	stc
	ret

Out64_readcmd:
	call	SmiIn64
	mov	UsbWfp, 1
	jmp	short read_cmd_proc2_exit

Out60_readcmd:
	call	SmiIn60
	mov	UsbWfp, 0
	call	EnableIrq1

read_cmd_proc2_exit:
	call	SetTrapData
	clc
	ret
read_cmd_proc2	endp


UsbKbdAction	proc	near
	test	IoTrapData, 10h		;Bit 4
	jz	short EnabKbd
        call    Suspend_Kbd
	ret

EnabKbd:
        call    Resume_Kbd
	ret
UsbKbdAction	endp

UsbMsAction	proc	near
	ret
UsbMsAction	endp


disable_kbrd_proc	proc	near
	call	DisableUSBKbd
	mov	UsbWfp, 0
	ret
disable_kbrd_proc	endp



enable_kbrd_proc	proc	near
	call	EnableUSBKbd
	mov	UsbWfp, 0
	ret

enable_kbrd_proc	endp

DisableUsbKbd	proc	near
        call    Suspend_Kbd
	mov	UsbWfp, 0
	ret
DisableUsbKbd	endp

EnableUsbKbd	proc	near
        call    Resume_Kbd
	ret
EnableUsbKbd	endp


disable_mouse_proc	proc	near
        call     Suspend_Ms
	mov	UsbWfp, 0
	ret
disable_mouse_proc	endp

enable_mouse_proc	proc	near
        call    Resume_Ms           ;V300
enable_mouse_proc_exit:
	mov	UsbWfp, 0
	ret

enable_mouse_proc	endp

InitStatus8042	proc	near
	call	SmiIn64
	mov	al, IoTrapData
	and	al, not IBF_8042
	or	al, OBF_8042
	mov	Status8042, al
	ret
InitStatus8042	endp

led_cmd_proc	proc	near
	mov	dx, offset led_cmd_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret

led_cmd_proc	endp

;Input:
;	TrapByStatus
;	Status8042
led_cmd_proc2	proc	near

        test    TrapByStatus, TRAPBY64R
	jnz	short Do64rLed
        test    TrapByStatus, TRAPBY60R
	jnz	short Do60rLed
        test    TrapByStatus, TRAPBY60W
	jnz	short Retrive60WLed
	mov	UsbWfp, 0
	call	EnableIrq1
	stc
	ret

Retrive60WLed:
	call	GetTrapData
	call	SmiOut60
	call	UsbLedAction
	mov	UsbWfp, 0
	call	EnableIrq1
	jmp	short led_cmd_proc2_exit

Do60rLed:
	mov	IoTrapData, KB_ACK
	and	Status8042, not (IBF_8042+OBF_8042)
        jmp	short Mv2TrapData

Do64rLed:
	mov	al, Status8042
	or	al, 01			;debug heavy
	mov	IoTrapData, al

Mv2TrapData:
	call	SetTrapData
	mov	UsbWfp, 1

led_cmd_proc2_exit:
	clc
	ret

led_cmd_proc2	endp


UsbLedAction	proc	near
	mov	al, IoTrapData
	shl	al, 4
	mov	bUsbLedCmd, al
	mov	bLedStatus, al

	call	Suspend_Tasks			;suspend all tasks
						;for LED updated
	call	Update_Led			;update keyboard LED

	call	Resume_Tasks			;resume all tasks
						;after LED updated

UsbLedAction_Exit:
	ret
UsbLedAction	endp

echo_cmd_proc	proc	near
	mov	dx, offset echo_cmd_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
echo_cmd_proc	endp

echo_cmd_proc2	proc	near
        test    TrapByStatus, TRAPBY64R
	jnz	short Do64rEcho
        test    TrapByStatus, TRAPBY60R
	jnz	short Do60rEcho
	mov	UsbWfp, 0
	call	EnableIrq1
	stc
	ret

Do60rEcho:
	mov	IoTrapData, ECHO_CMD
	and	Status8042, not (IBF_8042+OBF_8042)
	mov	UsbWfp, 0
	call	EnableIrq1
	jmp	short Mv2TrapDataEcho

Do64rEcho:
	call	SmiIn64
	or	IoTrapData, 01
	mov	UsbWfp, 1

Mv2TrapDataEcho:
	Call	SetTrapData
	clc
	ret
echo_cmd_proc2	endp

scan_cmd_proc	proc	near
	mov	dx, offset scan_cmd_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret

scan_cmd_proc	endp

;Input:
;	TrapByStatus
;	Status8042
scan_cmd_proc2	proc	near

        test    TrapByStatus, TRAPBY64R
	jnz	short Do64rScan
        test    TrapByStatus, TRAPBY60R
	jnz	short Do60rScan
        test    TrapByStatus, TRAPBY60W
	jnz	short Retrive60WScan
	mov	UsbWfp, 0
	call	EnableIrq1
	stc
	ret

Retrive60WScan:
	call	GetTrapData
	call	SmiOut60
	call	SelectScanTbl		;FICL first approach

FICL_1st:
	mov	UsbWfp, 0
	call	EnableIrq1
	jmp	short scan_cmd_proc2_exit

Do60rScan:
	mov	IoTrapData, KB_ACK
	and	Status8042, not (IBF_8042+OBF_8042)
	call	SetTrapData		;FICL first approach (temporary)
	jmp	short FICL_1st		;FICL first approach (temporary)
        jmp	short Mv2TrapDataScan

Do64rScan:
	mov	al, Status8042
	mov	IoTrapData, al

Mv2TrapDataScan:
	call	SetTrapData
	mov	UsbWfp, 1

scan_cmd_proc2_exit:
	clc
	ret

scan_cmd_proc2	endp

SelectScanTbl	proc	near
	ret
SelectScanTbl	endp

kb_rdid_proc	proc	near
	mov	dx, offset kb_rdid_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	Read60, 0
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
kb_rdid_proc	endp

kb_rdid_proc2	proc	near
	test	TrapByStatus, TRAPBY64R
	jnz	short Do64rKb_rdid
	test	TrapByStatus, TRAPBY60R
	jnz	short Do60rkb_rdid
	mov	UsbWfp, 0
	call	EnableIrq1
	stc
	ret

Do60rkb_rdid:
	cmp	Read60, 0
	jne	short Read60_2nd
	mov	IoTrapData, KB_ACK
	inc	Read60
	jmp     short kb_rdid_exit1

Read60_2nd:
	cmp	Read60, 1
	jne	short Read60_3rd
	mov	IoTrapData, KBDID_LO
	inc	Read60
	jmp     short kb_rdid_exit1

Read60_3rd:
	mov	IoTrapData, KBDID_HI
	and	Status8042, not (IBF_8042+OBF_8042)
	mov	Read60, 0
	mov	UsbWfp, 0
	call	EnableIrq1
	jmp     short kb_rdid_exit2

Do64rkb_rdid:
	mov	al, Status8042
	mov	IoTrapData, al

kb_rdid_exit1:
	mov	UsbWfp, 1

kb_rdid_exit2:
	call	SetTrapData

	clc
	ret

kb_rdid_proc2	endp

set_typma_proc	proc	near
	mov	dx, offset set_typma_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
set_typma_proc	endp

set_typma_proc2	proc	near
	test	TrapByStatus, TRAPBY64R
	jnz	short Do64rTypma
	test	TrapByStatus, TRAPBY60R
	jnz	short Do60rTypma
	mov	UsbWfp, 0
	call	EnableIrq1
	stc
	ret

Do60rTypma:
	mov	IoTrapData, KB_ACK
	and	Status8042, not (IBF_8042+OBF_8042)
	mov	UsbWfp, 0
	call	EnableIrq1
	jmp     short set_typma_proc2_exit

Do64rTypma:
	mov	al, Status8042
	mov	IoTrapData, al
	mov	UsbWfp, 1

set_typma_proc2_exit:
	call	SetTrapData
	clc
	ret

set_typma_proc2	endp

set_atypma_proc	proc	near
	mov	dx, offset set_atypma_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
set_atypma_proc	endp

set_atypma_proc2	proc
	call	set_typma_proc2
	ret
set_atypma_proc2	endp

kbd_reset_proc	proc	near
	mov	dx, offset kbd_reset_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	Read60, 0
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
kbd_reset_proc	endp

kbd_reset_proc2	proc	near
	test	TrapByStatus, TRAPBY64R
	jnz	short Do64rKbd_reset
	test	TrapByStatus, TRAPBY60R
	jnz	short Do60rkbd_reset
	mov	UsbWfp, 0
	call	EnableIrq1
	stc
	ret

Do60rkbd_reset:
	cmp	Read60, 0
	jne	short kbd_result
	mov	IoTrapData, KB_ACK
	inc	Read60
	jmp     short kbd_reset_exit1

kbd_result:
	mov	IoTrapData, TEST_OK
	and	Status8042, not (IBF_8042+OBF_8042)
	mov	Read60, 0
	mov	UsbWfp, 0
	call	EnableIrq1
	jmp     short kbd_reset_exit2


Do64rkbd_reset:
	mov	al, Status8042
	mov	IoTrapData, al

kbd_reset_exit1:
	mov	UsbWfp, 1

kbd_reset_exit2:
	call	SetTrapData

	clc
	ret

kbd_reset_proc2	endp

set_amkbk_proc	proc	near
	mov	dx, offset set_amkbk_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
set_amkbk_proc	endp

set_amkbk_proc2	proc	near
	call	set_typma_proc2		;FICL first approach
	ret
set_amkbk_proc2	endp

set_amk_proc	proc	near
	mov	dx, offset set_amk_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
set_amk_proc	endp

set_amk_proc2	proc	near
	call	set_typma_proc2		;FICL first approach
	ret
set_amk_proc2	endp

set_atymkbk_proc	proc	near
	mov	dx, offset set_atymkbk_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
set_atymkbk_proc	endp

set_atymkbk_proc2	proc	near
	call	set_typma_proc2		;FICL first approach
	ret
set_atymkbk_proc2	endp

set_aty_proc	proc	near
	mov	dx, offset set_aty_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
set_aty_proc	endp

set_aty_proc2	proc	near
	call	set_typma_proc2		;FICL first approach
	ret
set_aty_proc2	endp

set_mkbk_proc	proc	near
	mov	dx, offset set_mkbk_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
set_mkbk_proc	endp

set_mkbk_proc2	proc	near
	call	set_typma_proc2		;FICL first approach
	ret
set_mkbk_proc2	endp

set_mk_proc	proc	near
	mov	dx, offset set_mk_proc2 SMI_OFFSET
	mov	KbcProc, dx
	call	InitStatus8042
	mov	UsbWfp, 1
	call	DisableIrq1
	ret
set_mk_proc	endp

set_mk_proc2	proc	near
	call	set_typma_proc2		;FICL first approach
	ret
set_mk_proc2	endp

;[]========================================================================[]
;Procedure:	Suspend_Kbd
;Function:	Suspend all tasks for kbd int. transfer
;Input:		none
;Output:	none
;[]========================================================================[]

Suspend_Kbd      proc
        pusha
	mov	bx, offset wTaskLink

Suspend_kbd_Loop:
	cmp	word ptr [bx].wTkLink, 0FFFFh
	je	short Suspend_Kbd_Exit

	mov	bx, [bx].wTkLink
	test	[bx].bTkDevStatus, TK_KBD       ;Is it for a kbd?
        jz      short Suspend_Kbd_Loop
        mov     si, [bx].wTkQueueHead
        or      [si].dQhElement, T
	jmp	short Suspend_Kbd_Loop

Suspend_Kbd_Exit:
	popa
	ret

Suspend_Kbd     endp

;[]========================================================================[]
;Procedure:	Resume_Kbd
;Function:	Resume all tasks for kbd int. transfer
;Input:		none
;Output:	none
;[]========================================================================[]

Resume_Kbd      proc
        pusha
	mov	bx, offset wTaskLink

Resume_kbd_Loop:
	cmp	word ptr [bx].wTkLink, 0FFFFh
	je	short Resume_Kbd_Exit

	mov	bx, [bx].wTkLink
	test	[bx].bTkDevStatus, TK_KBD       ;Is it for a kbd?
        jz      short Resume_Kbd_Loop
        mov     si, [bx].wTkQueueHead
        and     [si].dQhElement, not T
	jmp	short Resume_Kbd_Loop

Resume_Kbd_Exit:
	popa
	ret

Resume_Kbd      endp

;[]========================================================================[]
;Procedure:	Suspend_Ms
;Function:	Suspend all tasks for mouse int. transfer
;Input:		none
;Output:	none
;[]========================================================================[]

Suspend_Ms      proc
        pusha
	mov	bx, offset wTaskLink

Suspend_Ms_Loop:
	cmp	word ptr [bx].wTkLink, 0FFFFh
	je	short Suspend_Ms_Exit

	mov	bx, [bx].wTkLink
	test	[bx].bTkDevStatus, TK_MS       ;Is it for a mouse?
        jz      short Suspend_Ms_Loop
        mov     si, [bx].wTkQueueHead
        or      [si].dQhElement, T
	jmp	short Suspend_Ms_Loop

Suspend_Ms_Exit:
	popa
	ret

Suspend_Ms     endp

;[]========================================================================[]
;Procedure:	Resume_Ms
;Function:	Resume all tasks for mouse int. transfer
;Input:		none
;Output:	none
;[]========================================================================[]

Resume_Ms      proc
        pusha
	mov	bx, offset wTaskLink

Resume_Ms_Loop:
	cmp	word ptr [bx].wTkLink, 0FFFFh
	je	short Resume_Ms_Exit

	mov	bx, [bx].wTkLink
	test	[bx].bTkDevStatus, TK_MS       ;Is it for a mouse?
        jz      short Resume_Ms_Loop
        mov     si, [bx].wTkQueueHead
        and     [si].dQhElement, not T
	jmp	short Resume_Ms_Loop

Resume_Ms_Exit:
	popa
	ret

Resume_Ms      endp


KbcCmdTbl  label   byte
	db   READ_CMD
	dw   offset read_cmd_proc SMI_OFFSET
        db   WRITE_CMD
        dw   offset write_cmd_proc SMI_OFFSET
        db   DISABLE_MOUSE
        dw   offset disable_mouse_proc SMI_OFFSET
        db   ENABLE_MOUSE
        dw   offset enable_mouse_proc SMI_OFFSET
        db   DISABLE_KBRD
        dw   offset disable_kbrd_proc SMI_OFFSET
        db   ENABLE_KBRD
        dw   offset enable_kbrd_proc SMI_OFFSET
KBC_CMD_CNT    EQU    ($-KbcCmdTbl)/3

KbdCmdTbl  label   byte
        db   LED_CMD
        dw   offset led_cmd_proc SMI_OFFSET
	db   ECHO_CMD
	dw   offset echo_cmd_proc SMI_OFFSET
	db   SCAN_CMD
	dw   offset scan_cmd_proc SMI_OFFSET
	db   KB_RDID
	dw   offset kb_rdid_proc SMI_OFFSET
	db   SET_TYPMA
	dw   offset set_typma_proc SMI_OFFSET
	db   SET_ATYPMA
	dw   offset set_atypma_proc SMI_OFFSET
	db   SET_AMKBK
 	dw   offset set_amkbk_proc SMI_OFFSET
	db   SET_AMK
 	dw   offset set_amk_proc SMI_OFFSET
 	db   SET_ATYMKBK
 	dw   offset set_atymkbk_proc SMI_OFFSET
	db   SET_TY
 	dw   offset set_aty_proc SMI_OFFSET
	db   SET_MKBK
 	dw   offset set_mkbk_proc SMI_OFFSET
	db   SET_MK
 	dw   offset set_mk_proc SMI_OFFSET
	db   KBD_RESET
 	dw   offset kbd_reset_proc SMI_OFFSET
KBD_CMD_CNT    EQU    ($-KbdCmdTbl)/3
endif	;IOTRAP_SUPPORT
;R22 - end

	assume	ds:nothing

ENDIF	;COMPILE_FOR_USBBIOS EQ 3
;R04ENDIF	;USB_SUPPORT
