;	[]===========================================================[]
;
;	NOTICE: THIS PROGRAM BELONGS TO AWARD SOFTWARE INTERNATIONAL(R)
;	        INC. IT IS CONSIDERED A TRADE SECRET AND IS NOT TO BE 	
;	        DIVULGED OR USED BY PARTIES WHO HAVE NOT RECEIVED	
;	        WRITTEN AUTHORIZATION FROM THE OWNER.
;
; 	[]===========================================================[]
;
;----------------------------------------------------------------------------
;Rev	Date	 Name	Description
;--------------------------2A5II---------------------------------------------
;R03	05/22/97 AVN	If CPU in 75Mhz and VGA maybe have snow in screen, so
;			slow down VGA performance by SiS suggest.
;R02	04/24/97 LRY	Support onboard VGA IDSEL not only at AD31
;R01	03/24/97 LRY	Fixed SDRAM shared memory screen blank
;R00	01/31/97 LRY	Initial revision

.386p
		PAGE	56,132
		TITLE	VUMA.ASM - VUMA Real/16 bit Interface
.XLIST
		INCLUDE	BIOS.CFG
		INCLUDE	COMMON.EQU
		INCLUDE	COMMON.MAC
		INCLUDE	POST.MAC
		INCLUDE	BSETUP.INC
		INCLUDE	CMOS.EQU


G_RAM		SEGMENT	USE16 AT 0

		ORG	04H*4
		INCLUDE	SEG_0.INC

		ORG	400H
		INCLUDE	G_RAM.INC

G_RAM		ENDS

.LIST

DGROUP		GROUP	FCODE
FCODE		SEGMENT	USE16 DWORD PUBLIC 'CODE'
		ASSUME	CS:DGROUP



ifdef	VUMA

		public	Shared_Mem_Addr
		public	Shared_Mem_Size
		public	Shared_Mem_RAS
		public	Shared_Mem_Type
Shared_Mem_Addr	dw	0		;(64k unit)
Shared_Mem_Size	dw	0		;(64K unit)
Shared_Mem_RAS	dw	01h		;(bit 0,1...= RAS 0,1)
Shared_Mem_Type	db	1		;(FP=1 , EDO=2, SDRAM=3)


;[]==================================================================[]
;
; Procedure Name: VUMA_Service
;
;	VUMA service routines entry point.
;
; Saves: all
;
; Input:
;	ax=0F401h
;       bl=00h : Report VUMA Core Capability
;	  =01h : Request VUMA Main Memory capabilities
;	  =02h : Set (Request) VUMA Main Memory for Next Boot
;	  =03h : Get VUMA Main Memory Size for Next Boot for a Device
;	  =04h : Get Memory Sixe for Next Boot for all VUMA Memory 
;	  =05h : Get Current Memory Size for a Debice or for all VUMA Memory
;	  =06h : Return Memory Speed/Type
;	  =07h : Enable/Disable Memory
;	  =08h : Set (Request) VUMA Aux Memory
;	  =09h : Get VUMA Memory Size
;	cx,dx,si,di dependent on the function requested
;
; Output:
;	al  =0F4h : Function is supported 
;	al !=0F4h : Function is NOT supported 
;	ah  =000h : Function call successful
;	    =001h :  Unknown PFA. PFA does not match devices in system
;	    =002h :  Imvalid Input Argument
;	    =003h :  Too many banks request
;	    =004h :  Requested bank(s) not supported
;	    =0FFh :  Other unknown error
;
;	bx,cx,dx,si,di depend the function requested
;
; [Note]:
;	This is the high level dispatch routine for VUMA interface.
;[]==================================================================[]
		Public	VUMA_Service
VUMA_Service	Proc	Far

		pushf
		cli

		push	ds			;save callers ds

	;2. invoke the corresponding function

		xor	si,si			;indicates real mode call
		call	Corresponding_VUMA_Func

		pop	ds			;restore callers ds
		jc	short VUMA_Fail
VUMA_Succ:
		popf
		clc
		retf	2
VUMA_Fail:
		popf
		stc
		retf	2

		ret

VUMA_Service	Endp

;--------------------------------------------------------------------------
		ALIGN	4
Corresponding_VUMA_Func	Proc	Near

		cmp	bl,NO_OF_VUMA_CALL
		jbe	short @F
Function_Not_Supp:				;by SiS programming guide
		mov	ax,00FFh
		stc				;function out of range
		ret
	@@:

		
		movzx	si,bl
		shl	si,1
		jmp	cs:VUMA_FUNC_TBL[si]


Corresponding_VUMA_Func	Endp

		ALIGN	4
VUMA_FUNC_TBL	DW	offset DGROUP:VUMA_FUNC00
		DW	offset DGROUP:VUMA_FUNC01
		DW	offset DGROUP:VUMA_FUNC02
		DW	offset DGROUP:VUMA_FUNC03
		DW	offset DGROUP:VUMA_FUNC04
		DW	offset DGROUP:VUMA_FUNC05
		DW	offset DGROUP:VUMA_FUNC06
NO_OF_VUMA_CALL  EQU ($-VUMA_FUNC_TBL)/2-1

;[]==================================================================[]
;Procedure:	VUMA_FUNC00
;
;Function :	VUMA function 0 - Report VUMA Core Capability
;
;Saves    :	all
;
;Input    :	AX = 0F401h
;		BL = 00h
;
;Output   :	if function successful:
;			carry = 0
;		ax = Status
;		bl = Major BIOS revision = 01h
;		bh = Minor BIOS revision = 00h
;		cx = Bank numbers wih memory. Bit set if has memory
;		     Bit 0 = bank 0
;		     Bit 1 = bank 1
;		     etc
;		dx[3:0] = Host capibilities
;		     	  0 = No special features
;		     	  Bit 0 = 1->Controller supports non-cacheable regions
;		     	  Bit 1 = 1->Controller supports write-thru cache
;				     regions
;		dx[4]   = Core logic supports snooping
;		     	  0 = snooping is NOT supported
;		     	  1 = snooping is supported
;		si = System memory cache size granularity in 1K blocks, Minimum
;		     block size in system memory that can have L2 cache turned
;		     on or off
;		     0 = Not defined
;[]==================================================================[]

VUMA_FUNC00	Proc	Near

VUMA_FUNC00	Endp

;[]==================================================================[]
;[]==================================================================[]
VUMA_FUNC01	Proc	Near

VUMA_FUNC01	Endp


;[]==================================================================[]
;[]==================================================================[]
VUMA_FUNC02	Proc	Near

VUMA_FUNC02	Endp


;[]==================================================================[]
;[]==================================================================[]
VUMA_FUNC03	Proc	Near

VUMA_FUNC03	Endp


;[]==================================================================[]
;[]==================================================================[]
VUMA_FUNC04	Proc	Near
		jmp	short Function_Not_Supp
VUMA_FUNC04	Endp

;[]==================================================================[]
;[]==================================================================[]
VUMA_FUNC05	Proc	Near
		
;R02		cmp	cx,00A0h
		cmp	cx,ONBOARD_PCI_VGA shl 3	;R02
		jne	short Function_Not_Supp
Ok_5596_Builtin:
		mov	bh,01h
		mov	cx,Shared_Mem_RAS
		mov	dx,Shared_Mem_Size
		or	dx,dx			;R01
		jnz	short @F		;R01
		xor	bh,bh			;R01
@@:						;R01
		jmp	Common_Func_5_6

VUMA_FUNC05	Endp

;[]==================================================================[]
;[]==================================================================[]
VUMA_FUNC06	Proc	Near

		or	cx,cx
		jnz	short CX_not_Zero
		mov	ax,0200h
		stc
		ret
CX_not_Zero:
		mov	ax,000Fh
Check_CX_Loop:
		rol	cx,1
		jnc	short Not_Found_One
		or	ah,ah
		jz	short First_One
		mov	ax,0300h
		stc
		ret
First_One:
		mov	ah,al
Not_Found_One:
		or	al,al
		jz	short RAS_In_AH
		dec	ax
		jmp	short Check_CX_Loop
RAS_In_AH:
		cmp	ah,5
		jbe	short RAS_Valid
		mov	ax,0400h
		stc
		ret
RAS_Valid:
		xor	ebx,ebx
		test	cx,Shared_Mem_RAS
		jz	short Not_VUMA_mem
		or	bh,40h		;VUMA memory or not
Not_VUMA_mem:
		mov	cl,ah
		shr	cl,1
		add	cl,60h
		
		extrn	Get_CfgSpace_Byte:near
		call	Get_CfgSpace_Byte
		test	al,10h
		jz	short @F
		or	bl,40h		;32bit
@@:
		and	al,0Fh
		shl	eax,28
		or	ebx,eax		;scramble table
	push	bx
		shr	eax,28
	push	ax

		mov	cx,57h			;R01
		call	Get_CfgSpace_Byte	;R01
		and	al,10h			;R01
		mov	ah,al			;R01
		mov	cl,51h			;R01
;R01		mov	cx,51h
		call	Get_CfgSpace_Byte
		and	al,04h
		shr	al,2
		xor	ecx,ecx
		xor	dx,dx
		xor	al,01h
		or	ch,al		;linear
;R01		or	cl,090h		;burst & CAS latency
		or	cl,080h		;burst			;R01
		or	cl,ah		;CAS latency		;R01
		or	cl,Shared_Mem_Type

;R01		cmp	Shared_Mem_Type,0C0h
		cmp	Shared_Mem_Type,3	;R01
		mov	bx,offset RAS_CAS_SDRAM
		jae	short SDRAM_bank
		mov	bx,offset RAS_CAS_tbl
SDRAM_bank:
	pop	ax
		xlat	cs:RAS_CAS_tbl
	pop	bx
		shl	ax,4
		shr	al,4
		mov	dx,ax		;save RAS CAS
		shl	eax,16
		or	ecx,eax

		add	dl,dh
		sub	dl,13
		test	bl,40h		;32bit ?
		jz	short @F	;yes, jump
		dec	dx
@@:
	push	cx

		mov	cl,dl
		mov	dx,1
		shl	dx,cl
		test	bh,40h		;is it VUMA bank ?
		jz	short Device_freq
		sub	dx,Shared_Mem_Size
Device_freq:
		mov	cx,860h
		call	Get_CfgSpace_Byte
	push	ax
;R01		mov	al,3Fh
		mov	al,4Fh NMI_OFF		;R01
		out	70h,al
		newiodelay
		in	al,71h
		and	al,1Fh
		or	bl,al
	pop	ax
		out	70h,al

	pop	cx
;R03 - start
		and	ecx,not 0000001000000000b		;Set bit 9 to Lo
		cmp	byte ptr CPU_INT_CLOCK[bp],150		;150Mhz ?
		jne	short Cpu_Clk_not_75Mhz
		or	ecx,0000001000000000b			;Set bit 9 to Hi
								;Low performance in VGA
Cpu_Clk_not_75Mhz:
;R03 - end
Bank_null:

Common_Func_5_6:
		mov	ax,00F4h		; function successful
		mov	si,Shared_Mem_Addr
		xor	di,di

		clc
		ret


		;     ROW  COLUMN	
		;	 \/
RAS_CAS_tbl	db	099h
		db	0AAh
		db	0BBh
		db	0CCh
		db	0C8h
		db	0C9h
		db	0CAh
		db	0CBh
		db	0A9h
		db	0B9h
		db	0BAh

RAS_CAS_SDRAM	db	0C8h
		db	0E8h
		db	0E8h
		db	0F8h
		db	0C9h
		db	0E9h
		db	0E9h
		db	0F9h
		db	0CAh
		db	0EAh
		db	0EAh
		db	0FAh
		db	0D8h

VUMA_FUNC06	Endp

endif	;VUMA

FCODE		ENDS
		END
